precision highp float;

#define HALF_INDEX 3
#define MIDDLE_INDEX HALF_INDEX + 1
#define MIDDLE_COUNT MIDDLE_INDEX + 1

varying vec2 uTexCoord;

uniform sampler2D uTexture;
uniform sampler2D uVecField;
uniform sampler2D uMask;

uniform float uStrength;
uniform float uDiffusion;

uniform float uWeights[MIDDLE_COUNT];
uniform vec2 uInverseResolution;

vec2 sampleVectorField(vec2 texCoord) {
    vec2 v = texture2D(uVecField, texCoord).rg;
    v = v * uStrength * uInverseResolution;
    return v;
}

void main() {
    vec4 oriCol = texture2D(uTexture, uTexCoord);
    vec4 col = oriCol * uWeights[MIDDLE_INDEX];
    vec2 oV = sampleVectorField(uTexCoord);

    vec4 mask = texture2D(uMask, uTexCoord);
    float mm = (mask.r + mask.g + mask.b) * 0.3333; // mask mixer

    vec2 v = oV;
    vec2 texCoord = uTexCoord + v;
    for (int i = 0; i < MIDDLE_INDEX; i++) {
        col += texture2D(uTexture, texCoord) * uWeights[HALF_INDEX - i];
        vec2 v2 = sampleVectorField(texCoord);
        v = sign(dot(v, v2)) * v2;
        texCoord += v;
    }

    v = -oV;
    texCoord = uTexCoord + v;
    for (int i = 0; i < MIDDLE_INDEX; i++) {
        col += texture2D(uTexture, texCoord) * uWeights[HALF_INDEX - i];
        vec2 v2 = sampleVectorField(texCoord);
        v = sign(dot(v, v2)) * v2;
        texCoord += v;
    }

    gl_FragColor = mix(oriCol, col, pow(mm, uDiffusion));
}
