precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform highp vec4 color;
uniform highp float hueOpacity;
uniform highp float exposure;

vec3 hardlight(vec4 base, vec4 overlay) {
    float ra;
    if (2.0 * overlay.r < overlay.a) {
        ra = 2.0 * overlay.r * base.r + overlay.r * (1.0 - base.a) + base.r * (1.0 - overlay.a);
    } else {
        ra = overlay.a * base.a - 2.0 * (base.a - base.r) * (overlay.a - overlay.r) + overlay.r * (1.0 - base.a) + base.r * (1.0 - overlay.a);
    }

    float ga;
    if (2.0 * overlay.g < overlay.a) {
        ga = 2.0 * overlay.g * base.g + overlay.g * (1.0 - base.a) + base.g * (1.0 - overlay.a);
    } else {
        ga = overlay.a * base.a - 2.0 * (base.a - base.g) * (overlay.a - overlay.g) + overlay.g * (1.0 - base.a) + base.g * (1.0 - overlay.a);
    }

    float ba;
    if (2.0 * overlay.b < overlay.a) {
        ba = 2.0 * overlay.b * base.b + overlay.b * (1.0 - base.a) + base.b * (1.0 - overlay.a);
    } else {
        ba = overlay.a * base.a - 2.0 * (base.a - base.b) * (overlay.a - overlay.b) + overlay.b * (1.0 - base.a) + base.b * (1.0 - overlay.a);
    }

    return vec3(ra, ga, ba);
}

vec4 blend(vec4 base, vec4 overlay) {
    return vec4(hardlight(overlay, base), base.a + overlay.a - base.a * overlay.a);
}


vec4 blend1(vec4 base, vec4 overlay) {
    return base + overlay - base * overlay.a;
}

const vec3 lumWeights = vec3(0.30, 0.59, 0.11);

vec3 applyExposure(vec3 color, float exposure) {
    return clamp(color * pow(2.0, exposure), 0.0, 1.0);
}

void main()
{

    vec4 srcColor = texture2D(inputImageTexture, textureCoordinate);
    vec4 blurColor = texture2D(inputImageTexture2, textureCoordinate);
    float srcAlpha = srcColor.a;
    vec4 unmulSrcColor = vec4(srcColor.rgb / (srcAlpha + step(srcAlpha, 0.0)), 1.0);

    float prgs = smoothstep(-0.1, 0.1, 0.5 - dot(unmulSrcColor.rgb, lumWeights));

    unmulSrcColor = vec4(applyExposure(unmulSrcColor.rgb, exposure), 1.0);

    vec4 aaa = color * (0.1 + 0.5 * prgs) * (1.0 - blurColor.a);
    vec4 bbb = color * hueOpacity;
    gl_FragColor = blend(blend1(unmulSrcColor, aaa), bbb) * srcAlpha;
}