#define PI 3.1415926
#define PI_2 1.5707963

precision highp float;
precision highp int;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform vec2 resolution;

// 瘦手臂
uniform float slim_arm_param[26];
uniform float slim_arm_scale[4];

#func_data2offset
#func_offset2data

vec3 get_line_param(vec2 point1, vec2 point2) {
    vec3 res;
    res.x = point1.y - point2.y;
    res.y = point2.x - point1.x;
    res.z = point1.x * point2.y - point2.x * point1.y;
    return res;
}

vec2 get_cross_point(vec4 line1, vec4 line2) {
    vec3 param1 = get_line_param(line1.xy, line1.zw);
    vec3 param2 = get_line_param(line2.xy, line2.zw);
    vec2 res = vec2(-1.0, -1.0);
    float tmp_d = param1.x * param2.y - param2.x * param1.y;
    if (tmp_d != 0.)
    {
        res.x = (param1.y * param2.z - param2.y * param1.z) / tmp_d;
        res.y = (param1.z * param2.x - param2.z * param1.x) / tmp_d;
    }
    return res;
}

vec2 get_vertical_cross_point_by_vec(vec2 point, vec2 vec, vec2 target_point) {
    vec2 res;
    vec2 other_point = point + vec;

    vec3 ABC = get_line_param(point, other_point);
    float A = ABC.x;
    float B = ABC.y;
    float C = ABC.z;
    float A2 = A * A;
    float B2 = B * B;
    float A2plusB2 = A2 + B2;

    if (A2plusB2 != 0.)
    {
        float AB = A * B;
        float AC = A * C;
        float BC = B * C;
        float inv_A2plusB2 = 1.0 / A2plusB2;
        res.x = (B2 * target_point.x - AB * target_point.y - AC) * inv_A2plusB2;
        res.y = (A2 * target_point.y - AB * target_point.x - BC) * inv_A2plusB2;
        return res;
    }
    else
    {
        return res;
    }
}

float linearstep(float t1, float t2, float x) {
    x = clamp((x - t1) / (t2 - t1), 0.0, 1.0);
    return x;
}

vec2 get_orthogonal_vec(vec2 vector) {
    vec2 res;
    res.x = vector.y;
    res.y = - vector.x;
    return res;
}

int is_in_angle(vec2 query_point, vec2 center_point, vec2 point1, vec2 point2, int mode) {
    vec2 line1_vec = point1 - center_point;
    vec2 line2_vec = point2 - center_point;
    if (mode != 3 && mode != 2)// 11 10
    {
        line1_vec = - line1_vec;
    }
    if (mode != 3 && mode != 1)// 11 01
    {
        line2_vec = - line2_vec;
    }
    vec2 point_vec = query_point - center_point;
    if (point_vec.x == 0. && point_vec.y == 0.)
    {
        return 0;
    }
    line1_vec = normalize(line1_vec);
    line2_vec = normalize(line2_vec);
    point_vec = normalize(point_vec);

    float angle1 = acos(dot(line1_vec, line2_vec));
    float angle2 = acos(dot(line1_vec, point_vec));
    float angle3 = acos(dot(point_vec, line2_vec));
    //    cross_product_vec2(line1_vec, point_vec) //    if(max(angle2, angle3) > angle1)
    if (angle2 < angle1 && angle3 < angle1 && dot(point_vec, line1_vec + line2_vec) > 0.)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

int is_in_box(vec4 box_params, vec2 point) {
    return (point.x >= box_params.x && point.x <= box_params.y && point.y >= box_params.z && point.y <= box_params.w) ? 1 : 0;
}

vec4 get_box_param(vec2 point1, vec2 point2) {
    vec4 box_params;
    // x_min
    box_params.x = min(point1.x, point2.x);
    box_params.y = max(point1.x, point2.x);
    box_params.z = min(point1.y, point2.y);
    box_params.w = max(point1.y, point2.y);
    if (box_params.y - box_params.x < 1.0) {
        box_params.x = box_params.x - 1.0;
        box_params.y = box_params.y + 1.0;
    }
    if (box_params.w - box_params.z < 1.0) {
        box_params.z = box_params.z - 1.0;
        box_params.w = box_params.w + 1.0;
    }
    return box_params;
}

float angel_between_vec(vec2 vec_1, vec2 vec_2) {
    return acos(dot(normalize(vec_1), normalize(vec_2)));
}

vec2 get_inner_contour_point(vec2 circle_vec, vec2 contour_point, float angle, float r, float d) {
    float tmp_length;
    if (abs(angle) > 1e-10)
    {
        tmp_length = min(r / sin(angle), d / cos(angle));
    }
    else
    {
        return vec2(-1.0, -1.0);
    }
    return contour_point + tmp_length * circle_vec;
}

float smoothtop(float x) {
    return (4.0 * (x / 2.0) * (1.0 - (x / 2.0)));// ^k=1
}

float smoothtop2(float x) {
    return x * x * (1.0 - x) + x;
}

vec2 slim_arm_process(vec2 coor, vec2 point1, vec2 point2, vec2 point3, float r, vec2 ori_part_ratio, int mode, float intensity_1, float intensity_2) {
    if (point1.x <= 0.0 || point2.x <= 0.0) {
        return vec2(0.);
    }
//if ((point1.x <= 0.0 && point1.y <= 0.0 ) || (point2.x <= 0.0 && point2.y <= 0.0)) {
//return vec2(0.);
//}

    vec2 ex_point1 = mix(point2, point1, 1.14);
    vec2 ex_point3 = mix(point2, point3, 1.14);

    float offset_x_ratio = 1.0, dis1, dis2, angle;//smooth_ratio1 = 0.0;
    int is_in_box1 = 0, is_in_box2 = 0, is_in_angle_region;
    vec2 end_point, anchor_point = vec2(0.);
    vec2 tmp_center_line_vec1, tmp_center_line_vec2, tmp_cross_point1, tmp_cross_point2, contour_point, circle_vec, corner_point;


    tmp_center_line_vec1 = point2 - ex_point1;
    float arm_center_ratio = 0.25;
    float sp_default_ratio = 0.5;
    tmp_cross_point1 = get_vertical_cross_point_by_vec(point1, tmp_center_line_vec1, coor);
    if (tmp_cross_point1.x == -1.) {
        return vec2(0.);
    }
    is_in_box1 = is_in_box(get_box_param(ex_point1, point2), tmp_cross_point1);
    dis1 = distance(tmp_cross_point1, coor);

    vec2 tmp_orthogonal_vec = get_orthogonal_vec(tmp_center_line_vec1);

    vec2 cross_point_in_corner1, cross_point_in_corner2;
    float length_corner = 0.0;

    int inner_or_outer;
    int in_corner = 0, in_circle = 0, in_other_area = 0;
    int one_or_two = 0;

    if (point3.x > 0.) {
        tmp_center_line_vec2 = ex_point3 - point2;
        tmp_cross_point2 = get_vertical_cross_point_by_vec(point3, tmp_center_line_vec2, coor);

        is_in_box2 = is_in_box(get_box_param(point2, ex_point3), tmp_cross_point2);
        dis2 = distance(tmp_cross_point2, coor);
        contour_point = point2;
        is_in_angle_region = is_in_angle(coor, contour_point, point1, point3, 3);


        vec2 test_inner_point = point1 + 0.5 * normalize(tmp_orthogonal_vec);
        if (is_in_angle(test_inner_point, contour_point, point1, point3, 3) == is_in_angle_region) {
            inner_or_outer = 0;
        } else {
            inner_or_outer = 1;
        }

        angle = (PI - angel_between_vec(tmp_center_line_vec1, tmp_center_line_vec2)) * 0.5;

        circle_vec = normalize(normalize(tmp_center_line_vec2) - normalize(tmp_center_line_vec1));
        float max_arm_distance = max(distance(point2, ex_point1), distance(point2, ex_point3));
        corner_point = get_inner_contour_point(circle_vec, contour_point, angle, r, max_arm_distance);
        float dis_corner = distance(contour_point, corner_point) * sin(angle);
        length_corner = distance(contour_point, corner_point) * cos(angle);

        cross_point_in_corner1 = get_cross_point(vec4(corner_point, coor), vec4(contour_point, ex_point1));
        cross_point_in_corner2 = get_cross_point(vec4(corner_point, coor), vec4(contour_point, ex_point3));

        int is_in_inner_box1 = is_in_box(get_box_param(ex_point1, point2), cross_point_in_corner1);
        int is_in_inner_box2 = is_in_box(get_box_param(ex_point3, point2), cross_point_in_corner2);

        //        if (distance(coor, corner_point) < 10.) {
        //            return -coor + resolution * 0.5;
        //        }
        if (is_in_angle_region == 1) {
            if (is_in_inner_box1 == 1 && dis1 < dis_corner && distance(cross_point_in_corner1, contour_point) < length_corner) {
                in_corner = 1;
            }
            if (is_in_inner_box2 == 1 && dis2 < dis_corner && distance(cross_point_in_corner2, contour_point) < length_corner) {
                in_corner = 2;
            }
        }
    }
    else
    {
        if (dot(coor - tmp_cross_point1, tmp_orthogonal_vec) > 0.)
        {
            inner_or_outer = 0;
        }
        else
        {
            inner_or_outer = 1;
        }
    }

    if (mode == 1) {
        inner_or_outer = 1 - inner_or_outer;
    }

    // 直线
    if (point3.x <= 0. || abs(angle - PI * 0.5) < 1e-4) {
        vec2 start_point, end_point;
        if (point3.x <= 0.) {
            start_point = ex_point1;
            end_point = normalize(point2 - point1) * r + point2;// mix(point1, point2, 1.135);;
        } else {
            start_point = ex_point1;
            end_point = ex_point3;
        }

        vec2 cross_point = tmp_cross_point1;
        float dis = dis1;
        int is_in_box = is_in_box(get_box_param(start_point, end_point), cross_point);
        if (is_in_box == 1 && dis < r) {
            one_or_two = 1;

            offset_x_ratio = dis / r;
            anchor_point = cross_point;

            vec2 center_point = mix(start_point, end_point, 0.5);
            float maxr = distance(start_point, center_point);
            float minr = min(r, maxr);
            float aaa = 1.0 - clamp((maxr - distance(anchor_point, center_point)) / minr, 0., 1.);
            offset_x_ratio = mix(offset_x_ratio, 1.0, smoothstep(0., 1., aaa));
        }
    }
    else
    {
        vec2 vertical_vec = get_orthogonal_vec(corner_point - contour_point);
        float sign_1 = dot(vertical_vec, ex_point1 - contour_point);
        float sign_2 = dot(vertical_vec, coor - contour_point);
        int side = sign_1 * sign_2 > 0.0 ? 1 : 2;

        int is_out_angle_region = angel_between_vec(coor - contour_point, circle_vec) > angle + PI_2 ? 1 : 0;

        //        if ((is_in_box1 == 1 && dis1 < r && side == 1 && param(4) > 0.1 && param(4) < 0.2)
        //        || (is_in_box2 == 1 && dis2 < r && side == 2 && param(4) > 0.2 && param(4) < 0.3)
        //        || (is_in_box1 == 0 && is_in_box2 == 0 && length(coor - contour_point) < r && dis1 < r && dis2 < r && is_out_angle_region == 1 && param(4) > 0.3 && param(4) < 0.4)
        //        || (in_corner > 0 && param(4) > 0.4 && param(4) < 0.5)
        //        ) {
        //            return -coor + resolution * 0.5;
        //        }

        if (in_corner > 0) {
            one_or_two = side;
        } else if ((is_in_box1 == 1 && dis1 < r && side == 1) || (is_in_box2 == 1 && dis2 < r && side == 2)) {
            in_other_area = side;
            one_or_two = side;
        } else if (is_in_box1 == 0 && is_in_box2 == 0 && length(coor - contour_point) < r && dis1 < r && dis2 < r && is_out_angle_region == 1) {
            in_circle = side;
            one_or_two = side;
        }

        if (one_or_two > 0) {
            if (in_corner > 0) { // 夹角
                vec2 cross_point = (in_corner == 2) ? cross_point_in_corner2 : cross_point_in_corner1;

                offset_x_ratio = distance(coor, cross_point) / distance(corner_point, cross_point);
                anchor_point = cross_point;

                //smooth_ratio1 = smoothtop(1.0 - length(contour_point - cross_point) / length_corner);

            } else if (in_circle > 0) { // 外圆
                offset_x_ratio = length(coor - contour_point) / r;
                anchor_point = contour_point;

                //float tmp_angle = abs(PI - angel_between_vec(circle_vec, coor - contour_point));
                //smooth_ratio1 = smoothtop(1.0 - tmp_angle / (PI_2 - angle));

            } else {
                vec2 cross_point = (one_or_two == 2) ? tmp_cross_point2 : tmp_cross_point1;
                float tmp_dis = (one_or_two == 2) ? dis2 : dis1;

                offset_x_ratio = tmp_dis / r;
                anchor_point = cross_point;
            }

            vec2 end_point = (one_or_two == 2) ? ex_point3 : ex_point1;
            float minr = min(r, distance(contour_point, end_point));
            float aaa = 1.0 - clamp(distance(anchor_point, end_point) / minr, 0., 1.);
            offset_x_ratio = mix(offset_x_ratio, 1.0, smoothstep(0., 1., aaa));
        }
    }

    offset_x_ratio = clamp(offset_x_ratio, 0., 1.);
    offset_x_ratio = smoothtop2(offset_x_ratio);

    float x_ratio_1 = intensity_1 * 0.6 * (1. - offset_x_ratio);
    float x_ratio_2 = intensity_2 * 0.6 * (1. - offset_x_ratio);

    float part_ratio = mix(ori_part_ratio.x, ori_part_ratio.y, float(inner_or_outer));
    float part_ratio_other = ori_part_ratio.x + ori_part_ratio.y - part_ratio;
    x_ratio_1 *= part_ratio;

    float x_ratio = x_ratio_1;

    if (point3.x > 0.) {
        float tttt = linearstep(PI_2 * 0.75, PI_2 * 0.25, angle);
        float part_ratio_2 = mix(part_ratio, part_ratio_other, tttt);
        x_ratio_2 *= part_ratio_2;

        //        if (one_or_two == 2) {
        //            x_ratio_2 = mix(x_ratio_2, x_ratio_1, smooth_ratio1);
        //        }

        if (one_or_two == 1) {
            float minr = min(r, distance(contour_point, ex_point1));
            float aaa = 1.0 - clamp(distance(anchor_point, contour_point) / minr, 0., 1.);
            x_ratio = mix(x_ratio_1, x_ratio_2, smoothstep(0., 1., aaa) * 0.5);
        } else if (one_or_two == 2) {
            float minr = min(r, distance(contour_point, ex_point3));
            float aaa = 1.0 - clamp(distance(anchor_point, contour_point) / minr, 0., 1.);
            x_ratio = mix(x_ratio_2, x_ratio_1, smoothstep(0., 1., aaa) * 0.5);
        }

        if (is_in_angle_region == 1) {
            x_ratio *= linearstep(0., PI_2 * 0.75, angle);
        }
    }

    vec2 uv = (1. + x_ratio) * (coor - anchor_point) + anchor_point;

    return uv - coor;
}

vec2 process(vec2 coor) {
    // 瘦手臂
    vec2 offset = vec2(0.0);

    vec4 scale = vec4(slim_arm_scale[0], slim_arm_scale[1], slim_arm_scale[2], slim_arm_scale[3]);
    int param_index = 0;
    if (scale.x != 0.0 || scale.y != 0.0) {
        vec2 p1 = vec2(slim_arm_param[param_index + 0],
        slim_arm_param[param_index + 1]);
        vec2 p2 = vec2(slim_arm_param[param_index + 2],
        slim_arm_param[param_index + 3]);
        vec2 p3 = vec2(slim_arm_param[param_index + 4],
        slim_arm_param[param_index + 5]);
        float r = slim_arm_param[param_index + 6];
        vec2 part_ratio = vec2(slim_arm_param[param_index + 7],
        slim_arm_param[param_index + 8]);
        vec4 max_area = vec4(slim_arm_param[param_index + 9],
        slim_arm_param[param_index + 10],
        slim_arm_param[param_index + 11],
        slim_arm_param[param_index + 12]);
        if (is_in_box(max_area, coor) == 1) {
            offset += slim_arm_process(coor, p1, p2, p3, r, part_ratio, 0, scale.x, scale.y);
        }
    }
    param_index += 13;
    if (scale.z != 0.0 || scale.w != 0.0) {
        vec2 p1 = vec2(slim_arm_param[param_index + 0],
        slim_arm_param[param_index + 1]);
        vec2 p2 = vec2(slim_arm_param[param_index + 2],
        slim_arm_param[param_index + 3]);
        vec2 p3 = vec2(slim_arm_param[param_index + 4],
        slim_arm_param[param_index + 5]);
        float r = slim_arm_param[param_index + 6];
        vec2 part_ratio = vec2(slim_arm_param[param_index + 7],
        slim_arm_param[param_index + 8]);
        vec4 max_area = vec4(slim_arm_param[param_index + 9],
        slim_arm_param[param_index + 10],
        slim_arm_param[param_index + 11],
        slim_arm_param[param_index + 12]);
        if (is_in_box(max_area, coor) == 1) {
            offset += slim_arm_process(coor, p1, p2, p3, r, part_ratio, 1, scale.z, scale.w);
        }
    }

    return offset;
}

void main() {
    vec2 coor = textureCoordinate;
    coor.y = 1. - coor.y;
    coor *= resolution;

    coor += process(coor);

    coor /= resolution;
    coor.y = 1. - coor.y;

    vec2 offset = coor - textureCoordinate;
    vec2 inOffset = data2offset(texture2D(inputImageTexture, textureCoordinate));
    gl_FragColor = offset2data(inOffset + offset);
}