precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D texture;
uniform sampler2D texture2;
uniform sampler2D texture3;
uniform sampler2D texture4;
uniform sampler2D texture5;
uniform sampler2D texture6;

uniform vec2 resolution;
uniform float type;
uniform float thickness;

const float opacity = 1.;
const float PI = 3.1415926;

const vec3 grayXfer = vec3(0.3, 0.59, 0.11);

vec3 hsv2rgb(vec3 c)
{
    vec3 rgb = clamp(abs(mod(c.x*6.0+vec3(0.0, 4.0, 2.0), 6.0)-3.0)-1.0, 0.0, 1.0);
    return c.z * mix(vec3(1.0), rgb, c.y);
}

vec4 blendNormal(vec4 base, vec4 overlay) {
    return base + overlay - base * overlay.a;;
}

vec4 blendScreen(vec4 base, vec4 overlay) {
    vec4 whiteColor = vec4(1.0);
    return whiteColor - ((whiteColor - overlay) * (whiteColor - base));
}

vec4 gaussianBlurFast(sampler2D tex, vec2 uv, vec2 resolution, float size) {
    float Directions = 8.0;// BLUR DIRECTIONS (Default 16.0 - More is better but slower)
    float Quality = 4.0;// BLUR QUALITY (Default 4.0 - More is better but slower)
    float Size = size;// BLUR SIZE (Radius)
    vec2 Radius = Size/resolution.xy;
    vec4 Color = texture2D(tex, uv);
    float count = 1.;
    if (Size > 0.0) {
        for (float d=0.0; d<2.*PI; d+=2.*PI/Directions) {
            for (float i=1.0/Quality; i<=1.0; i+=1.0/Quality) {
                Color += texture2D(tex, uv+vec2(cos(d), sin(d))*Radius*i);
                count += 1.;
            }
        }
    }
    Color /= count;
    return Color;
}


vec4 blendOverlay(vec4 base, vec4 overlay) {
    mediump float ra;
    if (2.0 * base.r < base.a) {
        ra = 2.0 * overlay.r * base.r + overlay.r * (1.0 - base.a) + base.r * (1.0 - overlay.a);
    } else {
        ra = overlay.a * base.a - 2.0 * (base.a - base.r) * (overlay.a - overlay.r) + overlay.r * (1.0 - base.a) + base.r * (1.0 - overlay.a);
    }

    mediump float ga;
    if (2.0 * base.g < base.a) {
        ga = 2.0 * overlay.g * base.g + overlay.g * (1.0 - base.a) + base.g * (1.0 - overlay.a);
    } else {
        ga = overlay.a * base.a - 2.0 * (base.a - base.g) * (overlay.a - overlay.g) + overlay.g * (1.0 - base.a) + base.g * (1.0 - overlay.a);
    }

    mediump float ba;
    if (2.0 * base.b < base.a) {
        ba = 2.0 * overlay.b * base.b + overlay.b * (1.0 - base.a) + base.b * (1.0 - overlay.a);
    } else {
        ba = overlay.a * base.a - 2.0 * (base.a - base.b) * (overlay.a - overlay.b) + overlay.b * (1.0 - base.a) + base.b * (1.0 - overlay.a);
    }

    float alpha = overlay.a + base.a - overlay.a * base.a;
    return vec4(ra, ga, ba, alpha);
}

void main() {

    vec4 sucai = texture2D(texture5, vec2(textureCoordinate.x, textureCoordinate.y));

  //  vec4 front = texture2D(texture, vec2(textureCoordinate.x, 1. - textureCoordinate.y));
    vec4 front = texture2D(texture, vec2(textureCoordinate.x, 1. - textureCoordinate.y));
    vec4 back = texture2D(texture2, vec2(textureCoordinate.x, 1. - textureCoordinate.y));

    front.rgb = mix(front.rgb, vec3(smoothstep(0.0, 0.05, front.r+front.g+front.b)), step(1., type));
    back.rgb = mix(back.rgb, vec3(smoothstep(0.0, 0.05, back.r+back.g+back.b)), step(1., type));

    vec4 frontBlur = gaussianBlurFast(texture, vec2(textureCoordinate.x, 1. - textureCoordinate.y), resolution, 12.*thickness);
    vec4 backBlur = gaussianBlurFast(texture2, vec2(textureCoordinate.x, 1. - textureCoordinate.y), resolution, 12.*thickness);

    vec4 scene = texture2D(texture3, textureCoordinate);

    vec4 scene2 = texture2D(texture6, textureCoordinate);

    vec4 mask = texture2D(texture4, vec2(textureCoordinate.x, 1. - textureCoordinate.y));

    vec4 obj = mix(vec4(0.), scene, mask.r);

    vec4 sucai1 = mix(vec4(0.), sucai, front.a);
    vec4 sucai2 = mix(vec4(0.), sucai, back.a);

    backBlur.a *= 0.5;
    frontBlur.a *= 0.5;
    front.a *= 0.5;
    back.a *= 0.5;

    front = blendOverlay(sucai1, front);
    back = blendOverlay(sucai2, back);

    vec4 col = scene2;
    col = blendNormal(col, backBlur);
    col = blendNormal(col, back);
    col = blendNormal(col, obj);
    col = blendNormal(col, frontBlur);
    col = blendNormal(col, front);

    gl_FragColor = col;

}
