precision highp float;
varying vec2 textureCoordinate;
// scene
uniform sampler2D texture;
// mask
uniform sampler2D texture2;
// contour
uniform sampler2D texture3;
// scene2
uniform sampler2D texture4;

uniform vec2 resolution;



uniform mat4 params;

#define PI 3.141592653
#define TRANSPARENT vec4(0.)
#define WHITE vec4(1.)

vec4 blendScreen(vec4 base, vec4 overlay) {
    vec4 whiteColor = vec4(1.0);
    return whiteColor - ((whiteColor - overlay) * (whiteColor - base));
}

vec4 blendNormal(vec4 base, vec4 overlay) {
    return base + overlay - base * overlay.a;;
}

void blend(in vec4 src, inout vec4 res, float opacity) {
    res = vec4(src.rgb*opacity*(1.-step(src.a, 0.))+ (1.-src.a*opacity)*res.rgb, max(res.a, src.a*opacity));
}

vec3 hsv2rgb(in vec3 c)
{
    vec3 rgb = clamp(abs(mod(c.x*6.0+vec3(0.0, 4.0, 2.0), 6.0)-3.0)-1.0, 0.0, 1.0);
    return c.z * mix(vec3(1.0), rgb, c.y);
}

vec4 gaussianBlurFast(sampler2D tex, vec2 uv, vec2 resolution, float size) {
    // GAUSSIAN BLUR SETTINGS {{{
    float Directions = 8.0;// BLUR DIRECTIONS (Default 16.0 - More is better but slower)
    float Quality = 4.0;// BLUR QUALITY (Default 4.0 - More is better but slower)
    float Size = size;// BLUR SIZE (Radius)
    // GAUSSIAN BLUR SETTINGS }}}
    vec2 Radius = Size/resolution.xy;
    vec4 Color = texture2D(tex, uv);
    int count = 1;
    if (Size > 0.0) {
        for (float d=0.0; d<2.*PI; d+=2.*PI/Directions) {
            for (float i=1.0/Quality; i<=1.0; i+=1.0/Quality) {
                Color.rgb += texture2D(tex, uv+vec2(cos(d), sin(d))*Radius*i).rgb;
                count++;
            }
        }
    }
    Color.rgb /= float(count);
    return Color;
}


void main() {
    vec4 scene = texture2D(texture, textureCoordinate);
    float thickness = params[0][0];

    vec4 mask = texture2D(texture2, vec2(textureCoordinate.x, 1. - textureCoordinate.y));
    vec4 scene2 = texture2D(texture4, textureCoordinate);

    vec4 obj = mix(vec4(0.), scene, mask.r);
    vec4 outscene = blendNormal(scene2, obj);

    vec4 contour = texture2D(texture3, vec2(textureCoordinate.x, 1. - textureCoordinate.y));
    vec4 contourBlur = gaussianBlurFast(texture3, vec2(textureCoordinate.x, 1. - textureCoordinate.y), resolution, 12. * thickness);

    float p_hue = params[0][2];
    float p_sat = params[0][3];
    float p_opacity = params[1][0];
    float p_hue2 = params[1][3];
    float p_sat2 = params[2][0];

    vec3 col1 = hsv2rgb(vec3(p_hue, p_sat, 1.));
    vec3 col2 = hsv2rgb(vec3(p_hue2, p_sat2, 1.));

    blend(vec4(col2, 0.8), outscene, clamp(contourBlur.r*2., 0., 1.)*p_opacity);
    blend(vec4(col1, 0.5), outscene, contour.r*p_opacity);

    gl_FragColor = outscene;

}