precision highp float;
varying vec2 textureCoordinate;
//scene
uniform sampler2D texture;
//mask
uniform sampler2D texture2;
//scene2
uniform sampler2D texture3;
//sucai
uniform sampler2D texture4;
//scene3
uniform sampler2D texture5;

#define PI 3.141592653

uniform vec2 resolution;
uniform float opacity;
uniform mat4 params;
uniform float progress;

vec4 blendNormal(vec4 base, vec4 overlay) {
    return base + overlay - base * overlay.a;;
}

vec3 hsv2rgb(vec3 c)
{
    vec3 rgb = clamp(abs(mod(c.x*6.0+vec3(0.0, 4.0, 2.0), 6.0)-3.0)-1.0, 0.0, 1.0);
    return c.z * mix(vec3(1.0), rgb, c.y);
}

void blend(in vec4 src, inout vec4 res, float opacity) {
    res = vec4(src.rgb*opacity*(1.-step(src.a, 0.))+ (1.-src.a*opacity)*res.rgb, max(res.a, src.a*opacity));
}

vec4 blendOverlay(vec4 base, vec4 overlay) {
    mediump float ra;
    if (2.0 * base.r < base.a) {
        ra = 2.0 * overlay.r * base.r + overlay.r * (1.0 - base.a) + base.r * (1.0 - overlay.a);
    } else {
        ra = overlay.a * base.a - 2.0 * (base.a - base.r) * (overlay.a - overlay.r) + overlay.r * (1.0 - base.a) + base.r * (1.0 - overlay.a);
    }

    mediump float ga;
    if (2.0 * base.g < base.a) {
        ga = 2.0 * overlay.g * base.g + overlay.g * (1.0 - base.a) + base.g * (1.0 - overlay.a);
    } else {
        ga = overlay.a * base.a - 2.0 * (base.a - base.g) * (overlay.a - overlay.g) + overlay.g * (1.0 - base.a) + base.g * (1.0 - overlay.a);
    }

    mediump float ba;
    if (2.0 * base.b < base.a) {
        ba = 2.0 * overlay.b * base.b + overlay.b * (1.0 - base.a) + base.b * (1.0 - overlay.a);
    } else {
        ba = overlay.a * base.a - 2.0 * (base.a - base.b) * (overlay.a - overlay.b) + overlay.b * (1.0 - base.a) + base.b * (1.0 - overlay.a);
    }

    float alpha = overlay.a + base.a - overlay.a * base.a;
    return vec4(ra, ga, ba, alpha);
}

vec4 gaussianBlurFast(sampler2D tex, vec2 uv, vec2 resolution, float size) {
    // GAUSSIAN BLUR SETTINGS {{{
    float Directions = 8.0;// BLUR DIRECTIONS (Default 16.0 - More is better but slower)
    float Quality = 4.0;// BLUR QUALITY (Default 4.0 - More is better but slower)
    float Size = size;// BLUR SIZE (Radius)
    // GAUSSIAN BLUR SETTINGS }}}
    vec2 Radius = Size/resolution.xy;
    vec4 Color = texture2D(tex, uv);
    int count = 1;
    if (Size > 0.0) {
        for (float d=0.0; d<2.*PI; d+=2.*PI/Directions) {
            for (float i=1.0/Quality; i<=1.0; i+=1.0/Quality) {
                Color += texture2D(tex, uv+vec2(cos(d), sin(d))*Radius*i);
                count++;
            }
        }
    }
    Color /= float(count);
    return Color;
}

void main() {

    vec2 uv = textureCoordinate;
    vec4 scene1 = texture2D(texture, uv);
    vec4 sucai = texture2D(texture4, uv);
    vec4 scene3 = texture2D(texture5, uv);

    float thickness = params[0][0];
    float offsetX = params[0][1];
    float offsetY = params[0][2];
    float p_hue1 = params[0][3];
    float p_sat1 = params[1][0];
    float p_hue2 = params[1][1];
    float p_sat2 = params[1][2];
    float p_hue3 = params[1][3];
    float p_sat3 = params[2][0];

    float w = progress * 3. * PI;
    vec3 layer1 = hsv2rgb(vec3(p_hue1, p_sat1, 1.0));
    vec3 layer2 = hsv2rgb(vec3(p_hue2, p_sat2, 1.0));
    vec3 layer3 = hsv2rgb(vec3(p_hue3, p_sat3, 1.0));

    vec3 layer4 = mix(layer1, mix(layer2, layer3, step(2./3.*3.* PI, w)), step(1./3.*3. * PI, w));




    vec4 mask0 = texture2D(texture2, vec2(uv.x, 1. - uv.y));

    uv.x += (offsetX - 0.5);
    uv.y += (offsetY - 0.5);

    vec4 mask = texture2D(texture2, vec2(uv.x, 1. - uv.y));

    vec4 cutout = mix(vec4(0.), scene1, mask0.r);
    vec4 outscene = blendNormal(scene3, cutout);

    float opacity = sin(progress * PI * 2.);

    vec4 col2 = gaussianBlurFast(texture3, vec2(uv.x, uv.y), resolution, 12. * thickness);
    col2 = mix(vec4(0.), col2, mask.r);
    vec4 col3 = vec4(layer4, mix(0., col2.r, step(0.001, col2.r)));
    //    vec4 sucai1 = mix(vec4(0.), sucai, col2.a);
    //    vec4 col3 = blendNormal(sucai1, vec4(hsv2rgb(vec3(hue, sat, 1.)), mix(0., col2.r, step(0.001, col2.r))));
    blend(col3, outscene, abs(opacity)*clamp(col2.r*2., 0., 1.));

    col2 = texture2D(texture3, vec2(uv.x, uv.y));
    col2 = mix(vec4(0.), col2, mask.r);
    //    col2.a *= 0.3;

    vec4 sucai2 = mix(vec4(0.), sucai, col2.a);
    col3 = blendOverlay(sucai2, vec4(layer4, mix(0., col2.r, step(0.001, col2.r))));

    blend(col3, outscene, abs(opacity)*clamp(col2.r*2., 0., 1.));

    gl_FragColor = outscene;

}