precision mediump float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform vec2 resolution;
uniform float d;
uniform float flag;

float lookup(vec2 p, float dx, float dy)
{
    vec2 uv = (p.xy + vec2(dx * d, dy * d)) / resolution.xy;
    vec4 c = texture2D(inputImageTexture, uv.xy);

    // return as luma
    return 0.2126*c.r + 0.7152*c.g + 0.0722*c.b;
}

void main() {
    vec2 p = textureCoordinate * resolution;

    // simple sobel edge detection
    float gx = 0.0;
    gx += -1.0 * lookup(p, -1.0, -1.0);
    gx += -2.0 * lookup(p, -1.0, 0.0);
    gx += -1.0 * lookup(p, -1.0, 1.0);
    gx +=  1.0 * lookup(p, 1.0, -1.0);
    gx +=  2.0 * lookup(p, 1.0, 0.0);
    gx +=  1.0 * lookup(p, 1.0, 1.0);

    float gy = 0.0;
    gy += -1.0 * lookup(p, -1.0, -1.0);
    gy += -2.0 * lookup(p, 0.0, -1.0);
    gy += -1.0 * lookup(p, 1.0, -1.0);
    gy +=  1.0 * lookup(p, -1.0, 1.0);
    gy +=  2.0 * lookup(p, 0.0, 1.0);
    gy +=  1.0 * lookup(p, 1.0, 1.0);

    float g = gx*gx + gy*gy;
    vec4 col = vec4(0.0, 0.0, 0.0, 0.0);
    col += vec4(g, g, g, g);

    gl_FragColor = mix(col, col+texture2D(inputImageTexture, textureCoordinate), flag);
}