precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform highp vec2 uSize;


uniform lowp mat4 colorMatrix;
uniform highp float exposure;
uniform lowp float contrast;
uniform lowp float saturation;
uniform lowp float temperature;
uniform lowp float tint;
uniform lowp vec2 vignetteCenter;
uniform lowp vec3 vignetteColor;
uniform highp float vignetteStart;
uniform highp float vignetteEnd;
uniform lowp float shadows;
uniform lowp float highlights;
uniform lowp float ambiance;
uniform lowp float brightness;
uniform lowp float grain;
uniform lowp float fade;
uniform mediump float uHue;
uniform float uSharpness; // 0-3
uniform float uSharpnessRadius; // 0.3

const mediump vec3 luminanceWeightingHighlight = vec3(0.3, 0.3, 0.3);

const mediump vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);

const lowp vec3 warmFilter = vec3(0.93, 0.54, 0.0);
const mediump mat3 RGBtoYIQ = mat3(0.299, 0.587, 0.114, 0.596, -0.274, -0.322, 0.212, -0.523, 0.311);
const mediump mat3 YIQtoRGB = mat3(1.0, 0.956, 0.621, 1.0, -0.272, -0.647, 1.0, -1.105, 1.702);

//highp vec3 rgbToHsl(highp vec3 color)
//{
//    highp vec3 hsl;
//    highp float fmin = min(min(color.r, color.g), color.b);
//    highp float fmax = max(max(color.r, color.g), color.b);
//    highp float delta = fmax - fmin;
//    hsl.z = (fmax + fmin) / 2.0;
//    if (delta == 0.0)
//    {
//        hsl.x = 0.0;
//        hsl.y = 0.0;
//    }
//    else
//    {
//        if (hsl.z < 0.5) hsl.y = delta / (fmax + fmin);
//        else hsl.y = delta / (2.0 - fmax - fmin);
//        highp float deltaR = (((fmax - color.r) / 6.0) + (delta / 2.0)) / delta;
//        highp float deltaG = (((fmax - color.g) / 6.0) + (delta / 2.0)) / delta;
//        highp float deltaB = (((fmax - color.b) / 6.0) + (delta / 2.0)) / delta;
//        if (color.r == fmax ) hsl.x = deltaB - deltaG;
//        else if (color.g == fmax) hsl.x = (1.0 / 3.0) + deltaR - deltaB;
//        else if (color.b == fmax) hsl.x = (2.0 / 3.0) + deltaG - deltaR;
//        if (hsl.x < 0.0) hsl.x += 1.0;
//        else if (hsl.x > 1.0) hsl.x -= 1.0;
//    }
//    return hsl;
//}
            
//highp float hueToRgb(highp float f1, highp float f2, highp float hue)
//{
//    if (hue < 0.0) hue += 1.0;
//    else if (hue > 1.0) hue -= 1.0;
//    highp float res;
//    if ((6.0 * hue) < 1.0) res = f1 + (f2 - f1) * 6.0 * hue;
//    else if ((2.0 * hue) < 1.0) res = f2;
//    else if ((3.0 * hue) < 2.0) res = f1 + (f2 - f1) * ((2.0 / 3.0) - hue) * 6.0;
//    else res = f1;
//    return res;
//}
            
//highp vec3 hslToRgb(highp vec3 hsl)
//{
//    highp vec3 rgb;
//    if (hsl.y == 0.0)
//    {
//        rgb = vec3(hsl.z);
//    }
//
//    else
//    {
//        highp float f2;
//        if (hsl.z < 0.5) f2 = hsl.z * (1.0 + hsl.y);
//        else f2 = (hsl.z + hsl.y) - (hsl.y * hsl.z);
//        highp float f1 = 2.0 * hsl.z - f2;
//        rgb.r = hueToRgb(f1, f2, hsl.x + (1.0/3.0));
//        rgb.g = hueToRgb(f1, f2, hsl.x);
//        rgb.b = hueToRgb(f1, f2, hsl.x - (1.0/3.0));
//    }
//
//    return rgb;
//}
            
//lowp vec3 applyLuminanceCurve(lowp vec3 pixel)
//{
//    int index = int(clamp(pixel.z / (1.0 / 200.0), 0.0, 199.0));
//    highp float value = rgbCurveValues[index];
//    highp float grayscale = (smoothstep(0.0, 0.1, pixel.z) * (1.0 - smoothstep(0.8, 1.0, pixel.z)));
//    highp float saturation = mix(0.0, pixel.y, grayscale);
//    pixel.y = saturation;
//    pixel.z = value;
//    return pixel;
//}
            
//lowp vec3 applyRGBCurve(lowp vec3 pixel)
//{
//    int index = int(clamp(pixel.r / (1.0 / 200.0), 0.0, 199.0));
//    highp float value = redCurveValues[index];
//    pixel.r = value;
//    index = int(clamp(pixel.g / (1.0 / 200.0), 0.0, 199.0));
//    value = greenCurveValues[index];
//    pixel.g = clamp(value, 0.0, 1.0);
//    index = int(clamp(pixel.b / (1.0 / 200.0), 0.0, 199.0));
//    value = blueCurveValues[index];
//    pixel.b = clamp(value, 0.0, 1.0);
//    return pixel;
//}

vec3 blurSample(vec2 uv, vec2 xoff, vec2 yoff) {
    vec3 v11 = texture2D(inputImageTexture, uv + xoff).rgb;
    vec3 v12 = texture2D(inputImageTexture, uv + yoff).rgb;
    vec3 v21 = texture2D(inputImageTexture, uv - xoff).rgb;
    vec3 v22 = texture2D(inputImageTexture, uv - yoff).rgb;
    return (v11 + v12 + v21 + v22 + 2.0 * texture2D(inputImageTexture, uv).rgb) * 0.166667;
}

vec3 edgeStrength(vec2 uv) {
    float spread = uSharpnessRadius;
    vec2 offset = vec2(1.0) / vec2(uSize.x, uSize.y);
    vec2 up = vec2(0.0, offset.y) * spread;
    vec2 right = vec2(offset.x, 0.0) * spread;
    
    vec3 v12 = blurSample(uv + up, right, up);
    
    vec3 v21 = blurSample(uv - right, right, up);
    vec3 v22 = blurSample(uv, right, up);
    vec3 v23 = blurSample(uv + right, right, up);
    
    vec3 v32 = blurSample(uv - up, right, up);
    
    vec3 laplacian_of_g =  v12 *  1.0
    + v21 * 1.0 + v22 * -4.0 + v23 * 1.0
    + v32 * 1.0 ;
    laplacian_of_g = laplacian_of_g * (spread / 0.3 * 0.12 + 0.88);
    return laplacian_of_g.xyz;
}

vec4 adjustSharpness(vec4 color, highp vec2 uv) {
    return vec4(color.xyz - edgeStrength(uv) * uSharpness, 1.0);
}

const highp  vec4  kRGBToYPrime = vec4 (0.299, 0.587, 0.114, 0.0);
const highp  vec4  kRGBToI     = vec4 (0.595716, -0.274453, -0.321263, 0.0);
const highp  vec4  kRGBToQ     = vec4 (0.211456, -0.522591, 0.31135, 0.0);

const highp  vec4  kYIQToR   = vec4 (1.0, 0.9563, 0.6210, 0.0);
const highp  vec4  kYIQToG   = vec4 (1.0, -0.2721, -0.6474, 0.0);
const highp  vec4  kYIQToB   = vec4 (1.0, -1.1070, 1.7046, 0.0);

vec4 adjustHue(vec4 color)
{
    highp float   YPrime  = dot (color, kRGBToYPrime);
    highp float   I      = dot (color, kRGBToI);
    highp float   Q      = dot (color, kRGBToQ);
    
    highp float   hue     = atan (Q, I);
    highp float   chroma  = sqrt (I * I + Q * Q);

    hue -= uHue;
    
    Q = chroma * sin (hue);
    I = chroma * cos (hue);
    
    highp vec4    yIQ   = vec4 (YPrime, I, Q, 0.0);
    color.r = dot (yIQ, kYIQToR);
    color.g = dot (yIQ, kYIQToG);
    color.b = dot (yIQ, kYIQToB);
   
    return color;
}

vec4 adjust(vec4 srcColor, highp vec2 uv) {
//     if (uHue > 0.0) {
//         srcColor = adjustHue(srcColor);
//     }
    
    mediump float luminance2 = dot(srcColor.rgb, luminanceWeightingHighlight);
    mediump float shadow;
    if (shadows > 0.0) {
        shadow = clamp((pow(luminance2, 1.0/(shadows+1.0)) + (-0.76)*pow(luminance2, 2.0/(shadows+1.0))) - luminance2, 0.0, 1.0);
    } else {
        shadow = clamp((pow(luminance2, 1.0-shadows) + (0.76)*pow(luminance2, 2.0*(1.0-shadows))) - luminance2, -1.0, 0.0);
    }
    
    mediump float highlight;
    if (highlights < 0.0) {
        highlight = clamp((1.0 - (pow(1.0-luminance2, 1.0/(1.0-highlights)) + (-0.8)*pow(1.0-luminance2, 2.0/(1.0-highlights)))) - luminance2, -1.0, 0.0);
    } else {
        highlight = clamp((1.0 - pow(1.0-luminance2, 1.0+highlights) + (-0.8)*pow(1.0-luminance2, 2.0*(1.0+highlights))) - luminance2, 0.0, 1.0);
    }
        
    lowp vec3 rs = vec3(0.0, 0.0, 0.0) + ((luminance2 + shadow + highlight) - 0.0) * ((srcColor.rgb - vec3(0.0, 0.0, 0.0))/(luminance2 - 0.0));
    lowp vec4 color = vec4(rs, srcColor.a);


    highp float x = (uv.x + 4.0) * (uv.y + 4.0) * 10.0;
    highp vec4 grain = vec4(mod((mod(x, 13.0) + 1.0) * (mod(x, 123.0) + 1.0), 0.01) - 0.005) * grain;
    highp vec4 result =  color + grain;

    // sharpen
    if (uSharpnessRadius > 0.0) {
        result = adjustSharpness(result, uv);
    }
    
    lowp vec4 exposureColor = vec4(result.rgb * pow(2.0, exposure), result.w);

    lowp vec4 outputColor = exposureColor * colorMatrix;
    lowp vec4 fadeColor = (fade * outputColor) + ((1.0 - fade) * exposureColor);


    lowp vec4 contrastColor = vec4(((fadeColor.rgb - vec3(0.5)) * contrast + vec3(0.5)), fadeColor.w);


    lowp float luminance = dot(contrastColor.rgb, luminanceWeighting);
    lowp vec3 greyScaleColor = vec3(luminance);
    lowp vec4 saturationColor = vec4(mix(greyScaleColor, contrastColor.rgb, saturation), contrastColor.w);


    mediump vec3 yiq = RGBtoYIQ * saturationColor.rgb; //adjusting tint
    yiq.b = clamp(yiq.b + tint*0.5226*0.1, -0.5226, 0.5226);
    lowp vec3 rgb = YIQtoRGB * yiq;
    lowp vec3 processed = vec3(
            (rgb.r < 0.5 ? (2.0 * rgb.r * warmFilter.r) : (1.0 - 2.0 * (1.0 - rgb.r) * (1.0 - warmFilter.r))), //adjusting temperature
            (rgb.g < 0.5 ? (2.0 * rgb.g * warmFilter.g) : (1.0 - 2.0 * (1.0 - rgb.g) * (1.0 - warmFilter.g))),
            (rgb.b < 0.5 ? (2.0 * rgb.b * warmFilter.b) : (1.0 - 2.0 * (1.0 - rgb.b) * (1.0 - warmFilter.b))));
    lowp vec4 seWenSeDiao = vec4(mix(rgb, processed, temperature), saturationColor.a);


    lowp float d = distance(textureCoordinate, vec2(vignetteCenter.x, vignetteCenter.y));
    lowp float percent = smoothstep(vignetteStart, vignetteEnd, d);
    lowp vec4 vignette = vec4(mix(seWenSeDiao.x, vignetteColor.x, percent), mix(seWenSeDiao.y, vignetteColor.y, percent), mix(seWenSeDiao.z, vignetteColor.z, percent), 1.0);


    lowp vec4 rsShadow = vec4(vignette.rgb, vignette.a);
    lowp float luminance3 = dot(rsShadow.rgb, luminanceWeighting);
    lowp vec3 greyScaleColorFenWei = vec3(luminance3);
    lowp vec4 fenweiSaturation;
    if (ambiance < 0.0) {
        fenweiSaturation = vec4(mix(greyScaleColorFenWei, rsShadow.rgb, abs(ambiance / 5.0) + 1.0), rsShadow.a);
    } else {
        fenweiSaturation = vec4(mix(greyScaleColorFenWei, rsShadow.rgb, ambiance * 3.0 + 1.0), rsShadow.a);
    }
    
    highp float lum = fenweiSaturation.r * 0.3 + fenweiSaturation.g * 0.59 + fenweiSaturation.b * 0.11;
    lowp vec4 fenweiRs = vec4(fenweiSaturation.r - fenweiSaturation.r * ambiance * (lum - 0.5),
                        fenweiSaturation.g - fenweiSaturation.g * ambiance * (lum - 0.5),
                        fenweiSaturation.b - fenweiSaturation.b * ambiance * (lum - 0.5),
                        fenweiSaturation.a);
    return vec4((fenweiRs.rgb + vec3(brightness)), fenweiRs.w);
}

void main() {
//    vec4 result = vec4(applyRGBCurve(hslToRgb(applyLuminanceCurve(rgbToHsl(result.rgb)))), result.a);
    highp vec4 color = texture2D(inputImageTexture,fract(textureCoordinate));
    float a = color.a;
    color = vec4(color.rgb / (a + step(a, 0.0)), 1.0);
   color = adjust(color, textureCoordinate);
   if (uHue > 0.0) {
       color = adjustHue(color);
   }
    gl_FragColor = color  * a;
}
