precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float strength;
vec3 kLuminance = vec3(0.2125, 0.7154, 0.0721);

vec3 saturation(vec3 color, float k) {
    float luminance = dot(kLuminance, color);
    vec3 luminanceColor = vec3(luminance);
    return color - (luminanceColor - color) * k;
}
const mediump vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);

vec3 luminance(vec3 color, float k) {
    return color + k;
}
vec3 contrast(vec3 color, float k) {
    return 0.5 + (color-0.5)*(1.0+k);
}
vec3 brightness(vec3 color, float k) {
    return color * (1.0+k);
}

void main()
{
    vec2 uv = textureCoordinate;
    vec4 textureColor = texture2D(inputImageTexture, uv);
    vec3 color = textureColor.rgb;
    color = saturation(color, -0.208);
    color = luminance(color, (0.071) * strength);
    color = contrast(color, (1.5534) * strength);
    color = brightness(color, (-0.41) * strength);

    gl_FragColor = vec4(color, textureColor.a);
}