
precision highp float;

varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;

uniform vec2 iResolution;
uniform float uIntensity;
uniform float uDirection;

uniform int samplesMulti;

const int kSamples = 32; // multiple of 2

vec4 DirectionalBlur(vec2 UV, vec2 Direction, float Intensity, sampler2D Texture)
{
    vec4 Color = vec4(0.0);
    
    vec2 aspect = iResolution.yy / iResolution.xy;
    
    int samples = kSamples * samplesMulti;
    for (int i = 1; i <= samples / 2; i++) {
        vec2 delta = float(i) * Intensity / float(samples / 2) * Direction;
        vec2 add = UV + delta;
        vec2 sub = UV - delta;
        
        Color += texture2D(Texture, aspect * add);
        Color += texture2D(Texture, aspect * sub);
    }
    
    vec4 result = Color / float(samples);
    return result;
}

void main() {
    vec2 UV = textureCoordinate * iResolution.xy / iResolution.yy;
    vec2 Direction = vec2(cos(uDirection), sin(uDirection));
    float intensity = uIntensity * 0.1;
    vec4 color = DirectionalBlur(UV, Direction, intensity, inputImageTexture);
    gl_FragColor = color;
}
