precision highp float;

varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform vec2 iChannelResolution;
uniform vec2 uCenter;
uniform float uRadius;
uniform float rotateAngle[12];

vec2 platformUV(vec2 uv, vec2 mat) {
    return mat2(uv.x, mat.x, 0., mat.y)*vec2(1., uv.y);
}

vec3 hsv2rgb(in vec3 c) {
    vec3 rgb = clamp(abs(mod(c.x*6.0+vec3(0.0, 4.0, 2.0), 6.0)-3.0)-1.0, 0.0, 1.0);
//    return c.z * mix(vec3(1.0), rgb, c.y); c.y = 1.0, c.z = 1.0,直接return;
    return rgb;
}

vec2 rotateUV(vec2 st, vec2 center, float angle, float ratio) {
    vec2 difference = st - center;
    difference.y *= 1./ratio;
    float deg = radians(angle);
    vec2 rotatedDir = difference*mat2(cos(deg), -sin(deg), sin(deg), cos(deg));
    rotatedDir.y *= ratio;
    rotatedDir += center;
    return rotatedDir;
}

vec4 blend(vec4 src, vec4 layer, vec3 alpha) {
    vec4 color = src;
    color.r = mix(src.r, layer.r, alpha.r);
    color.g = mix(src.g, layer.g, alpha.g);
    color.b = mix(src.b, layer.b, alpha.b);
    return color;
}

float easeInCubic(float x) {
    return x * x * x;
}

vec4 prism(sampler2D samp, vec2 uv, vec2 center, vec4 src) {
    float ratio = iChannelResolution.x/iChannelResolution.y;
    vec2 pos = uv;
    vec4 color = src;
    int COUNT = 12;
    float colorst = -0.2;
    float colored = -1.;
    for (int i = 0; i < COUNT; i++) {
        vec4 layer = texture2D(samp, rotateUV(uv, center, rotateAngle[i], ratio));
        float hue = mix(colorst, colored, easeInCubic(float(i)/float(COUNT-1)));
        color = blend(color, layer, hsv2rgb(vec3(hue, 1., 1.)));
    }
    return color;
}


void main() {
    /// 为了兼容平台
//    vec2 uv = platformUV(textureCoordinate, vec2(0., 1.));
    vec2 uv = textureCoordinate;
    /// y值映射为真实比例值
    float ratio = iChannelResolution.y / iChannelResolution.x;
    vec4 color = vec4(0.);
    vec2 pos = uv-uCenter;
    pos.y *= 1. * ratio;
    float len = length(pos);
    
    vec4 source = texture2D(inputImageTexture, uv);
    vec2 relativeCenter = uCenter + normalize(uv-uCenter)*max(uRadius-.12, 0.);
    color = prism(inputImageTexture, uv, relativeCenter, source);
    color = mix(source, color, smoothstep(-0.04, 0.04, len-uRadius));

    gl_FragColor = color;
}
