
precision highp float;

varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform vec2 verticeSize;

uniform int hasInput;
uniform int hasLock;

uniform int mode;
uniform float degree;
uniform vec2 fromPoint;
uniform vec2 toPoint;
uniform float radius;
uniform float scale;
uniform float ratio;// w/h

uniform int highPrecision;

vec2 uv2uvInt(vec2 uv) {
    return floor(uv * verticeSize) / (verticeSize - 1.0);
}

vec2 uvInt2uv(vec2 uvInt) {
    return (floor(uvInt * (verticeSize - 1.0) + 0.5) + 0.5) / verticeSize;
}

vec4 uvInt2data(vec2 uvInt) {
    uvInt.x = clamp(uvInt.x, 0.0, 1.0);
    uvInt.y = clamp(uvInt.y, 0.0, 1.0);
    vec2 xy = uvInt * 255.;
    vec2 fxy = fract(xy) * 255.;
    xy = floor(xy) / 255.;
    fxy = floor(fxy + 0.5) / 255.;
    return vec4(xy, fxy);
}

vec2 data2uvInt(vec4 data) {
    vec2 xy = floor(data.xy * 255. + 0.5);
    vec2 fxy = floor(data.zw * 255. + 0.5);
    vec2 uvInt = (xy + fxy / 255.) / 255.;
    return uvInt;
}

vec4 uv2data(vec2 uv) {
    return uvInt2data(uv2uvInt(uv));
}

vec2 data2uv(vec4 data) {
    return uvInt2uv(data2uvInt(data));
}

float ptr_distance(vec2 p1, vec2 p2) {
    vec2 d = p2 - p1;
    d.y /= ratio;
    return length(d);
}

vec2 warp(vec2 p, float lock) {
    float dist = ptr_distance(p, fromPoint);
    if (dist < radius) {
        vec2 offset = toPoint - fromPoint;
        offset /= max(length(offset) / radius, 1.0);
        float tt = (1.0 - smoothstep(0.0, 1.0, dist/radius)) * degree;
        tt *= (1. - lock);
        if(highPrecision > 0){
            p += offset * tt;
        }else{
            p -= offset * tt;
        }
    }
    return p;
}

vec2 unwarp(vec2 p, vec2 origin, float lock) {
    float dist = ptr_distance(p, toPoint);
    if (dist < radius) {
        float d = ptr_distance(p, origin);
        if (d <= 0.000001) {
            p = origin;
        } else {
            float tt = (1.0 - smoothstep(0.0, 1.0, dist/radius));
            float td = min(ptr_distance(fromPoint, toPoint) * 0.5 / radius, 1.0);
            p = mix(p, origin, td * tt * (1. - lock));
        }
    }
    return p;
}

vec2 scaleAt(vec2 p, float lock) {
    float dist = ptr_distance(p, fromPoint);
    if (dist < radius) {
        float tt = (1.0 - smoothstep(0.0, 1.0, dist/radius)) * 0.4;
        tt *= (1. - lock);
        if(highPrecision > 0){
            p += (p - fromPoint) * (scale - 1.0) * tt;
        }else{
            p -= (p - fromPoint) * (scale - 1.0) * tt;
        }
    }
    return p;
}

void main()
{
    vec2 uv = textureCoordinate;
    vec2 originUvInt = uv2uvInt(uv);
    vec2 uvInt;
    if (hasInput == 0) {
        uvInt = originUvInt;
    } else {
        vec4 data = texture2D(inputImageTexture, uv);
        uvInt = data2uvInt(data);
    }

    float lock = hasLock == 0 ? 0.0 : texture2D(inputImageTexture2, uvInt).r;

    if (originUvInt.x <= 0.0 || originUvInt.x >= 1.0 || originUvInt.y <= 0.0 || originUvInt.y >= 1.0) {
        lock = 1.0;
    }

    //    if (lock < 0.5) {
    if (mode == 0) {
        uvInt = warp(uvInt, lock);
    } else if (mode == 2) {
        uvInt = scaleAt(uvInt, lock);
    }
    //    }
    if (mode == 1) {
        lock = min(lock, texture2D(inputImageTexture2, originUvInt).r);
        uvInt = unwarp(uvInt, originUvInt, lock);
    }

    gl_FragColor = uvInt2data(uvInt);
}
