precision highp float;
varying vec2 textureCoordinate;

uniform sampler2D sourceTexture;
uniform sampler2D maskTexture;
uniform vec2 iResolution;
uniform vec4 maskRect;
uniform vec2 texelSize;
uniform float strength;
uniform int flip;

const float kMultiplier = 2.2;

const vec3 kLunimanceWeights = vec3(0.299, 0.587, 0.114);

highp vec3 boxBlur() {
    highp vec3 sumColor = vec3(0.0);
    for (float v = -2.0; v < 2.5; v += 1.0) {
        for (float u = -2.0; u < 2.5; u += 1.0) {
            sumColor += texture2D(sourceTexture, textureCoordinate + vec2(u, v) * texelSize).rgb;
        }
    }
    return sumColor * 0.04;
}

highp vec3 blend(highp vec3 base, highp vec3 blend) {
    highp vec3 above = 2.0 * (base + blend - base * blend) - 1.0;
    highp vec3 below = 2.0 * base * blend;
    return mix(below, above, step(0.5, base));
}

void main() {
    highp vec4 color = texture2D(sourceTexture, textureCoordinate);

    vec2 xy = iResolution * textureCoordinate - maskRect.xy;
    if (0.0 <= xy.x && xy.x <= maskRect.z && 0.0 <= xy.y && xy.y <= maskRect.w) {
        vec2 pos = xy / maskRect.zw;
        if (flip == 1){
            pos.y = 1. - pos.y;
        }
        highp float maskValue = texture2D(maskTexture, pos).r;
        if (maskValue > 0.01) {
            vec3 blurColor = boxBlur();
            vec3 detail = vec3(dot(color.rgb - blurColor, kLunimanceWeights) * kMultiplier) + 0.5;
            vec3 resultColor = blend(color.rgb, detail);
            color = mix(color, vec4(resultColor, 1.0), maskValue * strength);
        }
    }

    gl_FragColor = color;
}
