#define sTexture inputImageTexture
#define u_Texture1 inputImageTexture2
#define u_Texture2 inputImageTexture3
#define u_Texture3 inputImageTexture4
#define vTexCoord textureCoordinate

precision highp float;

uniform sampler2D sTexture;
uniform sampler2D u_Texture1;
uniform sampler2D u_Texture2;
uniform sampler2D u_Texture3;
uniform sampler2D matteMaskTexture;
uniform sampler2D highlightMaskTexture;

varying vec2 vTexCoord;

vec3 blendSoftLight(in vec3 base, in vec3 blend) {
    vec3 above = sqrt(base) * (2.0 * blend - 1.0) + 2.0 * base * (1.0 - blend);
    vec3 below = 2.0 * base * blend + base * base * (1.0 - 2.0 * blend);
    return mix(below, above, step(0.5, blend));
}

vec3 highlight_inner(in vec3 inputColor, in vec3 coarseGuided, float intensity) {
    vec3 highlight = pow(inputColor, vec3(0.8));
    vec3 outputColor = blendSoftLight(highlight, coarseGuided);
    outputColor = clamp(outputColor, 0.0, 1.0);
    return mix(inputColor, outputColor, intensity);
}

vec3 bandScaling(vec3 details, float largeScale, float smallScale, float threshold) {
    return mix(details * smallScale, details * largeScale,
    smoothstep(0.8 * threshold, 1.2 * threshold, details));
}
vec3 bandScaling(vec3 details,
float negativeLargeScale, float negativeSmallScale,
float positiveLargeScale, float positiveSmallScale,
float negativeThreshold, float positiveThreshold) {
    return mix(-bandScaling(-details, negativeLargeScale, negativeSmallScale, negativeThreshold),
    bandScaling(details, positiveLargeScale, positiveSmallScale, positiveThreshold),
    step(0.0, details));
}

void main() {
    vec4 inputColor = texture2D(sTexture, vTexCoord);
    vec3 fineGaussian = texture2D(u_Texture1, vTexCoord).rgb;
    vec3 fineGuided = texture2D(u_Texture2, vTexCoord).rgb;
    vec3 coarseGuided = texture2D(u_Texture3, vTexCoord).rgb;
    vec4 matteMaskColor = texture2D(matteMaskTexture, vTexCoord);
    vec4 hihglightMaskColor = texture2D(highlightMaskTexture, vTexCoord);

    vec3 detailsI0 = inputColor.rgb - fineGaussian;
    vec3 details01 = fineGaussian - fineGuided;
    vec3 details12 = fineGuided - coarseGuided;

    vec3 outputColor = coarseGuided;

    outputColor += bandScaling(details12, 0.85, 1.0, 0.25, 1.0, 0.0015, 0.0015);
    outputColor += bandScaling(details01, 0.85, 1.0, 0.25, 1.0, 0.0015, 0.0015);
    outputColor += bandScaling(detailsI0, 0.95, 1.0, 0.75, 1.0, 0.0030, 0.0030);

    outputColor = clamp(outputColor, 0.0, 1.0);
    outputColor = mix(inputColor.rgb, outputColor, matteMaskColor.a);

    outputColor = highlight_inner(outputColor, coarseGuided, hihglightMaskColor.a);

    gl_FragColor = vec4(outputColor, inputColor.a);
}