precision highp float;
varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;       //brightness
uniform sampler2D inputImageTexture3;       //contrast
uniform sampler2D inputImageTexture4;       //saturation
uniform sampler2D inputImageTexture5;       //tem
uniform sampler2D inputImageTexture6;       //tone

mat3 matRGBtoXYZ = mat3(
    0.4124564390896922, 0.21267285140562253, 0.0193338955823293,
    0.357576077643909, 0.715152155287818, 0.11919202588130297,
    0.18043748326639894, 0.07217499330655958, 0.9503040785363679
);


mat3 matXYZtoRGB = mat3(
    3.2404541621141045, -0.9692660305051868, 0.055643430959114726,
    -1.5371385127977166, 1.8760108454466942, -0.2040259135167538,
    -0.498531409556016, 0.041556017530349834, 1.0572251882231791
);

mat3 matAdapt = mat3(
    0.8951, -0.7502, 0.0389,
    0.2664, 1.7135, -0.0685,
    -0.1614, 0.0367, 1.0296
);

mat3 matAdaptInv = mat3(
    0.9869929054667123, 0.43230526972339456, -0.008528664575177328,
    -0.14705425642099013, 0.5183602715367776, 0.04004282165408487,
    0.15996265166373125, 0.0492912282128556, 0.9684866957875502
);

vec3 refWhite;
vec3 refWhiteRGB;

vec3 d;
vec3 s;

vec3 RGBtoXYZ(vec3 rgb){
   vec3 xyz;
   vec3 XYZ;

    xyz = matRGBtoXYZ * rgb;

    // adaption
    XYZ = matAdapt * xyz;
    XYZ *= d/s;
    xyz = matAdaptInv * XYZ;

    return xyz;
}

vec3 XYZtoRGB(vec3 xyz){
   vec3 rgb;
   vec3 RGB;

    // adaption
    RGB = matAdapt * xyz;
    rgb *= s/d;
    xyz = matAdaptInv * RGB;

    rgb = matXYZtoRGB * xyz;

    return rgb;
}

float Lum(vec3 c){
    return 0.299*c.r + 0.587*c.g + 0.114*c.b;
}

vec3 ClipColor(vec3 c){
    float l = Lum(c);
    float n = min(min(c.r, c.g), c.b);
    float x = max(max(c.r, c.g), c.b);

    if (n < 0.0) c = (c-l)*l / (l-n) + l;
    if (x > 1.0) c = (c-l) * (1.0-l) / (x-l) + l;

    return c;
}

vec3 SetLum(vec3 c, float l){
    float d = l - Lum(c);

    c.r = c.r + d;
    c.g = c.g + d;
    c.b = c.b + d;

    return ClipColor(c);
}

// illuminants
//vec3 A = vec3(1.09850, 1.0, 0.35585);
vec3 D50 = vec3(0.96422, 1.0, 0.82521);
vec3 D65 = vec3(0.95047, 1.0, 1.08883);
//vec3 D75 = vec3(0.94972, 1.0, 1.22638);


//vec3 D50 = vec3(0.981443, 1.0, 0.863177);
//vec3 D65 = vec3(0.968774, 1.0, 1.121774);

vec3 CCT2K = vec3(1.274335, 1.0, 0.145233);
vec3 CCT4K = vec3(1.009802, 1.0, 0.644496);
vec3 CCT20K = vec3(0.995451, 1.0, 1.886109);



vec3 depremultiply(vec4 color)
{
    return color.rgb / (color.a + step(color.a, 0.0));
}

vec4 premultiply(vec3 color, float a)
{
    return vec4(color * a, a);
}

float getIntensity(sampler2D tex)
{
    vec4 c = texture2D(tex, textureCoordinate);
    return c.r - c.g;
}

vec4 useBrightness(vec4 baseColor, float intensity)
{
    if (intensity < 0.0) {
        vec4 res = baseColor + (baseColor + vec4(1.0) * 0.3) * intensity * 0.4;
        return vec4(res.rgb, 1.0);
    } else {
        vec4 res = baseColor + (baseColor + vec4(1.0) * 0.4) * intensity * 0.4;
        return vec4(res.rgb, 1.0);
    }
}

vec4 useContrast(vec4 baseColor, float intensity)
{
    vec4 mid =vec4(0.5, 0.5, 0.5, 1.0);
    if (intensity < 0.0){
        float intensity_ = 1.0 - intensity * 0.9;
        vec4 res = mid + (baseColor - mid) / intensity_;
        return vec4(res.rgb, 1.0);
    } else {
        float intensity_ = 1.0 + intensity * 0.9;
        vec4 res = mid + (baseColor - mid) * intensity_;
        return vec4(res.rgb, 1.0);
    }
}

vec4 useSaturation(vec4 baseColor, float intensity)
{
    vec3 luminance_w = vec3(0.2125, 0.7154, 0.0721);
    float luminance = dot(baseColor.rgb, luminance_w);
    vec4 mid =vec4(luminance, luminance, luminance, 1.0);
    if (intensity < 0.0){
        float intensity_ = 1.0 - intensity * 1.2;
        vec4 res = mid + (baseColor - mid) / intensity_;
        return vec4(res.rgb, 1.0);
    } else {
        float intensity_ = 1.0 + intensity * 1.2;
        vec4 res = mid + (baseColor - mid) * intensity_;
        return vec4(res.rgb, 1.0);
    }
}

vec4 useWhiteBalance(vec4 col, float temperatureIntensity, float toneIntensity)
{
    float temperature = 5000.0;
    if (temperatureIntensity < 0.0) {
        temperature += temperatureIntensity * 500.0;
        temperature = 0.0004 * (temperature - 5000.0);
    } else {
        temperature += temperatureIntensity * 5000.0;
        temperature = 0.00006 * (temperature - 5000.0);
    }

    float tint = toneIntensity * 0.9;
    //
    //    const highp vec3 warmFilter = vec3(0.93, 0.54, 0.0);
    //
    //    const mediump mat3 RGBtoYIQ = mat3(0.299, 0.587, 0.114, 0.596, -0.274, -0.322, 0.212, -0.523, 0.311);
    //    const mediump mat3 YIQtoRGB = mat3(1.0, 0.956, 0.621, 1.0, -0.272, -0.647, 1.0, -1.105, 1.702);
    //
    //    mediump vec3 yiq = RGBtoYIQ * color.rgb;
    //    yiq.b = clamp(yiq.b + tint*0.5226*0.1, -0.5226, 0.5226);
    //    lowp vec3 rgb = YIQtoRGB * yiq;
    //
    //    lowp vec3 processed = vec3(
    //        (rgb.r < 0.5 ? (2.0 * rgb.r * warmFilter.r) : (1.0 - 2.0 * (1.0 - rgb.r) * (1.0 - warmFilter.r))),
    //        (rgb.g < 0.5 ? (2.0 * rgb.g * warmFilter.g) : (1.0 - 2.0 * (1.0 - rgb.g) * (1.0 - warmFilter.g))),
    //        (rgb.b < 0.5 ? (2.0 * rgb.b * warmFilter.b) : (1.0 - 2.0 * (1.0 - rgb.b) * (1.0 - warmFilter.b))));
    //
    //    return vec4(mix(rgb, processed, temperature), color.a);

    vec3 to;
    vec3 from;

    if (temperature < 0.0) {
        to = CCT20K;
        from = D65;
    } else {
        to = CCT4K;
        from = D65;
    }

    vec3 base = col.rgb;
    float lum = Lum(base);
    // mask by luminance
    float temp = abs(temperature) * (1.0 - pow(lum, 2.72));

    // from
    refWhiteRGB = from;
    // to
    refWhite = vec3(mix(from.x, to.x, temp), mix(1.0, 0.9, tint), mix(from.z, to.z, temp));

    // mix based on alpha for local adjustments
    refWhite = mix(refWhiteRGB, refWhite, col.a);

    d = matAdapt * refWhite;
    s = matAdapt * refWhiteRGB;
    vec3 xyz = RGBtoXYZ(base);
    vec3 rgb = XYZtoRGB(xyz);
    // brightness compensation
    vec3 res = rgb * (1.0 + (temp + tint) / 10.0);
    // preserve luminance
    //vec3 res = SetLum(rgb, lum);

    return vec4(mix(base, res, col.a), col.a);
}

void main()
{
    vec4 color = texture2D(inputImageTexture, textureCoordinate);
    float alpha = color.a;
    color = vec4(depremultiply(color), 1.0);

    float intensity = getIntensity(inputImageTexture2);
    if (abs(intensity) > 1e-3) {
        color = useBrightness(color, intensity);
    }

    intensity = getIntensity(inputImageTexture3);
    if (abs(intensity) > 1e-3) {
        color = useContrast(color, intensity);
    }

    intensity = getIntensity(inputImageTexture4);
    if (abs(intensity) > 1e-3) {
        color = useSaturation(color, intensity);
    }

    intensity = getIntensity(inputImageTexture5);
    float intensity2 = getIntensity(inputImageTexture6);
    if (abs(intensity) > 1e-3 || abs(intensity2) > 1e-3) {
        color = useWhiteBalance(color, intensity, intensity2);
    }

    gl_FragColor = premultiply(color.rgb, alpha);
}