precision highp float;

varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform vec2 baseSize;
uniform vec2 sucaiSize;

uniform int pointCount;

uniform vec2 pos[8];
uniform float intensity[8];
uniform float scaleRate[8];

vec3 depremultiply(vec4 color) {
    return color.rgb / (color.a + step(color.a, 0.0));
}

vec4 premultiply(vec3 color, float a) {
    return vec4(color * a, a);
}

vec2 sucaiUV(vec2 anchorPos, vec2 srcPos, vec2 realSucaiSize) {
    vec2 xy = (srcPos - anchorPos) + realSucaiSize * 0.5;
    vec2 uv = xy / realSucaiSize;
    return clamp(uv, 0.0, 1.0);
}

void main()
{
    vec3 srcColor = depremultiply(texture2D(inputImageTexture, textureCoordinate));

    float positiveRes = 0.0;
    float negativeRes = 0.0;

    vec2 currPos = vec2(0.0);
    float currIntensity = 0.0;
    vec2 currRealSucaiSize = vec2(0.0);
    for (int i = 0; i < pointCount; i++) {
        currPos = pos[i];
        currIntensity = intensity[i];
        currRealSucaiSize = sucaiSize * scaleRate[i];

        float sucaiA = texture2D(inputImageTexture2, sucaiUV(currPos * baseSize, textureCoordinate * baseSize, currRealSucaiSize)).r;
        vec3 posColor = depremultiply(texture2D(inputImageTexture, currPos));
        float delta = min(distance(srcColor, posColor), 1.0);
        float res = (1.0 - delta) * currIntensity * sucaiA;

        if (res >= 0.0) {
            positiveRes = 1.0 - (1.0 - positiveRes) * (1.0 - res);
        } else {
            negativeRes = (1.0 + negativeRes) * (1.0 + res) - 1.0;
        }
    }
    float res = positiveRes + negativeRes;
    gl_FragColor = vec4(clamp(res, 0.0, 1.0), clamp(-res, 0.0, 1.0), 0.0, 1.0);
}
