#define KNOTS_N 7
precision highp float;

varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;

uniform float xs[KNOTS_N];
uniform float ys_max[KNOTS_N];
//uniform float ys[KNOTS_N];
//uniform float ms[KNOTS_N];

float ys[KNOTS_N];
float ms[KNOTS_N];

/// 计算ms
void calc() {
    int n = KNOTS_N;
    float x[KNOTS_N];
    float y[KNOTS_N];
    for (int i = 0; i < n; i++) {
        x[i] = xs[i];
        y[i] = ys[i];
    }
    float m[KNOTS_N];
    float delta[KNOTS_N];
    for (int i = 0; i < n - 1; i++) {
        delta[i] = (y[i + 1] - y[i]) / (x[i + 1] - x[i]);
        if (i > 0) {
            m[i] = (delta[i - 1] + delta[i]) / 2.0;
        }
    }
    m[0] = delta[0];
    m[n - 1] = delta[n - 2];

    int to_fix[KNOTS_N];
    int to_fix_index = 0;
    for (int i = 0; i < n - 1; i++) {
        if (delta[i] == 0.0) {
            to_fix[to_fix_index] = i;
            to_fix_index++;
        }
    }
    for (int fi = 0; fi < to_fix_index; fi++) {
        int i = to_fix[fi];
        m[i] = m[i + 1] = 0.0;
    }
    float alpha[KNOTS_N];
    float beta[KNOTS_N];
    float dist[KNOTS_N];
    float tau[KNOTS_N];
    for (int i = 0; i < n - 1; i++) {
        alpha[i] = m[i] / delta[i];
        beta[i] = m[i + 1] / delta[i];
        dist[i] = pow(alpha[i], 2.0) + pow(beta[i], 2.0);
        tau[i] = 3. / sqrt(dist[i]);
    }

    to_fix[n-1] = 0;
    to_fix_index = 0;
    for (int i = 0; i < n - 1; i++) {
        to_fix[i] = 0;
        if (dist[i] > 9.0) {
            to_fix[to_fix_index] = i;
            to_fix_index++;
        }
    }
    for (int fi = 0; fi < to_fix_index; fi++) {
        int i = to_fix[fi];
        m[i] = tau[i] * alpha[i] * delta[i];
        m[i + 1] = tau[i] * beta[i] * delta[i];
    }

    for (int i = 0; i < n; i++) {
        ms[i] = m[i];
    }
}

vec3 hsv2rgb(in vec3 c)
{
    vec3 rgb = clamp(abs(mod(c.x*6.0+vec3(0.0, 4.0, 2.0), 6.0)-3.0)-1.0, 0.0, 1.0);
    return c.z * mix(vec3(1.0), rgb, c.y);
}

vec3 rgb2hsv(in vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

float interpolate(float x) {
    int i = KNOTS_N - 2;
    for (; i >= 0; i--) {
        if (xs[i] <= x) {
            break;
        }
    }
    float h = xs[i + 1] - xs[i];
    float t = (x - xs[i]) / h;
    float t2 = pow(t, 2.);
    float t3 = pow(t, 3.);
    float h00 = 2. * t3 - 3. * t2 + 1.;
    float h10 = t3 - 2. * t2 + t;
    float h01 = -2. * t3 + 3. * t2;
    float h11 = t3 - t2;
    return h00 * ys[i] + h10 * h * ms[i] + h01 * ys[i + 1] + h11 * h * ms[i + 1];;
}

vec3 deVibrance(vec3 color, float intensity)
{
    float lumiance = dot(color, vec3(0.3, 0.6, 0.1));
    return mix(vec3(lumiance), color, intensity);
}

float getIntensity(sampler2D tex)
{
    vec4 c = texture2D(tex, textureCoordinate);
    return c.r - c.g;
}

void main() {
    vec4 srcColor = texture2D(inputImageTexture, textureCoordinate);

    vec3 outputRGB = srcColor.rgb;

    float intensity = getIntensity(inputImageTexture3);
    if (abs(intensity) > 1e-3) {
        if (intensity < 0.) {
            outputRGB = deVibrance(srcColor.rgb, 0.7*intensity+1.0);
        } else if (intensity > 0.) {

            // mode 1
            //            vec3 VibranceRGBBalance = vec3(1.);
            //            vec3 color = srcColor.rgb;
            //            vec3 coefLuma = vec3(0.212656, 0.715158, 0.072186);
            //            /*
            //        if (Vibrance_Luma)
            //            coefLuma = float3(0.333333, 0.333334, 0.333333);
            //        */
            //            float luma = dot(coefLuma, color);
            //            float max_color = max(color.r, max(color.g, color.b));// Find the strongest color
            //            float min_color = min(color.r, min(color.g, color.b));// Find the weakest color
            //            float color_saturation = max_color - min_color;// The difference between the two is the saturation
            //            // Extrapolate between luma and original by 1 + (1-saturation) - current
            //            vec3 coeffVibrance = vec3(VibranceRGBBalance * intensity);
            //            float r = mix(luma, color.r, 1.0 + (coeffVibrance.r * (1.0 - (sign(coeffVibrance.r) * color_saturation))));
            //            float g = mix(luma, color.g, 1.0 + (coeffVibrance.g * (1.0 - (sign(coeffVibrance.g) * color_saturation))));
            //            float b = mix(luma, color.b, 1.0 + (coeffVibrance.b * (1.0 - (sign(coeffVibrance.b) * color_saturation))));
            //            vec3 blur = texture2D(inputImageTexture2, textureCoordinate).rgb;
            //            float B = max(0., blur.r - blur.g);
            //            float C = B * B * 100.;
            //            C = max(0., C);
            //            C = min(1., C);
            //            outputRGB = mix(vec3(r, g, b), srcColor.rgb, C * 0.5);


            // mode 3

            // mode 7
            vec3 blur = texture2D(inputImageTexture2, textureCoordinate).rgb;
            float B = max(0., blur.r - blur.g);
            float C = B * B * 100.;
            C = max(0., C);
            C = min(1., C);

            vec3 rgb = srcColor.rgb;
            vec3 hsv = rgb2hsv(rgb);
            hsv.y += hsv.y*(1.0-hsv.y)*intensity;
            hsv.z = clamp(hsv.z + pow(max(0.0, intensity - 0.3), 2.0) * 0.078, 0.0, 1.0);
            outputRGB = mix(hsv2rgb(hsv), rgb, C * 0.65);

            // mode 5
            //            vec3 blur = texture2D(inputImageTexture2, textureCoordinate).rgb;
            //            float B = max(0., blur.r - blur.g);
            //            float C = B * B * 100.;
            //            C = max(0., C);
            //            C = min(1., C);
            //
            //            // darktable
            //            vec3 rgb = srcColor.rgb;
            //            vec3 lab = rgb2lab(rgb);
            //            float ll = lab.r;
            //            float la = lab.g;
            //            float lb = lab.b;
            //
            //            float amount = intensity*2.5;
            //            float sw = sqrt(la*la+lb*lb)/256.;
            //            float ls = 1.0 - amount*sw*0.25;
            //            float ss = 1.0 + amount*sw;
            //
            //            lab.r = max(lab.r, lab.r*ls);
            //            lab.g *= ss;
            //            lab.b *= ss;
            //
            //            outputRGB = lab2rgb(lab);

            // mode 4
            //            vec3 blur = texture2D(inputImageTexture2, textureCoordinate).rgb;
            //            float B = max(0., blur.r - blur.g);
            //            float C = B * B * 100.;
            //            C = max(0., C);
            //            C = min(1., C);
            //
            //            // gpuimage
            //            vec4 color = srcColor;
            //            float average = (color.r + color.g + color.b) / 3.0;
            //            float mx = max(color.r, max(color.g, color.b));
            //            float amt = (mx - average) * (-intensity * 3.0);
            //            //            amt = ((abs(mx - average) * 2.0 / 255.0) * intensity * 3.0) / 100.0;
            //            vec3 vibCol = mix(color.rgb, vec3(mx), amt);
            //            vec3 hsv = rgb2hsv(vibCol);
            //            hsv.z += intensity * 20./255. * hsv.y;
            //            outputRGB = mix(hsv2rgb(hsv), srcColor.rgb, C * 0.5);

            // mode 6
            //            const float Rf = 0.2999;
            //            const float Gf = 0.587;
            //            const float Bf = 0.114;
            //
            //            float Vib = intensity;
            //
            //            vec3 rgb = srcColor.rgb;
            //            float r = rgb.r;
            //            float g = rgb.g;
            //            float b = rgb.b;
            //            float red = (r-max(g, b)) * (1. / 256.);
            //            float S = Vib/(1.+ exp(-red*3.))+1.;
            //            float MS = 1. - S;
            //            float Rt = Rf * MS;
            //            float Gt = Gf * MS;
            //            float Bt = Bf * MS;
            //            float t = (r + g) * 2.;
            //            float R = r;
            //            float G = g;
            //            float B = b;
            //            float Rc = R * (Rt + S) + G * Gt + B * Bt;
            //            float Gc = R * Rt + G * (Gt + S) + B * Bt;
            //            float Bc = R * Rt + G * Gt + B * (Bt + S);
            //
            //            vec3 blur = texture2D(inputImageTexture2, textureCoordinate).rgb;
            //            float BB = max(0., blur.r - blur.g);
            //            float C = BB * BB * 100.;
            //            C = max(0., C);
            //            C = min(1., C);
            //
            //            outputRGB = mix(vec3(Rc, Gc, Bc), srcColor.rgb, C * 0.5);

            //            for (int i = 0; i < KNOTS_N; i++) {
            //                ys[i] = xs[i] + (ys_max[i] - xs[i]) * intensity;
            //            }
            //            calc();
            //            vec3 blur = texture2D(inputImageTexture2, textureCoordinate).rgb;
            //            float bb = max(0., blur.r - blur.g);
            //            float cc = bb * bb * 100.;
            //            cc = max(0., cc);
            //            cc = min(1., cc);
            //            vec3 hsv = rgb2hsv(srcColor.rgb);
            //            float s = hsv.y;
            //            float ss = interpolate(s*255.)/255.;
            //            hsv.y = ss;
            //            hsv.z += intensity * 20./255. * ss;
            //            outputRGB = mix(hsv2rgb(hsv), srcColor.rgb, cc * 0.65);
        }
    }

    gl_FragColor = vec4(outputRGB, srcColor.a);
}
