
precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform sampler2D inputImageTexture4;


vec3 hardlight(vec4 base, vec4 overlay) {
    float ra;
    if (2.0 * overlay.r < overlay.a) {
        ra = 2.0 * overlay.r * base.r + overlay.r * (1.0 - base.a) + base.r * (1.0 - overlay.a);
    } else {
        ra = overlay.a * base.a - 2.0 * (base.a - base.r) * (overlay.a - overlay.r) + overlay.r * (1.0 - base.a) + base.r * (1.0 - overlay.a);
    }
    
    float ga;
    if (2.0 * overlay.g < overlay.a) {
        ga = 2.0 * overlay.g * base.g + overlay.g * (1.0 - base.a) + base.g * (1.0 - overlay.a);
    } else {
        ga = overlay.a * base.a - 2.0 * (base.a - base.g) * (overlay.a - overlay.g) + overlay.g * (1.0 - base.a) + base.g * (1.0 - overlay.a);
    }
    
    float ba;
    if (2.0 * overlay.b < overlay.a) {
        ba = 2.0 * overlay.b * base.b + overlay.b * (1.0 - base.a) + base.b * (1.0 - overlay.a);
    } else {
        ba = overlay.a * base.a - 2.0 * (base.a - base.b) * (overlay.a - overlay.b) + overlay.b * (1.0 - base.a) + base.b * (1.0 - overlay.a);
    }
    
    return vec3(ra, ga, ba);
}

vec4 blend(vec4 base, vec4 overlay) {
    return vec4(hardlight(overlay, base), base.a + overlay.a - base.a * overlay.a);
}

void main()
{
    float b1 = texture2D(inputImageTexture, textureCoordinate).b;
    float b2 = texture2D(inputImageTexture2, textureCoordinate).b;
    vec4 color = vec4(vec3((b1 - b2) * 0.5 + 0.5), 1.0);
    for (int i = 0; i < 5; i++) {
        color = blend(color, color);
    }
    color = vec4(min(color.rgb, 0.5), 1.0);
    vec3 mask = texture2D(inputImageTexture3, textureCoordinate).rgb;
    float intensity = 1.0 - mask.b;
    intensity = pow(intensity, 4.0);
    intensity *= mask.g;
    intensity *= texture2D(inputImageTexture4, textureCoordinate).r;
    color = mix(vec4(0.5), color, intensity);
    
    float r = 1.0 - color.r * 2.0;
    gl_FragColor = vec4(step(0.38, r));
    
//    gl_FragColor = vec4(color.rgb, 1.0);
}
