precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform float strength;

mat4 contrastMatrix( float contrast )
{
    float t = ( 1.0 - contrast ) / 2.0;

    return mat4( contrast, 0.0, 0.0, 0.0,
    0.0, contrast, 0.0, 0.0,
    0.0, 0, contrast, 0.0,
    t, t, t, 1.0 );

}

void main(){

    //origin
    vec3 origin = vec3(350./360., 390./360., 430./360.);

    vec4 color = texture2D(inputImageTexture, textureCoordinate);

    color.rg = color.rg * 15. / 16. + vec2(1. / 32.);
    float blueColor = color.b * 63.0;

    vec2 quad1;
    quad1.y = floor(floor(blueColor) / 8.0);
    quad1.x = floor(blueColor) - (quad1.y * 8.0);
    vec2 quad2;
    quad2.y = floor(ceil(blueColor) / 8.0);
    quad2.x = ceil(blueColor) - (quad2.y * 8.0);

    vec2 texPos1;
    texPos1.x = (quad1.x * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * color.r);
    texPos1.y = (quad1.y * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * color.g);
    vec2 texPos2;
    texPos2.x = (quad2.x * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * color.r);
    texPos2.y = (quad2.y * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * color.g);
    vec4 newColor1 = texture2D(inputImageTexture2, texPos1);
    vec4 newColor2 = texture2D(inputImageTexture2, texPos2);
    vec4 newColor = mix(newColor1, newColor2, fract(blueColor));
    newColor.rg = (contrastMatrix(16. / 15.) * newColor).rg;
    color.rg = (contrastMatrix(16. / 15.) * color).rg;

    float a = texture2D(inputImageTexture3, textureCoordinate).a;

    gl_FragColor = mix( color, newColor, min(a * strength, 1.));
}
