precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform float skinStrength;


//void main(){
//    vec4 color = texture2D(inputImageTexture, st);
//    vec4 skin = texture2D(inputImageTexture2, st);
//    gl_FragColor = mix(color, skin, skinStrength * skin.a);
////    gl_FragColor = skin;
//}

float eachcomponent(float Cb, float Cs) {
    if (Cs <= 0.5) {
        return Cb - (1.0 - 2.0 * Cs) * Cb * (1.0 - Cb);
    } else {
        float zeus = 0.0;
        if (Cb <= 0.25) {
            zeus = ((16.0 * Cb - 12.0) * Cb + 4.0) * Cb;
        } else {
            zeus = sqrt(Cb);
        }
        return Cb + (2.0 * Cs - 1.0) * (zeus - Cb);
    }
}

vec3 blend(vec4 base, vec4 overlay) {
    vec3 baseRGB = base.rgb;
    vec3 overlayRGB = overlay.rgb;
    float r = eachcomponent(baseRGB.r, overlayRGB.r);
    float g = eachcomponent(baseRGB.g, overlayRGB.g);
    float b = eachcomponent(baseRGB.b, overlayRGB.b);
    vec3 color = clamp(vec3(r, g, b), 0.0, 1.0);
    return (1.0 - overlay.a) * base.rgb + (1.0 - base.a) * overlay.rgb + base.a * overlay.a * color;
}

void main() {
    vec4 texture1Color = texture2D(inputImageTexture, textureCoordinate);

    vec4 outputColor = texture1Color;

    vec4 texture2Color = texture2D(inputImageTexture2, textureCoordinate);
    texture2Color.a = texture2Color.a * skinStrength;

    float alpha = texture1Color.a + texture2Color.a - texture1Color.a * texture2Color.a;
    vec3 color = blend(texture1Color, texture2Color);
    outputColor = vec4(color, alpha);

    gl_FragColor = outputColor;
}