precision highp float;

varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform vec2 resolution;
uniform vec2 center;
uniform float radius;
uniform float intensity;

#func_data2offset
#func_offset2data

// Breast
vec2 breast_inner(vec2 uv, vec2 center, float radius, float scale) {
    vec2 res = uv;
    res -= center;
    float ratio = 0.95;
    res.y *= ratio;

    float innerScale = scale * 0.25;
    float m = radius * 0.5;
    float d = length(res);
    if (d < radius) {
        if (d < radius - m) {
            res *= 1. - innerScale;
        } else {
            float p = (radius - d) / m;
            p = smoothstep(0., 1., p);
            res *= 1. - p * innerScale;
        }
    }

    res.y /= ratio;
    res += center;
    return res - uv;
}

vec2 process(vec2 coor, float width, float height) {
    vec2 coor_res;
    vec2 vec_record = vec2(0., 0.);

    // Breast丰胸
    float br_scale = intensity;
    if (br_scale != 0.0) {
        vec2 dd = vec2(0.0);

        vec2 br_center = center * resolution;
        float br_radius = radius * resolution.x;
        dd += breast_inner(coor, br_center, br_radius, br_scale);

        coor_res = coor + dd;
        vec_record += coor_res - coor;
        coor = coor_res;
    }

    return vec_record;
}

void main() {
    vec2 coor = textureCoordinate;

    float width = resolution.x;
    float height = resolution.y;

    coor *= resolution;

    coor += process(coor, width, height);

    coor /= resolution;

    vec2 offset = coor - textureCoordinate;
    vec2 inOffset = data2offset(texture2D(inputImageTexture, textureCoordinate));
    gl_FragColor = offset2data(inOffset + offset);
}
