precision highp float;

varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform vec2 resolution;
uniform vec2 center;
uniform float radian;
uniform float width;
uniform float height;
uniform float intensityL;
uniform float intensityR;
uniform float liftIntensity;

float drag2(vec2 uv, vec2 center, float maxdx, float maxdy, float max_offset, float scale) {
    float offset = 0.;
    float dx = abs(uv.x - center.x);
    float dy = abs(uv.y - center.y);
    if (dx < maxdx && dy < maxdy) {
        float px = (maxdx - dx) / maxdx;
        px = smoothstep(0., 1., px);
        float py = (maxdy - dy) / maxdy;
        py = smoothstep(0., 1., py);
        offset -= px * py * max_offset;
        //        if (scale == 1.0 || scale == -1.0) {
        //            offset = -dx;
        //        }
    }
    return offset * scale;
}

float drag(float x, float centerx, float maxd, float max_offset, float scale) {
    float offset = 0.;
    float d = abs(x - centerx);
    if (d < maxd) {
        float p = (maxd - d) / maxd;
        //p *= p * p;
        p = smoothstep(0., 1., p);
        offset = -p * max_offset;
        //        if (scale == 1.0 || scale == -1.0) {
        //            offset = -x;
        //        }
    }
    return offset * scale;
}

vec2 rotate(vec2 src, vec2 anchor, float angle) {
    src -= vec2(anchor.x, anchor.y);
    return (src*mat2(cos(angle), -sin(angle), sin(angle), cos(angle)))+vec2(anchor.x, anchor.y);
}

vec2 hip_inner(vec2 coor, vec2 center, float w, float h, float radian, float scaleL, float scaleR, float liftScale) {
    vec2 res = coor;
    //res = spring_scale(res, center, w, w * 0.5, scale * 0.06, 0.);

    res = rotate(res, center, -radian);

    float quarterw = w * 0.25;
    float halfh = h * 0.5;

    float dx = 0.;
    dx += drag2(res, vec2(center.x + quarterw, center.y), quarterw, halfh, quarterw * 0.1, scaleR);
    dx -= drag2(res, vec2(center.x - quarterw, center.y), quarterw, halfh, quarterw * 0.1, scaleL);
    res.x += dx;

    res = rotate(res, center, radian);

    res.y += drag(res.y, center.y - h * 0.25, abs(h), h * 0.09, abs(liftScale));
    return res;
}

void main() {
    vec2 coor = textureCoordinate;
    coor = coor * resolution;
    if (intensityL != 0.0|| intensityR != 0.0 || liftIntensity != 0.0) {
        coor = hip_inner(coor, center, width, height, radian, intensityL, intensityR, liftIntensity);
    }
    coor = coor / resolution;
    gl_FragColor = texture2D(inputImageTexture, coor);
}
