precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform vec2 topLeft;
uniform vec2 bottomRight;
uniform float radian;
uniform float intensity;
uniform float aspectRatio;

const float PI = 3.14159;

//旋转中心，纹理比例，旋转弧度
vec2 rotate(vec2 uv, vec2 center, float ratio, float rotateRadian){
    uv -= center;
    uv.x *= ratio;
    uv = vec2(uv.x * cos(rotateRadian) - uv.y * sin(rotateRadian), uv.x * sin(rotateRadian) + uv.y * cos(rotateRadian));
    uv.x /= ratio;
    uv += center;
    return uv;
}

float drag_box2(vec2 uv, vec2 center, float left_margin, float right_margin, float top_margin, float bottom_margin, float max_offset, float scale) {
    float offset = 0.;
    float dx = abs(uv.x - center.x);
    float dy = abs(uv.y - center.y);
    float maxdx = mix(left_margin, right_margin, step(center.x, uv.x));
    float maxdy = mix(top_margin, bottom_margin, step(center.y, uv.y));
    if (dx < maxdx && dy < maxdy) {
        float px = (maxdx - dx) / maxdx;
        px = 1. - (1. - px) * (1. - px);
        px = smoothstep(0., 1., px);
        float py = (maxdy - dy) / maxdy;
        py = 1. - (1. - py) * (1. - py);
        py = smoothstep(0., 1., py);
        offset -= px * py * max_offset;
    }
    return offset * scale;
}

void main(){
    vec2 uv = textureCoordinate;
    vec2 center = (topLeft + bottomRight) / 2.0;

    uv.y = 1.0 - uv.y;

    uv = rotate(uv, center, aspectRatio, -radian);
    vec2 rotate_topLeft = rotate(topLeft, center, aspectRatio, -radian);
    vec2 rotate_bottomRight = rotate(bottomRight, center, aspectRatio, -radian);

    float w = rotate_bottomRight.x - rotate_topLeft.x;
    float h = rotate_bottomRight.y - rotate_topLeft.y;
    uv.x += drag_box2(uv, vec2(rotate_topLeft.x, center.y), w * 0.3, w * 0.5, h * 0.55, h * 0.55, w * 0.05, intensity * 1.5);
    uv.x -= drag_box2(uv, vec2(rotate_bottomRight.x, center.y), w * 0.5, w * 0.3, h * 0.55, h * 0.55, w * 0.05, intensity * 1.5);

    uv = rotate(uv, center, aspectRatio, radian);

    uv.y = 1.0 - uv.y;

    gl_FragColor = texture2D(inputImageTexture, uv);
}
