#define PI 3.1415926

precision highp float;

varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform vec2 resolution;

#define kMaxBodySize 3

// Neck天鹅颈
uniform float neck_center[kMaxBodySize * 6];
uniform float neck_offset[kMaxBodySize * 6];
uniform float neck_radius[kMaxBodySize * 3];
uniform float neck_scale[kMaxBodySize];

// Hip美胯
uniform float hip_center[kMaxBodySize * 2];
uniform float hip_size_radian[kMaxBodySize * 3];
uniform float hip_scale_l[kMaxBodySize];
uniform float hip_scale_r[kMaxBodySize];
uniform float lift_scale[kMaxBodySize];


// Breast丰胸
uniform float breast_left_param[kMaxBodySize * 3];
uniform float breast_right_param[kMaxBodySize * 3];
uniform float breast_scale[kMaxBodySize];

const float debug = 0.0;

vec3 get_line_param(vec2 point1, vec2 point2)
{
    vec3 res;
    res.x = point1.y - point2.y;
    res.y = point2.x - point1.x;
    res.z = point1.x * point2.y - point2.x * point1.y;
    return res;
}

vec2 get_cross_point(vec4 line1, vec4 line2)
{
    vec3 param1 = get_line_param(line1.xy, line1.zw);
    vec3 param2 = get_line_param(line2.xy, line2.zw);
    vec2 res = vec2(-1.0, -1.0);
    float tmp_d = param1.x * param2.y - param2.x * param1.y;
    if (tmp_d != 0.)
    {
        res.x = (param1.y * param2.z - param2.y * param1.z) / tmp_d;
        res.y = (param1.z * param2.x - param2.z * param1.x) / tmp_d;
    }
    return res;
}

vec2 get_vertical_cross_point_by_vec(vec2 point, vec2 vec, vec2 target_point)
{
    vec2 res;
    vec2 other_point = point + vec;

    vec3 ABC = get_line_param(point, other_point);
    float A = ABC.x;
    float B = ABC.y;
    float C = ABC.z;
    float A2 = A * A;
    float B2 = B * B;
    float A2plusB2 = A2 + B2;

    if (A2plusB2 != 0.)
    {
        float AB = A * B;
        float AC = A * C;
        float BC = B * C;
        float inv_A2plusB2 = 1.0 / A2plusB2;
        res.x = (B2 * target_point.x - AB * target_point.y - AC) * inv_A2plusB2;
        res.y = (A2 * target_point.y - AB * target_point.x - BC) * inv_A2plusB2;
        return res;
    }
    else
    {
        return res;
    }
}

vec2 processing_inner(vec2 coor, float ori_target_x_ratio, float ori_max_x_ratio, float ori_target_y_ratio, float ori_max_y_ratio, float left, float right, float top, float bottom, float x_min, float x_max, float y_min, float y_max, vec2 center_point, vec4 cross_point_param_x, vec4 cross_point_param_y, int mode, float height, float width, int direction, int line_type, int transition_type)
{
    vec2 res = vec2(-1.0, -1.0);

    if (ori_target_x_ratio == 1.0 && ori_target_y_ratio == 1.0)
    {
        return coor;
    }
    float coor_x = coor.x, coor_y = coor.y;

    if (direction == 0)
    {
        if (coor.x <= x_min || coor.x >= x_max || coor.y <= y_min || coor.y >= y_max)
        {
            return coor;
        }
    }
    else if (direction == 1)
    {
        if (coor.y <= y_min || coor.y >= y_max)
        {
            return coor;
        }
    }
    else
    {
        if (coor.x <= x_min || coor.x >= x_max)
        {
            return coor;
        }
    }
    float target_x_ratio = ori_target_x_ratio;
    float target_y_ratio = ori_target_y_ratio;
    float max_x_ratio = ori_max_x_ratio;
    float max_y_ratio = ori_max_y_ratio;

    float top_height = top;
    float bottom_height = bottom;
    float left_width = left;
    float right_width = right;
    float x_length = -1.0;
    float y_length = -1.0;
    float normal_ratio = 1.0;

    float offset_x_ratio, offset_y_ratio, offset_x_ratio2, offset_y_ratio2;

    //    vec2 x_vec = vec2(debug_param[0][0], debug_param[0][1]);
    vec2 x_vec = vec2(1., 0.);
    vec2 x_cross_point = get_vertical_cross_point_by_vec(center_point, x_vec, coor);

    if (x_cross_point.x == -1.0)
    {
        return coor;
    }

    float offset_x = x_cross_point.x - center_point.x;

    vec2 y_vec = vec2(0.0, 1.0);
    //    vec2 y_vec = vec2(debug_param[0][2], debug_param[0][3]);
    vec2 y_cross_point = get_vertical_cross_point_by_vec(center_point, y_vec, coor);

    if (y_cross_point.x == -1.0)
    {
        return coor;
    }

    float offset_y = y_cross_point.y - center_point.y;

    vec2 mapped_vec;
    mapped_vec.x = 0.;
    mapped_vec.y = 0.;

    float tmp_width = (offset_x <= 0.) ? left_width : right_width;
    offset_x = distance(x_cross_point, center_point);
    offset_x_ratio = (tmp_width == 0.) ? 1.0 : offset_x / tmp_width;
    x_length = tmp_width;
    offset_x_ratio = offset_x_ratio > 1.0 ? 1.0 : offset_x_ratio;
    offset_x_ratio2 = offset_x_ratio * offset_x_ratio;

    float tmp_height = (offset_y <= 0.) ? top_height : bottom_height;
    offset_y = distance(y_cross_point, center_point);
    offset_y_ratio = (tmp_height == 0.) ? 1.0 : offset_y / tmp_height;
    y_length = tmp_height;
    offset_y_ratio = offset_y_ratio > 1.0 ? 1.0 : offset_y_ratio;
    offset_y_ratio2 = offset_y_ratio * offset_y_ratio;

    float tmp_max_x_ratio, tmp_max_y_ratio, x_ratio, y_ratio, length_ratio, u, v;
    vec2 uv;

    if (mode == 0 || mode == 2)
    {
        if (direction == 2)
        {
            offset_y_ratio = 0.;
            offset_y_ratio2 = 0.;
        }
        if (transition_type == 1)
        {
            tmp_max_x_ratio = target_x_ratio - (target_x_ratio - normal_ratio) * offset_y_ratio2;
        }
        else
        {
            tmp_max_x_ratio = target_x_ratio - (target_x_ratio - normal_ratio) * offset_y_ratio;
        }
        if (line_type == 1)
        {
            x_ratio = tmp_max_x_ratio - (tmp_max_x_ratio - normal_ratio) * mix(offset_x_ratio2, 1., offset_x_ratio);//
        }
        else
        {
            x_ratio = tmp_max_x_ratio - (tmp_max_x_ratio - normal_ratio) * offset_x_ratio;
        }
        x_ratio = (max_x_ratio > 1.0) ? ((x_ratio > max_x_ratio) ? max_x_ratio : x_ratio)  : ((x_ratio > max_x_ratio) ? x_ratio : max_x_ratio);
        uv = x_ratio * (coor - y_cross_point) + y_cross_point;
    }
    else
    {
        x_ratio = 1.0;
        uv = coor;
    }
    mapped_vec += uv - coor;

    if (mode == 1 || mode == 2)
    {
        if (direction == 1)
        {
            offset_x_ratio = 0.;
            offset_x_ratio2 = 0.;
        }
        if (transition_type == 1)
        {
            tmp_max_y_ratio = target_y_ratio - (target_y_ratio - normal_ratio) * offset_x_ratio2;
        }
        else
        {
            tmp_max_y_ratio = target_y_ratio - (target_y_ratio - normal_ratio) * offset_x_ratio;
        }
        if (line_type == 1)
        {
            y_ratio = tmp_max_y_ratio - (tmp_max_y_ratio - normal_ratio) * mix(offset_y_ratio2, 1., offset_y_ratio);
        }
        else
        {
            y_ratio = tmp_max_y_ratio - (tmp_max_y_ratio - normal_ratio) * offset_y_ratio;
        }
        y_ratio = (max_y_ratio > 1.0) ? ((y_ratio > max_y_ratio) ? max_y_ratio : y_ratio)  : ((y_ratio > max_y_ratio) ? y_ratio : max_y_ratio);
        uv = y_ratio * (coor - x_cross_point) + x_cross_point;
    }
    else
    {
        y_ratio = 1.0;
        uv = coor;
    }

    mapped_vec += uv - coor;

    res = mapped_vec + coor;

    return res;
}

vec2 stretch_inner(vec2 coor, float ori_target_y_ratio, float top, float bottom, vec2 center_point, float stretch_ratio)
{
    vec2 res = coor;
    float top_pos = center_point.y - top;
    if (coor.y < top_pos)
    {
        return res;
    }

    //    float bottom_pos = center_point.y + bottom;
    //    float middle = center_point.y;

    float target_y_ratio = ori_target_y_ratio;
    float top_bottom_stretch_ratio = stretch_ratio;//stretch_ratio / (stretch_ratio + 1.0);
    float total_ori_num = bottom + top;
    float total_tmp_num = total_ori_num * target_y_ratio;
    float add_num = total_tmp_num - total_ori_num;

    float top_region_ori_num = top;
    float bottom_region_ori_num = bottom;
    float accum;


    float top_region_tmp_num = add_num * top_bottom_stretch_ratio + top_region_ori_num;
    float bottom_region_tmp_num = add_num * (1.0 - top_bottom_stretch_ratio) + bottom_region_ori_num;

    float top_region_gap = 1., bottom_region_gap = 1., y_pos = 0.;
    top_region_gap = (top_region_tmp_num != 0.) ? top_region_ori_num / top_region_tmp_num : 0.;

    bottom_region_gap = (bottom_region_tmp_num != 0.) ? bottom_region_ori_num / bottom_region_tmp_num : 0.;


    float state_1 = (top_region_tmp_num + top_pos) < coor.y ? 1.0 : 0.;
    float state_2 = (top_region_tmp_num + bottom_region_tmp_num + top_pos) < coor.y ? 1.0 : 0.;
    accum = top_pos + top_region_ori_num * state_1 + bottom_region_ori_num * state_2;
    float start_region = top_pos + top_region_tmp_num * state_1 + bottom_region_tmp_num * state_2;
    float tmp_gap;

    if (state_2 == 1.0)
    {
        tmp_gap = 1.0;
    }
    else if (state_1 == 1.0)
    {
        tmp_gap = bottom_region_gap;
    }
    else
    {
        tmp_gap = top_region_gap;
    }

    y_pos = tmp_gap * (coor.y - start_region) + accum;

    res.y = y_pos;
    return res;
}

// Neck
vec2 neck_inner(vec2 coor, vec2 center, vec2 offset, float radius, float scale) {

    vec2 res = coor;

    radius *= mix(0.9, 1.0, scale);

    float d = distance(coor, center);
    if (d < radius) {
        float p = abs(radius - d) / radius;
        res = coor - p * p * offset * scale;
        //        if (scale == 1.0) {
        //            res = vec2(-1.0);
        //        }
    }

    return res;
}

// Hip
vec2 rotate(vec2 src, vec2 anchor, float angle) {
    src -= vec2(anchor.x, anchor.y);
    return (src*mat2(cos(angle), -sin(angle), sin(angle), cos(angle)))+vec2(anchor.x, anchor.y);
}

float drag(float x, float centerx, float maxd, float max_offset, float scale) {
    float offset = 0.;
    float d = abs(x - centerx);
    if (d < maxd) {
        float p = (maxd - d) / maxd;
        //p *= p * p;
        p = smoothstep(0., 1., p);
        offset = -p * max_offset;
        //        if (scale == 1.0 || scale == -1.0) {
        //            offset = -x;
        //        }
    }
    return offset * scale;
}

float drag2(vec2 uv, vec2 center, float maxdx, float maxdy, float max_offset, float scale) {
    float offset = 0.;
    float dx = abs(uv.x - center.x);
    float dy = abs(uv.y - center.y);
    if (dx < maxdx && dy < maxdy) {
        float px = (maxdx - dx) / maxdx;
        px = smoothstep(0., 1., px);
        float py = (maxdy - dy) / maxdy;
        py = smoothstep(0., 1., py);
        offset -= px * py * max_offset;
        //        if (scale == 1.0 || scale == -1.0) {
        //            offset = -dx;
        //        }
    }
    return offset * scale;
}

vec2 hip_inner(vec2 coor, vec2 center, float w, float h, float radian, float hipScaleL, float hipScaleR) {
    vec2 res = coor;

    res = rotate(res, center, -radian);
    float dx = 0.;
    dx += drag2(res, vec2(center.x + w * 0.85, center.y), w, w * 0.8, w * 0.05, hipScaleR);
    dx -= drag2(res, vec2(center.x - w * 0.85, center.y), w, w * 0.8, w * 0.05, hipScaleL);
    res.x += dx;
    res = rotate(res, center, radian);

    return res;
}

vec2 lift_inner(vec2 coor, vec2 center, float w, float h, float radian,  float liftScale) {
    vec2 res = coor;
    res.y += drag(res.y, center.y - h * 0.25, abs(h), h * 0.09, abs(liftScale));
    return res;
}

// Breast
vec2 breast_inner(vec2 uv, vec2 center, float radius, float scale) {
    vec2 res = uv;
    res -= center;
    float ratio = 0.95;
    res.y *= ratio;

    float innerScale = scale * 0.25;
    float m = radius * 0.5;
    float d = length(res);
    if (d < radius) {
        if (d < radius - m) {
            res *= 1. - innerScale;
        } else {
            float p = (radius - d) / m;
            p = smoothstep(0., 1., p);
            res *= 1. - p * innerScale;
        }
    }

    res.y /= ratio;
    res += center;
    return res - uv;
}

vec2 process(vec2 coor, float width, float height, int i) {
    vec2 center_point, coor_res;
    float target_x_ratio;
    float max_x_ratio;
    int direction = 0;
    int line_type = 1;
    float target_y_ratio = 1.0;
    float max_y_ratio = 1.0;
    float left, right, top, bottom, x_min, x_max, y_min, y_max;
    int mode = 0, transition_type = 1;

    vec2 vec_record = vec2(0., 0.);

    // Neck天鹅颈
    float n_scale = neck_scale[i];
    if (n_scale != 0.0) {
        for (int t = 0; t < 3; t++) {
            vec2 center = vec2(neck_center[i * 6 + t * 2 + 0],
            neck_center[i * 6 + t * 2 + 1]);
            vec2 offset = vec2(neck_offset[i * 6 + t * 2 + 0],
            neck_offset[i * 6 + t * 2 + 1]);
            float radius = neck_radius[i * 3 + t];

            coor_res = neck_inner(coor, center, offset, radius, n_scale);
            vec_record += coor_res - coor;
            coor = coor_res;
        }
    }

    // Hip美胯
    float hipScaleL = hip_scale_l[i];
    float hipScaleR = hip_scale_r[i];
    float liftScale = lift_scale[i];
    if (hipScaleL != 0.0 || hipScaleR != 0.0 || liftScale != 0.0) {
        vec2 center = vec2(hip_center[i * 2 + 0], hip_center[i * 2 + 1]);
        float w = hip_size_radian[i * 3 + 0];
        float h = hip_size_radian[i * 3 + 1];
        float radian = hip_size_radian[i * 3 + 2];
        coor_res = hip_inner(coor, center, w, h, radian, hipScaleL, hipScaleR);
        coor_res = lift_inner(coor_res, center, w, h, radian, liftScale);
        vec_record += coor_res - coor;
        coor = coor_res;
    }

    // Breast丰胸
    float br_scale = breast_scale[i];
    if (br_scale != 0.0) {
        vec2 dd = vec2(0.0);
        // left
        vec2 centerL = vec2(breast_left_param[i * 3 + 0],
        breast_left_param[i * 3 + 1]);
        float radiusL = breast_left_param[i * 3 + 2];
        dd += breast_inner(coor, centerL, radiusL, br_scale);

        // right
        vec2 centerR = vec2(breast_right_param[i * 3 + 0],
        breast_right_param[i * 3 + 1]);
        float radiusR = breast_right_param[i * 3 + 2];
        dd += breast_inner(coor, centerR, radiusR, br_scale);

        coor_res = coor + dd;
        vec_record += coor_res - coor;
        coor = coor_res;
    }

    return vec_record;
}

void main() {
    vec2 coor = textureCoordinate;

    float width = resolution.x;
    float height = resolution.y;

    coor = coor * resolution;

    vec2 vec_record = vec2(0.);

    vec_record = process(coor, width, height, 0);
    coor = coor + vec_record;

    vec_record = process(coor, width, height, 1);
    coor = coor + vec_record;

    vec_record = process(coor, width, height, 2);
    coor = coor + vec_record;

    vec2 coor_res = coor;
    coor_res = coor_res / resolution;

    gl_FragColor = texture2D(inputImageTexture, coor_res);
}