#define PI 3.1415926
precision highp float;
precision highp int;

varying vec2 textureCoordinate;
uniform sampler2D sTexture;

uniform vec2 resolution;

#define kMaxBodySize 3

// 缩肚子
uniform float belly_param[kMaxBodySize * 4];
uniform float belly_scale[kMaxBodySize];

// 缩肩膀
uniform float shoulder_left_param[kMaxBodySize * 8];
uniform float shoulder_right_param[kMaxBodySize * 2];
uniform float shoulder_scale[kMaxBodySize];

// 直角肩
uniform float angleshoulder_left_params[kMaxBodySize * 5];
uniform float angleshoulder_right_params[kMaxBodySize * 5];
uniform float angleshoulder_scale[kMaxBodySize];

// LegsSlim
uniform mat4 legs_param;
uniform mat4 legs_center_scale;
uniform mat4 legs_min_max;

// WaistSlim
uniform int waist_mode;// 三种瘦身模式
// mode 0
uniform mat4 waist_param;
uniform mat4 waist_center_scale;
uniform mat4 waist_min_max;
// mode 1
uniform float waist_param1[kMaxBodySize * 6];
// mode 2
uniform float waist_param2[kMaxBodySize * 9];
// mode 3
uniform float waist_param3[kMaxBodySize * 9];

// LegsStretch
uniform mat4 stretch_legs_param;
//uniform mat4 debug_param;

const float debug = 0.0;

vec3 get_line_param(vec2 point1, vec2 point2)
{
    vec3 res;
    res.x = point1.y - point2.y;
    res.y = point2.x - point1.x;
    res.z = point1.x * point2.y - point2.x * point1.y;
    return res;
}

vec2 get_cross_point(vec4 line1, vec4 line2)
{
    vec3 param1 = get_line_param(line1.xy, line1.zw);
    vec3 param2 = get_line_param(line2.xy, line2.zw);
    vec2 res = vec2(-1.0, -1.0);
    float tmp_d = param1.x * param2.y - param2.x * param1.y;
    if (tmp_d != 0.)
    {
        res.x = (param1.y * param2.z - param2.y * param1.z) / tmp_d;
        res.y = (param1.z * param2.x - param2.z * param1.x) / tmp_d;
    }
    return res;
}

// target_point在（以point和vec确定的直线）上的垂直投影的点
vec2 get_vertical_cross_point_by_vec(vec2 point, vec2 vec, vec2 target_point)
{
    vec2 res;
    vec2 other_point = point + vec;

    vec3 ABC = get_line_param(point, other_point);
    float A = ABC.x;
    float B = ABC.y;
    float C = ABC.z;
    float A2 = A * A;
    float B2 = B * B;
    float A2plusB2 = A2 + B2;

    if (A2plusB2 != 0.)
    {
        float AB = A * B;
        float AC = A * C;
        float BC = B * C;
        float inv_A2plusB2 = 1.0 / A2plusB2;
        res.x = (B2 * target_point.x - AB * target_point.y - AC) * inv_A2plusB2;
        res.y = (A2 * target_point.y - AB * target_point.x - BC) * inv_A2plusB2;
        return res;
    }
    else
    {
        return res;
    }
}

vec2 processing_inner(vec2 coor, float ori_target_x_ratio, float ori_max_x_ratio, float ori_target_y_ratio, float ori_max_y_ratio, float left, float right, float top, float bottom, float x_min, float x_max, float y_min, float y_max, vec2 center_point, vec4 cross_point_param_x, vec4 cross_point_param_y, int mode, float height, float width, int direction, int line_type, int transition_type)
{
    vec2 res = vec2(-1.0, -1.0);

    if (ori_target_x_ratio == 1.0 && ori_target_y_ratio == 1.0)
    {
        return coor;
    }
    float coor_x = coor.x, coor_y = coor.y;

    if (direction == 0)
    {
        if (coor.x <= x_min || coor.x >= x_max || coor.y <= y_min || coor.y >= y_max)
        {
            return coor;
        }
    }
    else if (direction == 1)
    {
        if (coor.y <= y_min || coor.y >= y_max)
        {
            return coor;
        }
    }
    else
    {
        if (coor.x <= x_min || coor.x >= x_max)
        {
            return coor;
        }
    }
    float target_x_ratio = ori_target_x_ratio;
    float target_y_ratio = ori_target_y_ratio;
    float max_x_ratio = ori_max_x_ratio;
    float max_y_ratio = ori_max_y_ratio;

    float top_height = top;
    float bottom_height = bottom;
    float left_width = left;
    float right_width = right;
    float x_length = -1.0;
    float y_length = -1.0;
    float normal_ratio = 1.0;

    float offset_x_ratio, offset_y_ratio, offset_x_ratio2, offset_y_ratio2;

    //    vec2 x_vec = vec2(debug_param[0][0], debug_param[0][1]);
    vec2 x_vec = vec2(1., 0.);
    vec2 x_cross_point = get_vertical_cross_point_by_vec(center_point, x_vec, coor);

    if (x_cross_point.x == -1.0)
    {
        return coor;
    }

    float offset_x = x_cross_point.x - center_point.x;

    vec2 y_vec = vec2(0.0, 1.0);
    //    vec2 y_vec = vec2(debug_param[0][2], debug_param[0][3]);
    vec2 y_cross_point = get_vertical_cross_point_by_vec(center_point, y_vec, coor);

    if (y_cross_point.x == -1.0)
    {
        return coor;
    }

    float offset_y = y_cross_point.y - center_point.y;

    vec2 mapped_vec;
    mapped_vec.x = 0.;
    mapped_vec.y = 0.;

    float tmp_width = (offset_x <= 0.) ? left_width : right_width;
    offset_x = distance(x_cross_point, center_point);
    offset_x_ratio = (tmp_width == 0.) ? 1.0 : offset_x / tmp_width;
    x_length = tmp_width;
    offset_x_ratio = offset_x_ratio > 1.0 ? 1.0 : offset_x_ratio;
    offset_x_ratio2 = offset_x_ratio * offset_x_ratio;

    float tmp_height = (offset_y <= 0.) ? top_height : bottom_height;
    offset_y = distance(y_cross_point, center_point);
    offset_y_ratio = (tmp_height == 0.) ? 1.0 : offset_y / tmp_height;
    y_length = tmp_height;
    offset_y_ratio = offset_y_ratio > 1.0 ? 1.0 : offset_y_ratio;
    offset_y_ratio2 = offset_y_ratio * offset_y_ratio;

    float tmp_max_x_ratio, tmp_max_y_ratio, x_ratio, y_ratio, length_ratio, u, v;
    vec2 uv;

    if (mode == 0 || mode == 2)
    {
        if (direction == 2)
        {
            offset_y_ratio = 0.;
            offset_y_ratio2 = 0.;
        }
        if (transition_type == 1)
        {
            tmp_max_x_ratio = target_x_ratio - (target_x_ratio - normal_ratio) * offset_y_ratio2;
        }
        else
        {
            tmp_max_x_ratio = target_x_ratio - (target_x_ratio - normal_ratio) * offset_y_ratio;
        }
        if (line_type == 1)
        {
            x_ratio = tmp_max_x_ratio - (tmp_max_x_ratio - normal_ratio) * mix(offset_x_ratio2, 1., offset_x_ratio);//
        }
        else
        {
            x_ratio = tmp_max_x_ratio - (tmp_max_x_ratio - normal_ratio) * offset_x_ratio;
        }
        x_ratio = (max_x_ratio > 1.0) ? ((x_ratio > max_x_ratio) ? max_x_ratio : x_ratio)  : ((x_ratio > max_x_ratio) ? x_ratio : max_x_ratio);
        uv = x_ratio * (coor - y_cross_point) + y_cross_point;
    }
    else
    {
        x_ratio = 1.0;
        uv = coor;
    }
    mapped_vec += uv - coor;

    if (mode == 1 || mode == 2)
    {
        if (direction == 1)
        {
            offset_x_ratio = 0.;
            offset_x_ratio2 = 0.;
        }
        if (transition_type == 1)
        {
            tmp_max_y_ratio = target_y_ratio - (target_y_ratio - normal_ratio) * offset_x_ratio2;
        }
        else
        {
            tmp_max_y_ratio = target_y_ratio - (target_y_ratio - normal_ratio) * offset_x_ratio;
        }
        if (line_type == 1)
        {
            y_ratio = tmp_max_y_ratio - (tmp_max_y_ratio - normal_ratio) * mix(offset_y_ratio2, 1., offset_y_ratio);
        }
        else
        {
            y_ratio = tmp_max_y_ratio - (tmp_max_y_ratio - normal_ratio) * offset_y_ratio;
        }
        y_ratio = (max_y_ratio > 1.0) ? ((y_ratio > max_y_ratio) ? max_y_ratio : y_ratio)  : ((y_ratio > max_y_ratio) ? y_ratio : max_y_ratio);
        uv = y_ratio * (coor - x_cross_point) + x_cross_point;
    }
    else
    {
        y_ratio = 1.0;
        uv = coor;
    }

    mapped_vec += uv - coor;

    res = mapped_vec + coor;

    return res;
}

vec2 stretch_inner(vec2 coor, float ori_target_y_ratio, float top, float bottom, vec2 center_point, float stretch_ratio)
{
    vec2 res = coor;
    float top_pos = center_point.y - top;
    if (coor.y < top_pos)
    {
        return res;
    }

    //    float bottom_pos = center_point.y + bottom;
    //    float middle = center_point.y;

    float target_y_ratio = ori_target_y_ratio;
    float top_bottom_stretch_ratio = stretch_ratio;//stretch_ratio / (stretch_ratio + 1.0);
    float total_ori_num = bottom + top;
    float total_tmp_num = total_ori_num * target_y_ratio;
    float add_num = total_tmp_num - total_ori_num;

    float top_region_ori_num = top;
    float bottom_region_ori_num = bottom;
    float accum;


    float top_region_tmp_num = add_num * top_bottom_stretch_ratio + top_region_ori_num;
    float bottom_region_tmp_num = add_num * (1.0 - top_bottom_stretch_ratio) + bottom_region_ori_num;

    float top_region_gap = 1., bottom_region_gap = 1., y_pos = 0.;
    top_region_gap = (top_region_tmp_num != 0.) ? top_region_ori_num / top_region_tmp_num : 0.;

    bottom_region_gap = (bottom_region_tmp_num != 0.) ? bottom_region_ori_num / bottom_region_tmp_num : 0.;


    float state_1 = (top_region_tmp_num + top_pos) < coor.y ? 1.0 : 0.;
    float state_2 = (top_region_tmp_num + bottom_region_tmp_num + top_pos) < coor.y ? 1.0 : 0.;
    accum = top_pos + top_region_ori_num * state_1 + bottom_region_ori_num * state_2;
    float start_region = top_pos + top_region_tmp_num * state_1 + bottom_region_tmp_num * state_2;
    float tmp_gap;

    if (state_2 == 1.0)
    {
        tmp_gap = 1.0;
    }
    else if (state_1 == 1.0)
    {
        tmp_gap = bottom_region_gap;
    }
    else
    {
        tmp_gap = top_region_gap;
    }

    y_pos = tmp_gap * (coor.y - start_region) + accum;

    res.y = y_pos;
    return res;
}


vec2 rotate(vec2 src, vec2 anchor, float angle) {
    src -= vec2(anchor.x, anchor.y);
    return (src*mat2(cos(angle), -sin(angle), sin(angle), cos(angle)))+vec2(anchor.x, anchor.y);
}

vec2 spring_scale2(vec2 uv, vec2 center, float radius, float m, float scale, float angle, float ratio) {

    uv -= center;

    uv = rotate(uv, vec2(0.), -angle);

    if (ratio <= 0.) ratio = 1.0;
    ratio = mix(1., ratio, step(0.0001, ratio));
    uv.y *= ratio;

    float d = length(uv);

    m -= (radius - m) * scale;

    if (d < radius) {
        if (d < (radius - m)) {
            uv *= 1. - scale;
        } else {
            uv *= 1. - scale * (radius - length(uv)) / m;
        }
    }

    uv.y /= ratio;

    uv = rotate(uv, vec2(0.), angle);

    uv += center;

    return uv;
}


// 直角肩
vec2 drag_round(vec2 coor, vec2 center, vec2 max_offset, float radius, float scale) {
    vec2 offset = vec2(0.0);
    float d = distance(coor, center);
    if (d < radius) {
        float p = abs(radius - d) / radius;
        offset = -p * p * max_offset * scale;
    }
    return offset;
}

float drag_box2(vec2 uv, vec2 center, float minx, float maxx, float miny, float maxy, float max_offset, float scale) {
    float offset = 0.;
    float dx = abs(uv.x - center.x);
    float dy = abs(uv.y - center.y);
    float maxdx = mix(minx, maxx, step(center.x, uv.x));
    float maxdy = mix(miny, maxy, step(center.y, uv.y));
    if (dx < maxdx && dy < maxdy) {
        float px = (maxdx - dx) / maxdx;
        px = 1. - (1. - px) * (1. - px);
        px = smoothstep(0., 1., px);
        float py = (maxdy - dy) / maxdy;
        py = 1. - (1. - py) * (1. - py);
        py = smoothstep(0., 1., py);
        offset -= px * py * max_offset;
    }
    return offset * scale;
}

vec2 process(vec2 coor, float width, float height, int i) {
    vec2 center_point, coor_res;
    float target_x_ratio;
    float max_x_ratio;
    int direction = 0;
    int line_type = 1;
    float target_y_ratio = 1.0;
    float max_y_ratio = 1.0;
    float left, right, top, bottom, x_min, x_max, y_min, y_max;
    int mode = 0, transition_type = 1;

    vec2 vec_record = vec2(0., 0.);

    //    float factor = 1.0, factor1 = 1.0, factor2 = 1.0;

    // AngleShoulder直角肩
    if (angleshoulder_scale[i] != 0.0) {
        float scale = angleshoulder_scale[i];

        vec2 centerl = vec2(angleshoulder_left_params[i * 5 + 0],
        angleshoulder_left_params[i * 5 + 1]);
        vec2 offsetl = vec2(angleshoulder_left_params[i * 5 + 2],
        angleshoulder_left_params[i * 5 + 3]);
        float radiusl = angleshoulder_left_params[i * 5 + 4];

        vec2 centerr = vec2(angleshoulder_right_params[i * 5 + 0],
        angleshoulder_right_params[i * 5 + 1]);
        vec2 offsetr = vec2(angleshoulder_right_params[i * 5 + 2],
        angleshoulder_right_params[i * 5 + 3]);
        float radiusr = angleshoulder_right_params[i * 5 + 4];

        //        float kkk = mix(0.9, 1.0, scale);
        //        radiusl *= kkk;
        //        radiusr *= kkk;

        vec2 offset = vec2(0.0);
        offset += drag_round(coor, centerl, offsetl, radiusl, scale);
        offset += drag_round(coor, centerr, offsetr, radiusr, scale);
        coor_res = coor + offset;
        vec_record += coor_res - coor;
        coor = coor_res;
    }

    // Belly缩肚子
    if (belly_scale[i] != 0.0) {
        float scale = belly_scale[i];
        vec2 center = vec2(belly_param[i * 4 + 0],
        belly_param[i * 4 + 1]);
        float radius = belly_param[i * 4 + 2];
        float in_radius = belly_param[i * 4 + 3];
        coor_res = spring_scale2(coor, center, radius, radius - in_radius, scale * -0.5, 0., 1.);
        vec_record += coor_res - coor;
        coor = coor_res;
    }

    // 缩肩膀
    if (shoulder_scale[i] != 0.0) {
        float scale = shoulder_scale[i];
        // left
        vec2 centerL = vec2(shoulder_left_param[i * 8 + 0],
        shoulder_left_param[i * 8 + 1]);
        float radiusxin = shoulder_left_param[i * 8 + 2];
        float radiusxout = shoulder_left_param[i * 8 + 3];
        float radiusyup = shoulder_left_param[i * 8 + 4];
        float radiusydowm = shoulder_left_param[i * 8 + 5];
        float offset = shoulder_left_param[i * 8 + 6];
        float radian = shoulder_left_param[i * 8 + 7];

        vec2 centerR = vec2(shoulder_right_param[i * 2 + 0],
        shoulder_right_param[i * 2 + 1]);

        coor_res = coor;
        coor_res = rotate(coor_res, centerL, -radian);
        coor_res.x += drag_box2(coor_res, centerL, radiusxout, radiusxin, radiusyup, radiusydowm, offset, scale);
        coor_res = rotate(coor_res, centerL, radian);

        coor_res = rotate(coor_res, centerR, -radian);
        coor_res.x -= drag_box2(coor_res, centerR, radiusxin, radiusxout, radiusyup, radiusydowm, offset, scale);
        coor_res = rotate(coor_res, centerR, radian);

        //        coor_res.x += drag_box2(coor_res, centerL, radiusxL, radiusxL, radiusyL * 0.5, radiusyL * 1.5, offsetL, scale);
        //        coor_res.x -= drag_box2(coor_res, centerR, radiusxL, radiusxL, radiusyL * 0.5, radiusyL * 1.5, offsetL, scale);
        //        coor_res.x += drag_box(coor_res, centerL, radiusxL, radiusyL, offsetL, scale);
        //        coor_res.x -= drag_box(coor_res, centerR, radiusxL, radiusyL, offsetL, scale);
        //        coor_res = rotate(coor_res, centerL, radian);

        vec_record += coor_res - coor;
        coor = coor_res;
    }

    // WaistSlim
    float waist_scale = waist_center_scale[i][2];
    if (waist_scale != 0.0)
    {
        if (waist_mode == 0) {
            vec2 waist_center_point = vec2(waist_center_scale[i][0], waist_center_scale[i][1]);
            float waist_left = waist_param[i][0];
            float waist_right = waist_param[i][1];
            float waist_top = waist_param[i][2];
            float waist_bottom = waist_param[i][3];
            float waist_x_min = waist_min_max[i][0];
            float waist_x_max = waist_min_max[i][1];
            float waist_y_min = waist_min_max[i][2];
            float waist_y_max = waist_min_max[i][3];

            target_x_ratio = mix(1.0, 1.8, waist_scale);// 1.3
            max_x_ratio = (1.0 - target_x_ratio) * 0.8 + target_x_ratio;
            target_y_ratio = 1.0;
            max_y_ratio = (1.0 - target_y_ratio) * 0.8 + target_y_ratio;
            center_point = waist_center_point;

            // x_vec = waist_x_vec;
            // y_vec = waist_y_vec;
            left = waist_left;
            right = waist_right;
            top = waist_top;
            bottom = waist_bottom;
            x_min = waist_x_min;
            x_max = waist_x_max;
            y_min = waist_y_min;
            y_max = waist_y_max;
            mode = 0;
            direction = 0;
            line_type = 1;

            coor_res = processing_inner(coor, target_x_ratio, max_x_ratio, target_y_ratio, max_y_ratio, left, right, top, bottom, x_min, x_max, y_min, y_max,
            center_point, vec4(0.), vec4(0.), mode, height, width, direction, line_type, transition_type);
            vec_record += coor_res - coor;
            coor = coor_res;
        }
        else if (waist_mode == 1) {
            float scale = waist_scale;

            vec2 center = vec2(waist_param1[i * 6 + 0],
            waist_param1[i * 6 + 1]);
            float radius = waist_param1[i * 6 + 2];
            float in_radius = waist_param1[i * 6 + 3];
            float radian = waist_param1[i * 6 + 4];
            float ratio = waist_param1[i * 6 + 5];

            coor_res = spring_scale2(coor, center, radius, radius - in_radius, scale * -0.16, radian, ratio);
            float dy = coor_res.y - coor.y;
            coor_res.y = coor.y + dy * 0.5;
            vec_record += coor_res - coor;
            coor = coor_res;
        }
        else if (waist_mode == 2) {
            float scale = waist_scale;

            float left_min_x = waist_param2[i * 9 + 0];
            float left_mid_x = waist_param2[i * 9 + 1];
            float left_max_x = waist_param2[i * 9 + 2];
            float right_min_x = waist_param2[i * 9 + 3];
            float right_mid_x = waist_param2[i * 9 + 4];
            float right_max_x = waist_param2[i * 9 + 5];
            float center_y = waist_param2[i * 9 + 6];
            float height = waist_param2[i * 9 + 7];
            float offset = waist_param2[i * 9 + 8];

            float dx = 0.;
            dx += drag_box2(coor, vec2(left_mid_x, center_y), left_mid_x - left_min_x, left_max_x - left_mid_x, height / 2., height * 10000., offset, scale);
            dx -= drag_box2(coor, vec2(right_mid_x, center_y), right_mid_x - right_min_x, right_max_x - right_mid_x, height / 2., height * 10000., offset, scale);
            coor_res = coor;
            coor_res.x += dx;
            vec_record += coor_res - coor;
            coor = coor_res;
        } else if (waist_mode == 3) {
            float scale = waist_scale;

            float left_min_x = waist_param3[i * 9 + 0];
            float left_mid_x = waist_param3[i * 9 + 1];
            float left_max_x = waist_param3[i * 9 + 2];
            float right_min_x = waist_param3[i * 9 + 3];
            float right_mid_x = waist_param3[i * 9 + 4];
            float right_max_x = waist_param3[i * 9 + 5];
            float center_y = waist_param3[i * 9 + 6];
            float height = waist_param3[i * 9 + 7];
            float offset = waist_param3[i * 9 + 8];

            float dx = 0.;
            dx += drag_box2(coor, vec2(left_mid_x, center_y), left_mid_x - left_min_x, left_max_x - left_mid_x, height / 2., height / 2., offset, scale);
            dx -= drag_box2(coor, vec2(right_mid_x, center_y), right_mid_x - right_min_x, right_max_x - right_mid_x, height / 2., height / 2., offset, scale);
            coor_res = coor;
            coor_res.x += dx;
            vec_record += coor_res - coor;
            coor = coor_res;
        }
    }

    // WaistSlim
    /*float waist_scale = waist_center_scale[i][2];
    if (waist_scale != 1.0)
    {
        vec2 waist_center_point = vec2(waist_center_scale[i][0], waist_center_scale[i][1]);
        float waist_left = waist_param[i][0];
        float waist_right = waist_param[i][1];
        float waist_top = waist_param[i][2];
        float waist_bottom = waist_param[i][3];
        float waist_x_min = waist_min_max[i][0];
        float waist_x_max = waist_min_max[i][1];
        float waist_y_min = waist_min_max[i][2];
        float waist_y_max = waist_min_max[i][3];

        target_x_ratio = waist_scale;// 1.3
        max_x_ratio = (1.0 - target_x_ratio) * 0.8 + target_x_ratio;
        target_y_ratio = 1.0;
        max_y_ratio = (1.0 - target_y_ratio) * 0.8 + target_y_ratio;
        center_point = waist_center_point;

        // x_vec = waist_x_vec;
        // y_vec = waist_y_vec;
        left = waist_left;
        right = waist_right;
        top = waist_top;
        bottom = waist_bottom;
        x_min = waist_x_min;
        x_max = waist_x_max;
        y_min = waist_y_min;
        y_max = waist_y_max;
        mode = 0;
        direction = 0;
        line_type = 1;

//        if (debug > 0.) {
//            float circleR = width/250.;
//            if (distance(coor, center_point) < circleR) {
//                gl_FragColor = vec4(1., float(i)/3., 0., 1.);
//                return vec2(-100.);
//            }
//            if (distance(coor, vec2(center_point.x - left, (center_point.y - top))) < circleR) {
//                gl_FragColor = vec4(1., float(i)/3., 0., 1.);
//                return vec2(-100.);
//            }
//            if (distance(coor, vec2(center_point.x + right, (center_point.y - top))) < circleR) {
//                gl_FragColor = vec4(1., float(i)/3., 0., 1.);
//                return vec2(-100.);
//            }
//            if (distance(coor, vec2(center_point.x - left, (center_point.y + bottom))) < circleR) {
//                gl_FragColor = vec4(1., float(i)/3., 0., 1.);
//                return vec2(-100.);
//            }
//            if (distance(coor, vec2(center_point.x + right, (center_point.y + bottom))) < circleR) {
//                gl_FragColor = vec4(1., float(i)/3., 0., 1.);
//                return vec2(-100.);
//            }
//        }

        coor_res = processing_inner(coor, target_x_ratio, max_x_ratio, target_y_ratio, max_y_ratio, left, right, top, bottom, x_min, x_max, y_min, y_max,
        center_point, vec4(0.), vec4(0.), mode, height, width, direction, line_type, transition_type);
        vec_record += coor_res - coor;
        coor = coor_res;
    }*/

    // LegsSlim
    float legs_scale = legs_center_scale[i][2];
    if (legs_scale != 1.0)
    {
        vec2 legs_center_point = vec2(legs_center_scale[i][0], legs_center_scale[i][1]);
        float legs_left = legs_param[i][0];
        float legs_right = legs_param[i][1];
        float legs_top = legs_param[i][2];
        float legs_bottom = legs_param[i][3];
        float legs_x_min = legs_min_max[i][0];
        float legs_x_max = legs_min_max[i][1];
        float legs_y_min = legs_min_max[i][2];
        float legs_y_max = legs_min_max[i][3];

        target_x_ratio = legs_scale;// 1.3
        max_x_ratio = (1.0 - target_x_ratio) * 0.8 + target_x_ratio;
        target_y_ratio = 1.0;
        max_y_ratio = (1.0 - target_y_ratio) * 0.8 + target_y_ratio;
        center_point = legs_center_point;

        // x_vec = legs_x_vec;
        // y_vec = legs_y_vec;
        left = legs_left;
        right = legs_right;
        top = legs_top;
        bottom = legs_bottom;
        x_min = legs_x_min;
        x_max = legs_x_max;
        y_min = legs_y_min;
        y_max = legs_y_max;
        mode = 0;
        direction = 0;
        line_type = 1;

//        if (debug > 0.) {
//            float circleR = width/250.;
//            if (distance(coor, center_point) < circleR) {
//                gl_FragColor = vec4(1., float(i)/3., 0., 1.);
//                return vec2(-100.);
//            }
//            if (distance(coor, vec2(center_point.x - left, (center_point.y - top))) < circleR) {
//                gl_FragColor = vec4(1., float(i)/3., 0., 1.);
//                return vec2(-100.);
//            }
//            if (distance(coor, vec2(center_point.x + right, (center_point.y - top))) < circleR) {
//                gl_FragColor = vec4(1., float(i)/3., 0., 1.);
//                return vec2(-100.);
//            }
//            if (distance(coor, vec2(center_point.x - left, (center_point.y + bottom))) < circleR) {
//                gl_FragColor = vec4(1., float(i)/3., 0., 1.);
//                return vec2(-100.);
//            }
//            if (distance(coor, vec2(center_point.x + right, (center_point.y + bottom))) < circleR) {
//                gl_FragColor = vec4(1., float(i)/3., 0., 1.);
//                return vec2(-100.);
//            }
//        }

        coor_res = processing_inner(coor, target_x_ratio, max_x_ratio, target_y_ratio, max_y_ratio, left, right, top, bottom, x_min, x_max, y_min, y_max,
        center_point, vec4(0.), vec4(0.), mode, height, width, direction, line_type, transition_type);
        vec_record += coor_res - coor;
        coor = coor_res;
    }

    return vec_record;
}

vec2 process2(vec2 coor, float width, float height) {
    vec2 center_point, coor_res;
    float target_x_ratio;
    float max_x_ratio;
    int direction = 0;
    int line_type = 1;
    float target_y_ratio = 1.0;
    float max_y_ratio = 1.0;
    float left, right, top, bottom, x_min, x_max, y_min, y_max;
    int mode = 0, transition_type = 1;

    vec2 vec_record = vec2(0., 0.);

    // StretchLegsSlim
    float stretch_legs_scale = stretch_legs_param[1][2];
    if (stretch_legs_scale != 1.0)
    {
        float stretch_legs_waist_bottom  = stretch_legs_param[0][2];
        float stretch_legs_bottom = stretch_legs_param[0][3];
        vec2 stretch_legs_center_point = vec2(stretch_legs_param[1][0], stretch_legs_param[1][1]);

        float stretch_ratio = -0.15;//debug_param[3][0];//1.0;
        target_y_ratio = stretch_legs_scale;// 1.16
        center_point = stretch_legs_center_point;
        top = stretch_legs_waist_bottom;
        bottom = stretch_legs_bottom;

//        if (debug > 0.) {
//            if (abs(coor.y - center_point.y) <= height/1000.) {
//                gl_FragColor = vec4(1., 0., 0., 1.);
//                return vec2(-100.);
//            }
//            if (abs(coor.y - (center_point.y - top)) <= height/1000.) {
//                gl_FragColor = vec4(1., 0.5, 0., 1.);
//                return vec2(-100.);
//            }
//            if (abs(coor.y - (center_point.y + bottom)) <= height/1000.) {
//                gl_FragColor = vec4(1., 1., 0., 1.);
//                return vec2(-100.);
//            }
//        }

        coor_res = stretch_inner(coor, target_y_ratio, top, bottom, center_point, stretch_ratio);
        vec_record += coor_res - coor;
    }

    return vec_record;
}

void main() {

    vec2 coor = textureCoordinate;
    coor.y = 1. - coor.y;

    float width = resolution.x;
    float height = resolution.y;

    coor = coor * resolution;

    vec2 vec_record = vec2(0.);

    vec_record = process(coor, width, height, 0);
//    if (vec_record.x < -99.) {
//        return;
//    }
    coor = coor + vec_record;

    vec_record = process(coor, width, height, 1);
//    if (vec_record.x < -99.) {
//        return;
//    }
    coor = coor + vec_record;

    vec_record = process(coor, width, height, 2);
//    if (vec_record.x < -99.) {
//        return;
//    }
    coor = coor + vec_record;

    vec_record = process2(coor, width, height);
//    if (vec_record.x < -99.) {
//        return;
//    }
    coor = coor + vec_record;

    vec2 coor_res = coor;
    coor_res = coor_res / resolution;

    coor_res.y = 1. - coor_res.y;

    gl_FragColor = texture2D(sTexture, coor_res);
}