precision highp float;

varying vec2 vTexCoord;
varying vec2 vPosition;

uniform float aspectRatio;
uniform float faceRadian;
uniform vec2 points[106];
uniform vec2 resolution;

int RESHAPE_TYPE_INDEX_NOSE_WIDE = 5;// 鼻子变窄
int RESHAPE_TYPE_INDEX_EYE_WIDTH = 13;//眼宽
int RESHAPE_TYPE_INDEX_NOSE_WIDTH = 17;//鼻子宽度
int RESHAPE_TYPE_INDEX_NOSE_LENGTH = 18;//鼻子长度
int RESHAPE_TYPE_INDEX_NOSE_TIP = 21;//缩鼻头
int RESHAPE_TYPE_INDEX_EYE_DROP = 22;//眼睑
int RESHAPE_TYPE_INDEX_EYE_RISE = 23;//眼尾
int RESHAPE_TYPE_INDEX_EYE_INNER = 24;//内眼角
int RESHAPE_TYPE_INDEX_EYE_OUTER = 25;//外眼角
int RESHAPE_TYPE_INDEX_EYE_PUPIL = 26;//瞳孔
int RESHAPE_TYPE_INDEX_FACE_TEMPLE = 27;//太阳穴
int RESHAPE_TYPE_INDEX_FACE_SHARP = 28;//尖下巴
int RESHAPE_TYPE_INDEX_BROW_DISTANCE = 56;//眉间距
int RESHAPE_TYPE_INDEX_NOSE_NASION = 58;    //山根
int RESHAPE_TYPE_INDEX_FACE_CHEEKBONE = 59;  //颧骨


uniform float reshapeIntensitys[60];// 左脸
uniform float reshapeIntensitys2[60];// 右脸


vec2 getPoint(int index) {
    return points[index] * resolution;
}

vec2 curveWarp(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius) {
    vec2 offset = vec2(0.0);
    vec2 result = vec2(0.0);
    vec2 direction = targetPosition - originPosition;
    float infect = distance(textureCoord, originPosition) / radius;
    infect = 1.0 - infect;
    infect = clamp(infect, 0.0, 1.0);
    offset = direction * infect;
    result = textureCoord - offset;
    return result;
}

float getdistance(vec2 point1, vec2 point2, vec2 textureCoord) {
    float k = (point1.y - point2.y) / (point1.x - point2.x);
    float b = point1.y - k * point1.x;
    float dis = abs((k * textureCoord.x + (-1.0 * textureCoord.y) + b) / sqrt(k * k + 1.0));
    return dis;
}


float distanceFrom(vec2 point1, vec2 point2) {
    if (aspectRatio > 1.0) {
        return (point1.x - point2.x) * (point1.x - point2.x) + (point1.y - point2.y) * (point1.y - point2.y) * aspectRatio * aspectRatio;
    } else {
        return (point1.x - point2.x) * (point1.x - point2.x) / aspectRatio / aspectRatio + (point1.y - point2.y) * (point1.y - point2.y);
    }
}

vec2 interactiveWarping(vec2 positionToUse, vec2 center, vec2 direction, float radius, float strength) {

    vec2 textureCoordinateToUse = vec2(positionToUse.x, ((positionToUse.y - center.y) * aspectRatio) + center.y);
    float dist = distance(center, textureCoordinateToUse);
    textureCoordinateToUse = positionToUse;

    //         highp vec2 textureCoordinateToUse = positionToUse;
    //         float dist = distance(textureCoordinateToUse, center);

    if (dist < radius) {
        float a = radius * radius - dist * dist;
        float b = pow(direction.x - center.x, 2.0) + pow(direction.y - center.y, 2.0);
        float percent = pow(a / (a + b), 2.0) * strength;
        textureCoordinateToUse = textureCoordinateToUse - percent * (direction - center);
        //             return vec2(0.0, 0.0);
        return textureCoordinateToUse;
    } else {
        return textureCoordinateToUse;
    }
}

vec2 interactiveWarping1(vec2 positionToUse, vec2 center, vec2 direction, float radius, float strength, vec2 point1, vec2 point2, vec2 point3) {

    vec2 textureCoordinateToUse = vec2(positionToUse.x, ((positionToUse.y - center.y) * aspectRatio) + center.y);
    float dist = distance(center, textureCoordinateToUse);
    textureCoordinateToUse = positionToUse;

    //         highp vec2 textureCoordinateToUse = positionToUse;
    //         float dist = distance(textureCoordinateToUse, center);

    if (dist < radius) {
        float a = radius * radius - dist * dist;
        float b = pow(direction.x - center.x, 2.0) + pow(direction.y - center.y, 2.0);
        float percent = pow(a / (a + b), 2.0) * strength;
        vec2 offset = percent * (direction - center);
        float distance1 = getdistance(point1, point2, positionToUse);
        distance1 = pow(1.0 - distance1 / distance(point3, point2), 4.0);
        textureCoordinateToUse = textureCoordinateToUse - vec2(offset.x * distance1, offset.y);
        //             return vec2(0.0, 0.0);
        return textureCoordinateToUse;
    } else {
        return textureCoordinateToUse;
    }
}

float isPointInRect(vec2 textureCoord, vec2 point1, vec2 point2, vec2 point3, vec2 point4) {
    float a, b, c, d;//分别存四个向量的计算结果;
    a = (point2.x - point1.x) * (textureCoord.y - point1.y) - (point2.y - point1.y) * (textureCoord.x - point1.x);
    b = (point3.x - point2.x) * (textureCoord.y - point2.y) - (point3.y - point2.y) * (textureCoord.x - point2.x);
    c = (point4.x - point3.x) * (textureCoord.y - point3.y) - (point4.y - point3.y) * (textureCoord.x - point3.x);
    d = (point1.x - point4.x) * (textureCoord.y - point4.y) - (point1.y - point4.y) * (textureCoord.x - point4.x);
    if ((a > 0.0 && b > 0.0 && c > 0.0 && d > 0.0) || (a < 0.0 && b < 0.0 && c < 0.0 && d < 0.0))
    {
        return 1.0;
    }
    else
    {
        return 0.0;
    }
}


//53 52 57 73
vec2 curveWarp3(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius, vec2 point1, vec2 point2, vec2 point3, vec2 point4) {
    vec2 offset = vec2(0.0);
    vec2 result = vec2(0.0);
    vec2 direction = (targetPosition - originPosition);
    direction = vec2(direction.x, direction.y * 1.2);
    float infect = distance(textureCoord, originPosition) / radius;

/*vec2 p1 = point1 * 0.7 + point2 * 0.3 + (point1 - point3) / 2.0;
    vec2 p2 = point4 - vec2((point1 - point4).x / 2.0, (point1 - point4).y / 1.8);
    vec2 p3 = point3 - (point1 - point3) / 1.2;
    vec2 p4 = point2 - vec2((point1 - point2).x * 1.2, (point1 - point2).y * 0.8);*/

    vec2 p1 = (point1 + point2) * 0.5 - (point3 - point1) * 0.2;
    vec2 p2 = vec2(point2.x + (point2.x - point1.x) * 2.2, point2.y + (point2.y - point1.y) * 1.2);
    vec2 p3 = point2 + (point3 - point1) * 1.8;
    vec2 p4 = (point3 + point4) * 0.5 + (point3 - point1);
    p4 = vec2(p4.x + (point4.x - point3.x) * 0.6, p4.y - (point3.y - point1.y) * 0.3);
    float distance2 = getdistance(p1, p2, textureCoord);
    float distance3 = getdistance(p2, p3, textureCoord);
    float distance4 = getdistance(p3, p4, textureCoord);
    float distance5 = getdistance(p1, p4, textureCoord);
    float distance6 = min(min(min(distance2, distance3), distance4), distance5);

    vec2 centerPoint = (p1 + p3 + p2 + p4) / 4.0;
    distance2 = getdistance(p1, p2, centerPoint);
    distance3 = getdistance(p2, p3, centerPoint);
    distance4 = getdistance(p3, p4, centerPoint);
    distance5 = getdistance(p1, p4, centerPoint);
    float maxDistance = min(min(min(distance2, distance3), distance4), distance5);
    float ratio = distance6 / maxDistance;
    if (isPointInRect(textureCoord, p1, p2, p3, p4) > 0.0) {
        infect = pow(ratio * 0.25, 0.8);
        //infect = 0.5;
    } else {
        infect = 0.0;
    }

    infect = clamp(infect, 0.0, 1.0);
    offset = direction * infect;
    result = textureCoord - offset;
    return result;
}

vec2 curveWarp1(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius) {
    vec2 offset = vec2(0.0);
    vec2 result = vec2(0.0);
    vec2 direction = (targetPosition - originPosition);
    direction = vec2(direction.x, direction.y * 1.6);
    float infect = distance(textureCoord, originPosition) / radius;
    if (infect < 1.0 && infect > 0.0) {
        infect = (1.0 - infect) * (1.0 - infect) * (1.0 - infect) * (1.0 - infect);
    } else {
        infect = 1.0 - infect;
    }
    infect = clamp(infect, 0.0, 1.0);
    offset = direction * infect;
    result = textureCoord - offset;
    return result;
}

vec2 curveWarp2(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius) {
    vec2 offset = vec2(0.0);
    vec2 result = vec2(0.0);
    vec2 direction = (targetPosition - originPosition);
    //direction = vec2(direction.x, direction.y * 1.6);
    float infect = distance(textureCoord, originPosition) / radius;
    if (infect < 1.0 && infect > 0.0) {
        infect = (1.0 - infect) * (1.0 - infect);
    } else {
        infect = 1.0 - infect;
    }
    infect = clamp(infect, 0.0, 1.0);
    offset = direction * infect;
    result = textureCoord - offset;
    return result;
}

vec2 curveWarp4(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius, vec2 point1, vec2 point2, vec2 point3, vec2 point4) {
    vec2 offset = vec2(0.0);
    vec2 result = vec2(0.0);
    vec2 direction = (targetPosition - originPosition);
    direction = vec2(direction.x, direction.y * 1.2);
    float infect = distance(textureCoord, originPosition) / radius;

/*vec2 p1 = points[60] * 0.7 + points[61] * 0.3 + (points[60] - points[62]) / 2.0;
     vec2 p2 = points[76] - vec2((points[60] - points[76]).x / 2.0, (points[60] - points[76]).y / 1.8);
     vec2 p3 = points[62] - (points[60] - points[62]) / 1.2;
     vec2 p4 = points[61] - (points[60] - points[61]);*/

    vec2 p1 = point1 * 0.7 + point2 * 0.3 + (point1 - point3) / 2.0;
    vec2 p2 = point4 - vec2((point1 - point4).x / 2.0, (point1 - point4).y / 1.8);
    vec2 p3 = point3 - (point1 - point3) / 1.2;
    vec2 p4 = point2 - (point1 - point2);
    float distance2 = getdistance(p1, p2, textureCoord);
    float distance3 = getdistance(p2, p3, textureCoord);
    float distance4 = getdistance(p3, p4, textureCoord);
    float distance5 = getdistance(p1, p4, textureCoord);
    float distance6 = min(min(min(distance2, distance3), distance4), distance5);

    vec2 centerPoint = (p1 + p3 + p2 + p4) / 4.0;
    distance2 = getdistance(p1, p2, centerPoint);
    distance3 = getdistance(p2, p3, centerPoint);
    distance4 = getdistance(p3, p4, centerPoint);
    distance5 = getdistance(p1, p4, centerPoint);
    float maxDistance = min(min(min(distance2, distance3), distance4), distance5);
    float ratio = distance6 / maxDistance;
    if (isPointInRect(textureCoord, p1, p2, p3, p4) > 0.0) {
        infect = pow(ratio * 0.28, 1.1);
        //infect = 0.5;
    } else {
        infect = 0.0;
    }

    infect = clamp(infect, 0.0, 1.0);
    offset = direction * infect;
    result = textureCoord - offset;
    return result;
}

float drag_box_nasion(vec2 uv, vec2 center, float left_margin, float right_margin, float top_margin, float bottom_margin, float max_offset, float scale) {
    float offset = 0.;
    float dx = abs(uv.x - center.x);
    float dy = abs(uv.y - center.y);
    float maxdx = mix(left_margin, right_margin, step(center.x, uv.x));
    float maxdy = mix(top_margin, bottom_margin, step(center.y, uv.y));
    if (dx < maxdx && dy < maxdy) {
        float px = (maxdx - dx) / maxdx;
        float py = (maxdy - dy) / maxdy;
        offset -= px * py * max_offset;
    }
    return offset * scale;
}

vec2 rotate(vec2 uv, vec2 center, float ratio, float rotateRadian) {
    uv -= center;
    uv.x *= ratio;
    uv = vec2(uv.x * cos(rotateRadian) - uv.y * sin(rotateRadian), uv.x * sin(rotateRadian) + uv.y * cos(rotateRadian));
    uv.x /= ratio;
    uv += center;
    return uv;
}

vec2 eyesRiseLeft(vec2 coordinate) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_RISE] == 0.0) {
        return coordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_RISE];
    vec2 p52 = points[52];
    p52.x = (((points[53] + points[33]) / 2.0).x + points[52].x) * 0.5;
    if (strength > 0.0) {
        highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_RISE];
        vec2 currentPoint = vec2(0.0);
        vec2 destPoint = vec2(0.0);
        float eyeDistance = distance(points[52], points[55]);
        float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_RISE] * 0.25;

        vec2 p104 = (points[52] + points[55]) / 2.0;
        float radius = mix(eyeDistance * 0.4, distance(points[0], p52) * 0.5, 0.22);
        vec2 temp = (p104 - p52);
        vec2 center = (points[64] + 2.0 * points[33]) / 3.0;
        currentPoint = p52 - temp / 2.0;
        destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp1(coordinate, currentPoint, destPoint, radius * 1.75);

    } else if (strength < 0.0) {
        vec2 currentPoint = vec2(0.0);
        vec2 destPoint = vec2(0.0);
        float eyeDistance = distance(points[52], points[55]);
        float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_RISE] * 0.08;
        float radius = mix(eyeDistance * 0.4, distance(points[0], points[52]) * 0.5, 0.22) * 1.2;
        vec2 temp = (points[53] - points[52]);
        vec2 center = (p52 + points[6] + points[9]) / 3.0 + temp * 0.2;
        currentPoint = p52 - temp * 0.7;
        destPoint = currentPoint - (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp2(coordinate, currentPoint, destPoint, radius * 1.5);
    }
    return coordinate;
}

vec2 eyesRiseRight(vec2 coordinate) {
    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_RISE] == 0.0) {
        return coordinate;
    }
    highp float strength = reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_RISE];
    vec2 p61 = points[61];
    p61.x = (((points[60] + points[42]) / 2.0).x + points[61].x) * 0.5;
    if (strength > 0.0) {
        highp float strength = reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_RISE];
        vec2 currentPoint = vec2(0.0);
        vec2 destPoint = vec2(0.0);
        float eyeDistance = distance(points[58], points[61]);
        float faceShaveScale = reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_RISE] * 0.25;
        vec2 p105 = (points[58] + points[61]) / 2.0;
        float radius = mix(eyeDistance * 0.4, distance(p61, points[32]) * 0.5, 0.22);
        vec2 temp = (p105 - p61);
        vec2 center = (points[71] + 2.0 * points[42] + points[32]) / 4.0;
        currentPoint = p61 - temp / 1.8;
        destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp1(coordinate, currentPoint, destPoint, radius * 1.75);

    } else if (strength < 0.0) {
        vec2 currentPoint = vec2(0.0);
        vec2 destPoint = vec2(0.0);
        float eyeDistance = distance(points[58], points[61]);
        float faceShaveScale = reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_RISE] * 0.08;
        float radius = mix(eyeDistance * 0.4, distance(p61, points[32]) * 0.5, 0.22) * 1.2;
        vec2 temp = (points[60] - p61);
        vec2 center = (p61 + points[26] + points[23]) / 3.0 + temp * 0.2;
        currentPoint = p61 - temp * 0.7;
        destPoint = currentPoint - (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp2(coordinate, currentPoint, destPoint, radius * 1.5);
    }
    return coordinate;
}

vec2 eyesDropLeft(vec2 coordinate) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_DROP] == 0.0) {
        return coordinate;
    }
    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_DROP] * min((0.5 / distance(points[104], points[105])) * 0.8, 1.3);
    highp float radius;
    highp float faceShaveScale;
    vec2 temp;
    vec2 center;
    vec2 currentPoint;
    vec2 destPoint = vec2(0.0);

    radius = (distance(points[65], points[104]) + distance(points[52], points[55]) + 2.0 * distance(points[72], points[73])) / 8.0;
    temp = points[72] - points[57];
    center = points[57] - temp * 5.2;
    currentPoint = points[57] - vec2(temp.x * 0.3, temp.y * 0.2);
    faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_DROP] * 0.06;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_DROP] > 0.0) {
        coordinate = curveWarp3(coordinate, currentPoint, destPoint, radius * 3.0, points[53], points[52], points[57], points[73]);
    } else {
        coordinate = curveWarp3(coordinate, currentPoint, destPoint, radius * 1.7, points[53], points[52], points[57], points[73]);
    }
    return coordinate;
}

vec2 eyesDropRight(vec2 coordinate) {
    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_DROP] == 0.0) {
        return coordinate;
    }
    highp float strength = reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_DROP] * min((0.5 / distance(points[104], points[105])) * 0.8, 1.3);
    highp float radius;
    highp float faceShaveScale;
    vec2 temp;
    vec2 center;
    vec2 currentPoint;
    vec2 destPoint = vec2(0.0);
    radius = (distance(points[70], points[105]) + distance(points[58], points[61]) + 2.0 * distance(points[75], points[76])) / 8.0;
    temp = points[75] - points[62];
    center = points[62] - temp * 5.2;
    currentPoint = points[62] - vec2(temp.x * 0.3, temp.y * 0.2);
    faceShaveScale = reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_DROP] * 0.06;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;

    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_DROP] > 0.0) {
        coordinate = curveWarp3(coordinate, currentPoint, destPoint, radius * 3.0, points[60], points[61], points[62], points[76]);
    } else {
        coordinate = curveWarp3(coordinate, currentPoint, destPoint, radius * 1.7, points[60], points[61], points[62], points[76]);
    }

    return coordinate;
}

vec2 moveto(vec2 p, float angle, float d) {
    p.x += d * sin(angle);
    p.y -= d * cos(angle);
    return p;
}

vec2 enlargeEye(vec2 currentCoordinate, vec2 circleCenter, float radius, float intensity) {
    float currentDistance = distance(currentCoordinate, circleCenter);
    float weight = currentDistance / radius;
    weight = 1.0 - intensity * (1.0 - weight * weight);
    weight = clamp(weight, 0.0, 1.0);
    currentCoordinate = circleCenter + (currentCoordinate - circleCenter) * weight;
    return currentCoordinate;
}

vec2 noseNarrowLeft(vec2 currentCoordinate, float faceLength) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_WIDE] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_WIDE] * 0.1;
    float radius = faceLength * 0.2;

    vec2 center = points[46];
    currentPoint = points[82];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    vec2 noseleft = mix(points[78], points[80], 0.5);
    vec2 noseright = mix(points[79], points[81], 0.5);
    center = (noseleft + points[80] + points[81] + noseright) / 4.0;
    currentPoint = points[80];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    center = (noseleft + points[78] + points[79] + noseright) / 4.0;
    currentPoint = noseleft;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    //    float scale = reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_WIDE];
    //    vec2 offset = coordinate - currentCoordinate;
    //    if (scale > 0.44 && (abs(offset.x) > 0.000001 || abs(offset.y) > 0.000001)) {
    //        coordinate = vec2(0.0);
    //    }

    return coordinate;
}

vec2 noseNarrowRight(vec2 currentCoordinate, float faceLength) {
    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_NOSE_WIDE] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys2[RESHAPE_TYPE_INDEX_NOSE_WIDE] * 0.1;
    float radius = faceLength * 0.2;

    vec2 center = points[46];
    currentPoint = points[83];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    vec2 noseleft = mix(points[78], points[80], 0.5);
    vec2 noseright = mix(points[79], points[81], 0.5);
    center = (noseleft + points[80] + points[81] + noseright) / 4.0;
    currentPoint = points[81];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    center = (noseleft + points[78] + points[79] + noseright) / 4.0;
    currentPoint = noseright;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    //    float scale = reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_WIDE];
    //    vec2 offset = coordinate - currentCoordinate;
    //    if (scale > 0.44 && (abs(offset.x) > 0.000001 || abs(offset.y) > 0.000001)) {
    //        coordinate = vec2(0.0);
    //    }

    return coordinate;
}

vec2 noseWidthLeft(vec2 coordinate) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_WIDTH] == 0.0) {
        return coordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_WIDTH] * 0.8;
    highp float radius;
    highp vec2 direction;


    vec2 noseright = mix(points[79], points[81], 0.5);
    //        radius = distance(points[83], noseright);
    radius = distance(points[83], noseright) * aspectRatio;
    direction = vec2(points[82].x - radius * cos(faceRadian), points[82].y - radius * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[82], direction, radius / 1.5, strength);

    return coordinate;
}

vec2 noseWidthRight(vec2 coordinate) {
    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_NOSE_WIDTH] == 0.0) {
        return coordinate;
    }

    highp float strength = reshapeIntensitys2[RESHAPE_TYPE_INDEX_NOSE_WIDTH] * 0.8;
    highp float radius;
    highp vec2 direction;


    vec2 noseright = mix(points[79], points[81], 0.5);
    //        radius = distance(points[83], noseright);
    radius = distance(points[83], noseright) * aspectRatio;

    direction = vec2(points[83].x + radius * cos(faceRadian), points[83].y + radius * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[83], direction, radius / 1.5, strength);

    return coordinate;
}

vec2 noseLength(vec2 coordinate) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_LENGTH] == 0.0) {
        return coordinate;
    }

    float strength = -reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_LENGTH] * 0.5;
    float radius;
    vec2 direction;

    radius = max(distance(points[83], points[46]), distance(points[82], points[46]));
    //        radius = max(distance(points[83], points[46]), distance(points[82], points[46])) * aspectRatio;
    direction = vec2(points[46].x - radius * sin(faceRadian) * 1.5, points[46].y + radius * cos(faceRadian) * 1.5);
    coordinate = interactiveWarping(coordinate, points[46], direction, radius, strength);

    radius = distance(points[82], points[48]);
    //        radius = distance(points[82], points[48]) * aspectRatio;
    direction = vec2(points[82].x - radius * sin(faceRadian) * 1.5, points[82].y + radius * cos(faceRadian) * 1.5);
    coordinate = interactiveWarping(coordinate, points[82], direction, radius, strength);

    radius = distance(points[50], points[83]);
    //        radius = distance(points[50], points[83]) * aspectRatio;
    direction = vec2(points[83].x - radius * sin(faceRadian) * 1.5, points[83].y + radius * cos(faceRadian) * 1.5);
    coordinate = interactiveWarping(coordinate, points[83], direction, radius, strength);

    return coordinate;
}

vec2 noseTipLeft(vec2 currentCoordinate, float faceLength) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_TIP] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_TIP] * 0.2;
    float radius = faceLength * 0.2;

    vec2 center = points[46];
    currentPoint = (points[82] + points[46] + points[46]) / 3.0;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    currentPoint = (points[80] + points[46] + points[46]) / 3.0;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    return coordinate;
}

vec2 noseTipRight(vec2 currentCoordinate, float faceLength) {
    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_NOSE_TIP] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys2[RESHAPE_TYPE_INDEX_NOSE_TIP] * 0.2;
    float radius = faceLength * 0.2;

    vec2 center = points[46];

    currentPoint = (points[83] + points[46] + points[46]) / 3.0;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    currentPoint = (points[81] + points[46] + points[46]) / 3.0;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);


    return coordinate;
}

vec2 noseNasionLeft(vec2 currentCoordinate) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_NASION] == 0.0) {
        return currentCoordinate;
    }

    vec2 uv = currentCoordinate;
    vec2 center = points[43];
    float ratio = 1.0 / aspectRatio;
    uv = rotate(uv, center, ratio, -faceRadian);

    float top = getdistance(points[37], points[38], center);
    vec2 b = mix(points[44], points[45], 0.5);
    float bot = distance(b, center);
    float left = distance(points[78], points[55]) * 0.8;
    float right = getdistance(points[43], points[44], points[78]);
    float intensity = reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_NASION];
    uv.x += drag_box_nasion(uv, points[78], left, right, top, bot, right * 0.5, intensity);
    uv = rotate(uv, center, ratio, faceRadian);
    return uv;
}

vec2 noseNasionRight(vec2 currentCoordinate) {
    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_NOSE_NASION] == 0.0) {
        return currentCoordinate;
    }

    vec2 uv = currentCoordinate;
    vec2 center = points[43];
    float ratio = 1.0 / aspectRatio;
    uv = rotate(uv, center, ratio, -faceRadian);

    float top = getdistance(points[37], points[38], center);
    vec2 b = mix(points[44], points[45], 0.5);
    float bot = distance(b, center);
    float left = getdistance(points[43], points[44], points[79]);
    float right = distance(points[79], points[58]) * 0.8;
    float intensity = reshapeIntensitys2[RESHAPE_TYPE_INDEX_NOSE_NASION];
    uv.x -= drag_box_nasion(uv, points[79], left, right, top, bot, left * 0.5, intensity);

    uv = rotate(uv, center, ratio, faceRadian);
    return uv;
}

vec2 eyesWidthLeft(vec2 coordinate) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_WIDTH] == 0.0) {
        return coordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_WIDTH];
    highp float radius;
    highp vec2 direction;

    radius = distance(points[52], points[104]);
    direction = vec2(points[52].x - radius * 1.5 * cos(faceRadian), points[52].y - radius * 1.5 * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[52], direction, radius, strength);

    radius = distance(points[55], points[104]);
    direction = vec2(points[55].x + radius * 1.5 * cos(faceRadian), points[55].y + radius * 1.5 * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[55], direction, radius, strength);

    return coordinate;
}

vec2 eyesWidthRight(vec2 coordinate) {
    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_WIDTH] == 0.0) {
        return coordinate;
    }

    highp float strength = reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_WIDTH];
    highp float radius;
    highp vec2 direction;

    radius = distance(points[58], points[105]);
    direction = vec2(points[58].x - radius * 1.5 * cos(faceRadian), points[58].y - radius * 1.5 * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[58], direction, radius, strength);

    radius = distance(points[61], points[105]);
    direction = vec2(points[61].x + radius * 1.5 * cos(faceRadian), points[61].y + radius * 1.5 * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[61], direction, radius, strength);

    return coordinate;
}

float smoothnormal(float x, float k, float t) {
    float y;
    if (x <= k) {
        y = x * t / k;
    } else {
        y = (x - 1.0) * t / (k - 1.0);
    }
    return smoothstep(0.0, t, y) * t;
}

vec2 eyesPupilLeft(vec2 currentCoordinate) {
    float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_PUPIL];
    if (abs(strength) < 1e-4) {
        return currentCoordinate;
    }
    vec2 coor = currentCoordinate * resolution;
    vec2 center = getPoint(104);

    float radius = (distance(getPoint(72), getPoint(73)) * 0.15 + distance(getPoint(52), getPoint(55)) * 0.18);
    if (strength < 0.0) {
        strength *= 0.56;
        radius *= 1.4;
    } else {
        strength *= 0.8;
        radius *= 1.7;
    }

    float dist = distance(coor, center);
    if (dist < radius) {
        float ratio = smoothnormal(dist / radius, 0.5, 0.2);
        coor = mix(center, coor, 1.0 - ratio * strength);
        currentCoordinate = coor / resolution;
    }
    return currentCoordinate;
}

vec2 eyesPupilRight(vec2 currentCoordinate) {
    float strength = reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_PUPIL];
    if (abs(strength) < 1e-4) {
        return currentCoordinate;
    }
    vec2 coor = currentCoordinate * resolution;
    vec2 center = getPoint(105);

    float radius = (distance(getPoint(75), getPoint(76)) * 0.15 + distance(getPoint(58), getPoint(61)) * 0.18);
    if (strength < 0.0) {
        strength *= 0.56;
        radius *= 1.4;
    } else {
        strength *= 0.8;
        radius *= 1.7;
    }

    float dist = distance(coor, center);
    if (dist < radius) {
        float ratio = smoothnormal(dist / radius, 0.5, 0.2);
        coor = mix(center, coor, 1.0 - ratio * strength);
        currentCoordinate = coor / resolution;
    }
    return currentCoordinate;
}

vec2 eyesInnerLeft(vec2 currentCoordinate) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_INNER] == 0.5) {
        return currentCoordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_INNER];
    highp float radius;
    highp vec2 direction;

    vec2 pointLeft = points[55];
    vec2 pointRight = points[58];
    if (strength < 0.0) {
        pointLeft = vec2(points[55].x * 0.7 + points[78].x * 0.3, points[55].y * 0.7 + points[78].y * 0.3);
        pointRight = (points[58] + points[79]) / 2.0;
        radius = distance(points[52], points[55]) / 2.5;
        direction = vec2(pointLeft.x - radius * cos(faceRadian) * 1.5, pointLeft.y - radius * sin(faceRadian) * 1.5);
        currentCoordinate = interactiveWarping(currentCoordinate, pointLeft, direction, radius, -strength * 1.1);
    } else {
        radius = distance(points[52], points[55]) / 2.7;
        direction = vec2(points[55].x - radius * cos(faceRadian) * 1.5, points[55].y - radius * sin(faceRadian) * 1.5);
        currentCoordinate = interactiveWarping(currentCoordinate, pointLeft, direction, radius, -strength);
    }


    return currentCoordinate;
}

vec2 eyesInnerRight(vec2 currentCoordinate) {
    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_INNER] == 0.5) {
        return currentCoordinate;
    }

    highp float strength = reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_INNER];
    highp float radius;
    highp vec2 direction;

    vec2 pointLeft = points[55];
    vec2 pointRight = points[58];
    if (strength < 0.0) {
        pointLeft = vec2(points[55].x * 0.7 + points[78].x * 0.3, points[55].y * 0.7 + points[78].y * 0.3);
        pointRight = (points[58] + points[79]) / 2.0;
        radius = distance(points[58], points[61]) / 2.5;
        direction = vec2(pointRight.x + radius * cos(faceRadian) * 1.5, pointRight.y + radius * sin(faceRadian) * 1.5);
        currentCoordinate = interactiveWarping(currentCoordinate, pointRight, direction, radius, -strength * 1.1);
    } else {
        radius = distance(points[58], points[61]) / 2.7;
        direction = vec2(points[58].x + radius * cos(faceRadian) * 1.5, points[58].y + radius * sin(faceRadian) * 1.5);
        currentCoordinate = interactiveWarping(currentCoordinate, pointRight, direction, radius, -strength);
    }


    return currentCoordinate;
}

vec2 eyesOuterLeft(vec2 currentCoordinate) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_OUTER] == 0.5) {
        return currentCoordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_OUTER] * 0.8;
    highp float radius;
    highp vec2 direction;

    radius = distance(points[52], points[55]) / 2.8;
    direction = vec2(points[52].x - radius * cos(faceRadian) * 1.5, points[52].y - radius * sin(faceRadian) * 1.5);
    vec2 center = points[52];
    center.x = ((points[33] + points[53]) / 2.0).x;
    vec2 point1 = (points[52] + points[55]) / 2.0;
    currentCoordinate = interactiveWarping1(currentCoordinate, center, direction, radius, strength, point1, points[52], points[55]);

    return currentCoordinate;
}

vec2 eyesOuterRight(vec2 currentCoordinate) {
    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_OUTER] == 0.5) {
        return currentCoordinate;
    }

    highp float strength = reshapeIntensitys2[RESHAPE_TYPE_INDEX_EYE_OUTER] * 0.8;
    highp float radius;
    highp vec2 direction;

    radius = distance(points[58], points[61]) / 2.8;
    direction = vec2(points[61].x + radius * cos(faceRadian) * 1.5, points[61].y + radius * sin(faceRadian) * 1.5);
    vec2 point1 = (points[58] + points[61]) / 2.0;
    vec2 center = points[61];
    center.x = ((points[42] + points[60]) / 2.0).x;
    currentCoordinate = interactiveWarping1(currentCoordinate, center, direction, radius, strength, point1, points[61], points[58]);

    return currentCoordinate;
}

vec2 faceTempleLeft(vec2 currentCoordinate, float faceLength) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_TEMPLE] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_TEMPLE] * 0.2 * 1.0;
    float radius = faceLength * 0.6;

    vec2 center = vec2((points[52].x + points[0].x) / 2.0, (points[52].y + points[0].y) / 2.0);
    currentPoint = points[0];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    return coordinate;
}

vec2 faceTempleRight(vec2 currentCoordinate, float faceLength) {
    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_FACE_TEMPLE] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys2[RESHAPE_TYPE_INDEX_FACE_TEMPLE] * 0.2 * 1.0;
    float radius = faceLength * 0.6;

    vec2 center = vec2((points[61].x + points[32].x) / 2.0, (points[61].y + points[32].y) / 2.0);
    currentPoint = points[32];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    return coordinate;
}

vec2 faceSharpLeft(vec2 currentCoordinate, float faceLength) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_SHARP] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_SHARP] * 0.05 * 1.0;
    float radius = faceLength * 0.4;

    vec2 center = points[93];
    currentPoint = points[14];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    return coordinate;
}

vec2 faceSharpRight(vec2 currentCoordinate, float faceLength) {
    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_FACE_SHARP] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys2[RESHAPE_TYPE_INDEX_FACE_SHARP] * 0.05 * 1.0;
    float radius = faceLength * 0.4;

    vec2 center = points[93];
    currentPoint = points[18];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    return coordinate;
}

vec2 browDistanceLeft(vec2 currentCoordinate) {
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_BROW_DISTANCE] == 0.0) {
        return currentCoordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_BROW_DISTANCE];
    highp float radius;
    highp float radian = atan(points[66].y - points[67].y, points[66].x - points[67].x);

    highp vec2 direction;

    vec2 pointLeft = points[37];
    if (strength < 0.0) {
        radius = distance(points[33], points[37]) / 2.5;
        direction = vec2(pointLeft.x - radius * cos(radian) * 1.5, pointLeft.y - radius * sin(radian) * 1.5);
        currentCoordinate = interactiveWarping(currentCoordinate, pointLeft, direction, radius, strength);
    } else {
        radius = distance(points[33], points[37]) / 2.7;
        direction = vec2(points[37].x - radius * cos(radian) * 1.5, points[37].y - radius * sin(radian) * 1.5);
        currentCoordinate = interactiveWarping(currentCoordinate, pointLeft, direction, radius, strength * 2.0);
    }

    return currentCoordinate;
}

vec2 browDistanceRight(vec2 currentCoordinate) {
    if (reshapeIntensitys2[RESHAPE_TYPE_INDEX_BROW_DISTANCE] == 0.0) {
        return currentCoordinate;
    }

    highp float strength = reshapeIntensitys2[RESHAPE_TYPE_INDEX_BROW_DISTANCE];
    highp float radius;
    highp float radian = atan(points[69].y - points[68].y, points[69].x - points[68].x);

    highp vec2 direction;

    vec2 pointLeft = points[38];
    if (strength < 0.0) {
        radius = distance(points[42], points[38]) / 2.5;
        direction = vec2(pointLeft.x - radius * cos(radian) * 1.5, pointLeft.y - radius * sin(radian) * 1.5);
        currentCoordinate = interactiveWarping(currentCoordinate, pointLeft, direction, radius, strength);
    } else {
        radius = distance(points[42], points[38]) / 2.7;
        direction = vec2(points[38].x - radius * cos(radian) * 1.5, points[38].y - radius * sin(radian) * 1.5);
        currentCoordinate = interactiveWarping(currentCoordinate, pointLeft, direction, radius, strength * 2.0);
    }

    return currentCoordinate;
}

float cross(vec2 a, vec2 b) {
    return a.x * b.y - a.y * b.x;
}

vec2 cheekboneLeft(vec2 coordinate) {
    float intensity = reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_CHEEKBONE];
    if (abs(intensity) < 0.001) {
        return coordinate;
    }
    if (intensity > 0.) {
        intensity *= 0.7;
    } else {
        intensity *= 0.5;
    }
    vec2 p2 = points[2], p6 = points[6], p44 = points[44], p49 = points[49];

    p2.y = 1. - p2.y;
    p6.y = 1. - p6.y;
    p44.y = 1. - p44.y;
    p49.y = 1. - p49.y;
    coordinate.y = 1. - coordinate.y;

    vec2 ll = p6 - p2, lr = p49 - p44, lt = p44 - p2;
    vec2 ill = normalize(ll), ilr = normalize(lr), ilt = normalize(lt);
    float dl = cross(coordinate - p2, ill);
    float dr = -cross(coordinate - p44, ilr);
    float dt = -cross(coordinate - p2, ilt);
    float lenll = length(ll);

    float pvo = 0.6;
    float pv = dt / (lenll * (1. + pvo));
    float ph = dl / (dl + dr);

    vec2 offsety = vec2(0.);
    vec2 offsetx = vec2(0.);

    float pvv = clamp(pv, 0., 1.);
    float fv = min(2. * (1. - sin(pvv + 0.57)), 1. - cos(pvv * 1.2));
    float pvh = (0.38 - abs(clamp(ph, 0., 0.76) - 0.38)) / 0.38;
    float f1 = 0.2 * pvh * pvh;
    float fh = f1 + max(cos(1.4 * (pvh - 1.)) - 0.75 - f1, 0.);
    offsety = fv * fh * vec2(ilt.y, -ilt.x) * lenll * 3.5;


    float m = clamp(ph, 0., 1.) * 0.6;
    float phh = ph - 0.1;
    float h = 1. - (phh > 0. ? phh * 1.4 : phh * (-2.7));
    if (h > 0.) {
        float p = clamp((pv + 0.3 - m) / (1.3 - m), 0., 1.);
        float f1 = clamp(pow(p + 0.18, 0.065) - 0.9, 0.,1.);
        float f2 = (0.91 - pow(p, 0.14));
        float f3 = min(pow(p, 0.09) - 0.998, 0.);
        float d = min(f1, max(max(f2, f3), -0.031));

        offsetx = d * h * vec2(-ill.y, ill.x) * lenll * 2.86;
    }

    vec2 res = coordinate - (offsety + offsetx) * intensity;
    res.y = 1. - res.y;
    return res;
}

vec2 cheekboneRight(vec2 coordinate) {
    float intensity = reshapeIntensitys2[RESHAPE_TYPE_INDEX_FACE_CHEEKBONE];
    if (abs(intensity) < 0.001) {
        return coordinate;
    }
    if (intensity > 0.) {
        intensity *= 0.7;
    } else {
        intensity *= 0.5;
    }
    vec2 p2 = points[30], p6 =  points[26], p44 = points[44], p49 = points[49];


    p2.y = 1. - p2.y;
    p6.y = 1. - p6.y;
    p44.y = 1. - p44.y;
    p49.y = 1. - p49.y;
    coordinate.y = 1. - coordinate.y;

    vec2 ll = p6 - p2, lr = p49 - p44, lt = p44 - p2;
    vec2 ill = normalize(ll), ilr = normalize(lr), ilt = normalize(lt);
    float dl = cross(coordinate - p2, ill);
    float dr = -cross(coordinate - p44, ilr);
    float dt = cross(coordinate - p2, ilt);
    float lenll = length(ll);

    float pvo = 0.6;
    float pv = dt / (lenll * (1. + pvo));
    float ph = dl / (dl + dr);

    vec2 offsety = vec2(0.);
    vec2 offsetx = vec2(0.);
    if (true) {
        float pvv = clamp(pv, 0., 1.);
        float fv = min(2. * (1. - sin(pvv + 0.57)), 1. - cos(pvv * 1.2));
        float pvh = (0.38 - abs(clamp(ph, 0., 0.76) - 0.38)) / 0.38;
        float f1 = 0.2 * pvh * pvh;
        float fh = f1 + max(cos(1.4 * (pvh - 1.)) - 0.75 - f1, 0.);
        offsety = fv * fh * vec2(-ilt.y, ilt.x) * lenll * 3.5;
    }
    if (true) {
        float m = clamp(ph, 0., 1.) * 0.6;
        float phh = ph - 0.1;
        float h = 1. - (phh > 0. ? phh * 1.4 : phh * (-2.7));
        if (h > 0.) {
            float p = clamp((pv + 0.3 - m) / (1.3 - m), 0., 1.);
            float f1 = clamp(pow(p + 0.18, 0.065) - 0.9, 0.,1.);
            float f2 = (0.91 - pow(p, 0.14));
            float f3 = min(pow(p, 0.09) - 0.998, 0.);
            float d = min(f1, max(max(f2, f3), -0.031));

            offsetx = d * h * vec2(ill.y, -ill.x) * lenll * 2.86;
        }
    }
    vec2 res = coordinate - (offsety + offsetx) * intensity;
    res.y = 1. - res.y;
    return res;
}


#ifdef ENABLE_FLOAT_TEXTURE
vec4 offset2data(vec2 uv) {
    return vec4(uv, 0.0, 0.0);
}
#else
vec4 uv2data(vec2 uv) {
    vec2 xy = uv * 255.;
    vec2 fxy = fract(xy) * 255.;
    xy = floor(xy) / 255.;
    fxy = floor(fxy + 0.5) / 255.;
    return vec4(xy, fxy);
}
vec4 offset2data(vec2 offset) {
    return uv2data(offset / 2.0 + vec2(0.5));
}
#endif

void main() {
    vec2 coordinate = vTexCoord;

    float eyeDistance = distance(points[104], points[105]);
    if (aspectRatio < 1.0) {
        eyeDistance = eyeDistance / aspectRatio;
    }
    coordinate = cheekboneLeft(coordinate);
    coordinate = cheekboneRight(coordinate);
    coordinate = noseNarrowLeft(coordinate, eyeDistance);
    coordinate = noseNarrowRight(coordinate, eyeDistance);
    coordinate = noseWidthLeft(coordinate);
    coordinate = noseWidthRight(coordinate);
    coordinate = noseLength(coordinate);
    coordinate = noseTipLeft(coordinate, eyeDistance);
    coordinate = noseTipRight(coordinate, eyeDistance);
    coordinate = noseNasionLeft(coordinate);
    coordinate = noseNasionRight(coordinate);

    coordinate = eyesRiseLeft(coordinate);
    coordinate = eyesRiseRight(coordinate);
    coordinate = eyesDropLeft(coordinate);
    coordinate = eyesDropRight(coordinate);
    coordinate = eyesWidthLeft(coordinate);
    coordinate = eyesWidthRight(coordinate);
    coordinate = eyesInnerLeft(coordinate);
    coordinate = eyesInnerRight(coordinate);
    coordinate = eyesOuterLeft(coordinate);
    coordinate = eyesOuterRight(coordinate);
    coordinate = eyesPupilLeft(coordinate);
    coordinate = eyesPupilRight(coordinate);

    coordinate = faceTempleLeft(coordinate, eyeDistance);
    coordinate = faceTempleRight(coordinate, eyeDistance);
    coordinate = faceSharpLeft(coordinate, eyeDistance);
    coordinate = faceSharpRight(coordinate, eyeDistance);
    coordinate = browDistanceLeft(coordinate);
    coordinate = browDistanceRight(coordinate);

    gl_FragColor = offset2data(coordinate - vPosition);
}
