precision highp float;

varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform lowp int enable;
uniform float aspectRatio;
uniform float faceRadian;
uniform vec2 points[72];

int RESHAPE_TYPE_INDEX_FACE_SLIM = 3;// 脸型
int RESHAPE_TYPE_INDEX_EYE_ENLARGE = 4;// 大眼
int RESHAPE_TYPE_INDEX_NOSE_WIDE  = 5;// 鼻子变窄
int RESHAPE_TYPE_INDEX_MOUTH_ENLARGE = 6;//唇形
int RESHAPE_TYPE_INDEX_FACE_NARROW = 7;//小脸
int RESHAPE_TYPE_INDEX_FACE_SHAVE = 8;//窄脸
int RESHAPE_TYPE_INDEX_FACE_CHEEKBONE  = 9;//瘦颧骨
int RESHAPE_TYPE_INDEX_FACE_JAWBONE  = 10;//瘦下额骨
int RESHAPE_TYPE_INDEX_FACE_CHIN  = 11;//下巴
int RESHAPE_TYPE_INDEX_EYE_DISTANCE = 12;//眼距
int RESHAPE_TYPE_INDEX_EYE_WIDTH = 13;//眼宽
int RESHAPE_TYPE_INDEX_EYE_HEIGHT = 14;//眼高
int RESHAPE_TYPE_INDEX_LIPS_WIDTH= 15;//唇宽
int RESHAPE_TYPE_INDEX_NOSE_SIZE = 16;//鼻子大小
int RESHAPE_TYPE_INDEX_NOSE_WIDTH = 17;//鼻子宽度
int RESHAPE_TYPE_INDEX_NOSE_LENGTH = 18;//鼻子长度
int RESHAPE_TYPE_INDEX_MOUTH_LOWER = 19;//丰下唇
int RESHAPE_TYPE_INDEX_MOUTH_UPPER = 20;//丰上唇
int RESHAPE_TYPE_INDEX_NOSE_TIP = 21;//缩鼻头
int RESHAPE_TYPE_INDEX_EYE_DROP = 22;//眼睑
int RESHAPE_TYPE_INDEX_EYE_RISE = 23;//眼尾
int RESHAPE_TYPE_INDEX_EYE_INNER = 24;//内眼角
int RESHAPE_TYPE_INDEX_EYE_OUTER = 25;//外眼角
int RESHAPE_TYPE_INDEX_EYE_PUPIL = 26;//瞳孔

uniform float reshapeIntensitys[27];

vec2 curveWarp(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius){
    vec2 offset = vec2(0.0);
    vec2 result = vec2(0.0);
    vec2 direction = targetPosition - originPosition;
    float infect = distance(textureCoord, originPosition) / radius;
    infect = 1.0 - infect;
    infect = clamp(infect, 0.0, 1.0);
    offset = direction * infect;
    result = textureCoord - offset;
    return result;
}

float getdistance(vec2 point1, vec2 point2, vec2 textureCoord) {
    highp float k = (point1.y - point2.y) / (point1.x - point2.x);
    highp float b = point1.y - k * point1.x;
    float dis = abs((k * textureCoord.x + (-1.0 * textureCoord.y) + b) / sqrt(k * k + 1.0));
    return dis;
}


float distanceFrom(vec2 point1, vec2 point2){
    if (aspectRatio > 1.0) {
        return (point1.x - point2.x) * (point1.x - point2.x) + (point1.y - point2.y) * (point1.y - point2.y) * aspectRatio * aspectRatio;
    } else {
        return (point1.x - point2.x) * (point1.x - point2.x) / aspectRatio / aspectRatio + (point1.y - point2.y) * (point1.y - point2.y);
    }
}

highp vec2 interactiveWarping(highp vec2 positionToUse, highp vec2 center, highp vec2 direction, highp float radius, highp float strength){

    highp vec2 textureCoordinateToUse = vec2(positionToUse.x, ((positionToUse.y - center.y) * aspectRatio) + center.y);
    highp float dist = distance(center, textureCoordinateToUse);
    textureCoordinateToUse = positionToUse;

    //         highp vec2 textureCoordinateToUse = positionToUse;
    //         highp float dist = distance(textureCoordinateToUse, center);

    if (dist < radius){
        highp float a = radius * radius - dist * dist;
        highp float b = pow(direction.x - center.x, 2.0) + pow(direction.y - center.y, 2.0);
        highp float percent = pow(a / (a + b), 2.0) * strength;
        textureCoordinateToUse = textureCoordinateToUse - percent * (direction - center);
        //             return vec2(0.0, 0.0);
        return textureCoordinateToUse;
    } else {
        return textureCoordinateToUse;
    }
}

highp vec2 interactiveWarping1(highp vec2 positionToUse, highp vec2 center, highp vec2 direction, highp float radius, highp float strength, int point1, int point2, int point3){

    highp vec2 textureCoordinateToUse = vec2(positionToUse.x, ((positionToUse.y - center.y) * aspectRatio) + center.y);
    highp float dist = distance(center, textureCoordinateToUse);
    textureCoordinateToUse = positionToUse;

    //         highp vec2 textureCoordinateToUse = positionToUse;
    //         highp float dist = distance(textureCoordinateToUse, center);

    if (dist < radius){
        highp float a = radius * radius - dist * dist;
        highp float b = pow(direction.x - center.x, 2.0) + pow(direction.y - center.y, 2.0);
        highp float percent = pow(a / (a + b), 2.0) * strength;
        vec2 offset = percent * (direction - center);
        float distance1 = getdistance(points[point1], points[point2], positionToUse);
        distance1 = pow(1.0 - distance1 / distance(points[point3], points[point2]), 4.0);
        textureCoordinateToUse = textureCoordinateToUse - vec2(offset.x * distance1, offset.y);
        //             return vec2(0.0, 0.0);
        return textureCoordinateToUse;
    } else {
        return textureCoordinateToUse;
    }
}

float isPointInRect(vec2 textureCoord, vec2 point1, vec2 point2, vec2 point3, vec2 point4) {
    float a, b, c, d;//分别存四个向量的计算结果;
    a = (point2.x - point1.x)*(textureCoord.y - point1.y) - (point2.y - point1.y)*(textureCoord.x - point1.x);
    b = (point3.x - point2.x)*(textureCoord.y - point2.y) - (point3.y - point2.y)*(textureCoord.x - point2.x);
    c = (point4.x - point3.x)*(textureCoord.y - point3.y) - (point4.y - point3.y)*(textureCoord.x - point3.x);
    d = (point1.x - point4.x)*(textureCoord.y - point4.y) - (point1.y - point4.y)*(textureCoord.x - point4.x);
    if ((a > 0.0 && b > 0.0 && c > 0.0 && d > 0.0) || (a < 0.0 && b < 0.0 && c < 0.0 && d < 0.0))
    {
        return 1.0;
    }
    else
    {
        return 0.0;
    }
}




//14 13 20 19
vec2 curveWarp3(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius, vec2 point1, vec2 point2, vec2 point3, vec2 point4){
    vec2 offset = vec2(0.0);
    vec2 result = vec2(0.0);
    vec2 direction = (targetPosition - originPosition);
    direction = vec2(direction.x, direction.y * 1.2);
    float infect = distance(textureCoord, originPosition) / radius;

    vec2 p1 = point1 * 0.7 + point2 * 0.3 + (point1 - point3) / 2.0;
    vec2 p2 = point4 - vec2((point1 - point4).x / 2.0, (point1 - point4).y / 1.8);
    vec2 p3 = point3 - (point1 - point3) / 1.2;
    vec2 p4 = point2 - vec2((point1 - point2).x * 1.2,(point1 - point2).y * 0.8) ;
    float distance2 = getdistance(p1, p2, textureCoord);
    float distance3 = getdistance(p2, p3, textureCoord);
    float distance4 = getdistance(p3, p4, textureCoord);
    float distance5 = getdistance(p1, p4, textureCoord);
    float distance6 = min(min(min(distance2, distance3), distance4), distance5);

    vec2 centerPoint = (p1 + p3 + p2 + p4) / 4.0;
    distance2 = getdistance(p1, p2, centerPoint);
    distance3 = getdistance(p2, p3, centerPoint);
    distance4 = getdistance(p3, p4, centerPoint);
    distance5 = getdistance(p1, p4, centerPoint);
    float maxDistance = min(min(min(distance2, distance3), distance4), distance5);
    float ratio = distance6 / maxDistance;
    if (isPointInRect(textureCoord, p1, p2, p3, p4) > 0.0) {
        infect = pow(ratio * 0.25, 1.1);
        //infect = 0.5;
    } else {
        infect = 0.0;
    }

    infect = clamp(infect, 0.0, 1.0);
    offset = direction * infect;
    result = textureCoord - offset;
    return result;
}

vec2 curveWarp1(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius){
    vec2 offset = vec2(0.0);
    vec2 result = vec2(0.0);
    vec2 direction = (targetPosition - originPosition);
    direction = vec2(direction.x, direction.y * 1.6);
    float infect = distance(textureCoord, originPosition) / radius;
    if (infect < 1.0 && infect > 0.0) {
        infect = (1.0 - infect) * (1.0 - infect) * (1.0 - infect) * (1.0 - infect);
    } else {
        infect = 1.0 - infect;
    }
    infect = clamp(infect, 0.0, 1.0);
    offset = direction * infect;
    result = textureCoord - offset;
    return result;
}

vec2 curveWarp2(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius){
    vec2 offset = vec2(0.0);
    vec2 result = vec2(0.0);
    vec2 direction = (targetPosition - originPosition);
    //direction = vec2(direction.x, direction.y * 1.6);
    float infect = distance(textureCoord, originPosition) / radius;
    if (infect < 1.0 && infect > 0.0) {
        infect = (1.0 - infect) * (1.0 - infect);
    } else {
        infect = 1.0 - infect;
    }
    infect = clamp(infect, 0.0, 1.0);
    offset = direction * infect;
    result = textureCoord - offset;
    return result;
}



vec2 curveWarp4(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius, vec2 point1, vec2 point2, vec2 point3, vec2 point4){
    vec2 offset = vec2(0.0);
    vec2 result = vec2(0.0);
    vec2 direction = (targetPosition - originPosition);
    direction = vec2(direction.x, direction.y * 1.2);
    float infect = distance(textureCoord, originPosition) / radius;

    /*vec2 p1 = points[33] * 0.7 + points[34] * 0.3 + (points[33] - points[35]) / 2.0;
    vec2 p2 = points[36] - vec2((points[33] - points[36]).x / 2.0, (points[33] - points[36]).y / 1.8);
    vec2 p3 = points[35] - (points[33] - points[35]) / 1.2;
    vec2 p4 = points[34] - (points[33] - points[34]);*/

    vec2 p1 = point1 * 0.7 + point2 * 0.3 + (point1 - point3) / 2.0;
    vec2 p2 = point4 - vec2((point1 - point4).x / 2.0, (point1 - point4).y / 1.8);
    vec2 p3 = point3 - (point1 - point3) / 1.2;
    vec2 p4 = point2 - (point1 - point2);
    float distance2 = getdistance(p1, p2, textureCoord);
    float distance3 = getdistance(p2, p3, textureCoord);
    float distance4 = getdistance(p3, p4, textureCoord);
    float distance5 = getdistance(p1, p4, textureCoord);
    float distance6 = min(min(min(distance2, distance3), distance4), distance5);

    vec2 centerPoint = (p1 + p3 + p2 + p4) / 4.0;
    distance2 = getdistance(p1, p2, centerPoint);
    distance3 = getdistance(p2, p3, centerPoint);
    distance4 = getdistance(p3, p4, centerPoint);
    distance5 = getdistance(p1, p4, centerPoint);
    float maxDistance = min(min(min(distance2, distance3), distance4), distance5);
    float ratio = distance6 / maxDistance;
    if (isPointInRect(textureCoord, p1, p2, p3, p4) > 0.0) {
        infect = pow(ratio * 0.28, 1.1);
        //infect = 0.5;
    } else {
        infect = 0.0;
    }

    infect = clamp(infect, 0.0, 1.0);
    offset = direction * infect;
    result = textureCoord - offset;
    return result;
}

vec2 eyesRise(vec2 coordinate){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_RISE] == 0.0) {
        return coordinate;
    }
    /*highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_WIDTH];
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float eyeDistance = distance(points[30], points[34]);
    if(strength > 0.25) {
        float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_WIDTH] * 0.25;
        float radius = mix(eyeDistance * 0.4, distance(points[34], points[12]) * 0.5, 0.22);
        vec2 temp = (points[38] - points[34]);
        vec2 center = (points[44] + 5.0 * points[43] + points[12]) / 7.0;
        currentPoint = points[34] - temp / 1.8;
        destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

        eyeDistance = distance(points[13], points[17]);
        radius = mix(eyeDistance * 0.4, distance(points[13], points[0]) * 0.5, 0.22);
        temp = (points[21] - points[13]);
        center = (points[29] + 5.0 * points[22] + points[0]) / 7.0;
        currentPoint = points[13] - temp / 1.8;
        destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    } else if(strength < 0.25){
        float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_WIDTH] * 0.08;
        float radius = mix(eyeDistance * 0.4, distance(points[34], points[12]) * 0.5, 0.22) * 1.2;
        vec2 temp = (points[33] - points[34]);
        vec2 center = (points[32] + points[10] + points[9]) / 3.0;
        currentPoint = points[34] - temp / 2.0;
        destPoint = currentPoint - (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

        radius = mix(eyeDistance * 0.4, distance(points[13], points[0]) * 0.5, 0.22) * 1.2;
        temp = (points[14] - points[13]);
        center = (points[15] + points[2] + points[3]) / 3.0;
        currentPoint = points[13] - temp / 2.0;
        destPoint = currentPoint - (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    }*/

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_RISE];
    if (strength > 0.0) {
        highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_RISE];
        vec2 currentPoint = vec2(0.0);
        vec2 destPoint = vec2(0.0);
        float eyeDistance = distance(points[30], points[34]);
        float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_RISE] * 0.25;
        float radius = mix(eyeDistance * 0.4, distance(points[34], points[12]) * 0.5, 0.22);
        vec2 temp = (points[38] - points[34]);
        vec2 center = (points[44] + 2.0 * points[43] + points[12]) / 4.0;
        currentPoint = points[34] - temp / 1.8;
        destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp1(coordinate, currentPoint, destPoint, radius * 1.75);

        eyeDistance = distance(points[13], points[17]);
        faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_RISE] * 0.25;
        radius = mix(eyeDistance * 0.4, distance(points[0], points[13]) * 0.5, 0.22);
        temp = (points[21] - points[13]);
        center = (points[29] + 2.0 * points[22] + points[0]) / 4.0;
        currentPoint = points[13] - temp / 1.8;
        destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp1(coordinate, currentPoint, destPoint, radius * 1.75);

    } else if (strength < 0.0) {
        vec2 currentPoint = vec2(0.0);
        vec2 destPoint = vec2(0.0);
        float eyeDistance = distance(points[30], points[34]);
        float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_RISE] * 0.08;
        float radius = mix(eyeDistance * 0.4, distance(points[34], points[12]) * 0.5, 0.22) * 1.2;
        vec2 temp = (points[33] - points[34]);
        vec2 center = (points[34] + points[10] + points[9]) / 3.0 + temp * 0.2;
        currentPoint = points[34] - temp * 0.7;
        destPoint = currentPoint - (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp2(coordinate, currentPoint, destPoint, radius * 1.5);

        eyeDistance = distance(points[13], points[17]);
        faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_RISE] * 0.08;
        radius = mix(eyeDistance * 0.4, distance(points[0], points[13]) * 0.5, 0.22) * 1.2;
        temp = (points[14] - points[13]);
        center = (points[13] + points[2] + points[3]) / 3.0 + temp * 0.2;
        currentPoint = points[13] - temp * 0.7;
        destPoint = currentPoint - (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp2(coordinate, currentPoint, destPoint, radius * 1.5);
    }
    return coordinate;
}

vec2 eyesDrop(vec2 coordinate){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_DROP] == 0.0) {
        return coordinate;
    }
    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_DROP] * min((0.5 / distance(points[21], points[38])) * 0.8, 1.3);
    highp float radius;
    highp float faceShaveScale;
    vec2 temp;
    vec2 center;
    vec2 currentPoint;
    vec2 destPoint = vec2(0.0);
    radius = (distance(points[45], points[38]) + distance(points[30], points[34]) + 2.0 *  distance(points[32], points[36])) / 8.0;
    temp = points[32] - points[35];
    center = points[35] - temp * 5.2;
    currentPoint =  points[35] - vec2(temp.x * 0.3, temp.y * 0.2);
    faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_DROP] * 0.06;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;

    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_DROP] > 0.0) {
        coordinate = curveWarp4(coordinate, currentPoint, destPoint, radius * 3.0,points[33],points[34],points[35],points[36]);
    } else {
        coordinate = curveWarp4(coordinate, currentPoint, destPoint, radius * 1.7,points[33],points[34],points[35],points[36]);
    }


    radius = (distance(points[28], points[21]) + distance(points[13], points[17]) + 2.0 *  distance(points[15], points[19])) / 8.0;
    temp = points[15] - points[20];
    center = points[20] - temp * 5.2;
    currentPoint =  points[20] - vec2(temp.x * 0.3, temp.y * 0.2);
    faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_DROP] * 0.06;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_DROP] > 0.0) {
        coordinate = curveWarp3(coordinate, currentPoint, destPoint, radius * 3.0,points[14],points[13],points[20],points[19]);
    } else {
        coordinate = curveWarp4(coordinate, currentPoint, destPoint, radius * 1.7,points[14],points[13],points[20],points[19]);
    }
    return coordinate;
}

vec2 faceSlim(vec2 currentCoordinate, float faceLength){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_SLIM] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    vec2 center = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_SLIM] * 0.12 * 0.3;
    float radius = faceLength * 0.8;

    center = (points[49] + points[54]) / 2.0;

    currentPoint = points[6];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    currentPoint = points[5];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[7];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    currentPoint = points[3];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[9];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    radius = faceLength * 0.6;
    center = points[57];
    currentPoint = points[49];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[50];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[53];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[54];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    currentPoint = points[51];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[52];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    return coordinate;
}

vec2 faceCheekBone(vec2 currentCoordinate, float faceLength){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_CHEEKBONE] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_CHEEKBONE] * 0.12 * 0.2;
    float radius = faceLength * 0.5;

    vec2 center = (points[47] + points[56]) / 2.0;

    for (int i=0;i<=1;i++) {
        currentPoint = points[i];
        destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    }
    for (int i=11;i<=12;i++) {
        currentPoint = points[i];
        destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    }

    return coordinate;
}

vec2 faceNarrow(vec2 currentCoordinate, float faceLength){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_NARROW] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_NARROW] * 0.12 * 0.16 * 2.0;
    float radius = faceLength * 0.5;

    vec2 center = points[57];
    //         vec2 direction = (center - points[6]);

    for (int i=3;i<=9;i++) {
        currentPoint = points[i];
        destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    }

    return coordinate;
}

vec2 faceChin(vec2 currentCoordinate, float faceLength){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_CHIN] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_CHIN] * 0.03;
    float radius = faceLength * 0.2;
    radius = distance(points[5], points[6]);

    vec2 center = points[57];
    vec2 direction = points[57] - points[6];

    for (int i=4;i<=8;i++) {
        currentPoint = points[i];
        destPoint = currentPoint + direction * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    }

    return coordinate;
}

vec2 faceJawBone(vec2 currentCoordinate, float faceLength){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_JAWBONE] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_JAWBONE] * 0.12 * 0.16 * 2.0;
    float radius = faceLength * 0.5;
    //    float radius = distance(points[1], points[2]);

    vec2 center = points[57];

    for (int i=3;i<=4;i++) {
        currentPoint = points[i];
        destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    }
    for (int i=8;i<=9;i++) {
        currentPoint = points[i];
        destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    }

    return coordinate;
}

vec2 faceShave(vec2 currentCoordinate, float faceLength){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_SHAVE] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    vec2 center = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_FACE_SHAVE] * 0.05 * 0.5;
    float radius = faceLength * 0.3;
    //         float radius = distance(points[1], points[2]);

    center = (points[1] + points[11]) / 2.0;
    currentPoint = points[1];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[11];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    radius = faceLength * 0.5;
    center = (points[2] + points[10]) / 2.0;
    currentPoint = points[2];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[10];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    center = (points[3] + points[9]) / 2.0;
    currentPoint = points[3];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[9];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    center = (points[4] + points[8]) / 2.0;
    currentPoint = points[4];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[8];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    center = (points[5] + points[7]) / 2.0;
    currentPoint = points[5];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[7];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    return coordinate;
}

vec2 enlargeEye(vec2 currentCoordinate, vec2 circleCenter, float radius, float intensity){
    float currentDistance = distance(currentCoordinate, circleCenter);
    float weight = currentDistance / radius;
    weight = 1.0 - intensity * (1.0 - weight * weight);
    weight = clamp(weight, 0.0, 1.0);
    currentCoordinate = circleCenter + (currentCoordinate - circleCenter) * weight;
    return currentCoordinate;
}

vec2 eyeDistanceAdjust(vec2 currentCoordinate, float faceLength){
    return currentCoordinate;
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_DISTANCE] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_DISTANCE] * 0.01;
    float radius = faceLength * 0.35;

    vec2 center = (points[21] + points[38]) / 2.0;
    vec2 direction = (center - points[21]);
    for (int i=13;i<=21;i++) {
        currentPoint = points[i];
        destPoint = currentPoint + direction * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    }

    direction = (center - points[38]);
    for (int i=30;i<=38;i++) {
        currentPoint = points[i];
        destPoint = currentPoint + direction * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    }

    return coordinate;
}

vec2 noseNarrow(vec2 currentCoordinate, float faceLength){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_WIDE] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_WIDE] * 0.1;
    float radius = faceLength * 0.2;

    vec2 center = points[57];
    currentPoint = points[50];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[53];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    center = (points[48] + points[49] + points[54]+ points[55]) / 4.0;
    currentPoint = points[49];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[54];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    center = (points[48] + points[47] + points[56]+ points[55]) / 4.0;
    currentPoint = points[48];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    currentPoint = points[55];
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    return coordinate;
}

vec2 noseSize(vec2 coordinate){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_SIZE] == 0.0) {
        return coordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_SIZE] * 0.25;
    highp float radius;
    highp vec2 direction;

    radius = distance(points[53], points[55]) * aspectRatio;
    //        radius = distance(points[53], points[55]) * 1.2;
    direction = vec2(points[48].x - radius * cos(faceRadian), points[48].y - radius * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[48], direction, radius, strength);

    direction = vec2(points[50].x - radius * cos(faceRadian), points[50].y - radius * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[50], direction, radius, strength);

    direction = vec2(points[53].x + radius * cos(faceRadian), points[53].y + radius * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[53], direction, radius, strength);

    direction = vec2(points[55].x + radius * cos(faceRadian), points[55].y + radius * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[55], direction, radius, strength);

    radius = distance(points[53], points[50]) / 2.0 * aspectRatio;
    //        radius = distance(points[53], points[50]) / 2.0;
    direction = vec2(points[57].x - radius * sin(faceRadian) * 1.5, points[57].y + radius * cos(faceRadian) * 1.5);
    coordinate = interactiveWarping(coordinate, points[57], direction, radius, strength);

    return coordinate;
}

vec2 noseWidth(vec2 coordinate){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_WIDTH] == 0.0) {
        return coordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_WIDTH] * 0.8;
    highp float radius;
    highp vec2 direction;

    //        radius = distance(points[53], points[55]);
    radius = distance(points[53], points[55]) * aspectRatio;
    direction = vec2(points[50].x - radius * cos(faceRadian), points[50].y - radius * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[50], direction, radius / 1.5, strength);

    direction = vec2(points[53].x + radius * cos(faceRadian), points[53].y + radius * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[53], direction, radius / 1.5, strength);

    return coordinate;
}

vec2 noseLength(vec2 coordinate){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_LENGTH] == 0.0) {
        return coordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_LENGTH] * 0.5;
    highp float radius;
    highp vec2 direction;

    radius = max(distance(points[53], points[57]), distance(points[50], points[57]));
    //        radius = max(distance(points[53], points[57]), distance(points[50], points[57])) * aspectRatio;
    direction = vec2(points[57].x - radius * sin(faceRadian) * 1.5, points[57].y + radius * cos(faceRadian) * 1.5);
    coordinate = interactiveWarping(coordinate, points[57], direction, radius, strength);

    radius = distance(points[50], points[51]);
    //        radius = distance(points[50], points[51]) * aspectRatio;
    direction = vec2(points[50].x - radius * sin(faceRadian) * 1.5, points[50].y + radius * cos(faceRadian) * 1.5);
    coordinate = interactiveWarping(coordinate, points[50], direction, radius, strength);

    radius = distance(points[52], points[53]);
    //        radius = distance(points[52], points[53]) * aspectRatio;
    direction = vec2(points[53].x - radius * sin(faceRadian) * 1.5, points[53].y + radius * cos(faceRadian) * 1.5);
    coordinate = interactiveWarping(coordinate, points[53], direction, radius, strength);

    return coordinate;
}

vec2 noseTip(vec2 currentCoordinate, float faceLength){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_TIP] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_NOSE_TIP] * 0.2;
    float radius = faceLength * 0.2;

    vec2 center = points[57];
    currentPoint = (points[50] + points[57] + points[57]) / 3.0;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    //vec2 center = points[57];
    currentPoint =  (points[53] + points[57] + points[57]) / 3.0;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    currentPoint =  (points[49] + points[57] + points[57]) / 3.0;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);

    currentPoint =  (points[54] + points[57] + points[57]) / 3.0;
    destPoint = currentPoint + (center - currentPoint) * faceShaveScale;
    coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);


    return coordinate;
}

vec2 mouthEnlarge(vec2 currentCoordinate, float faceLength){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_MOUTH_ENLARGE] == 0.0) {
        return currentCoordinate;
    }

    vec2 coordinate = currentCoordinate;
    vec2 currentPoint = vec2(0.0);
    vec2 destPoint = vec2(0.0);
    float faceShaveScale = reshapeIntensitys[RESHAPE_TYPE_INDEX_MOUTH_ENLARGE] * 0.24 * 0.2;
    float radius = faceLength * 0.2;

    vec2 mouthCenter = (points[67] + points[70]) * 0.5;

    for (int i=58;i<=65;i++) {
        currentPoint = points[i];
        destPoint = currentPoint + (mouthCenter - currentPoint) * faceShaveScale;
        coordinate = curveWarp(coordinate, currentPoint, destPoint, radius);
    }

    return coordinate;
}

vec2 eyesWidth(vec2 coordinate){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_WIDTH] == 0.0) {
        return coordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_WIDTH];
    highp float radius;
    highp vec2 direction;

    radius = distance(points[13], points[21]);
    direction = vec2(points[13].x - radius * 1.5 * cos(faceRadian), points[13].y - radius * 1.5 * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[13], direction, radius, strength);

    radius = distance(points[17], points[21]);
    direction = vec2(points[17].x + radius * 1.5 * cos(faceRadian), points[17].y + radius * 1.5 * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[17], direction, radius, strength);

    radius = distance(points[30], points[38]);
    direction = vec2(points[30].x - radius * 1.5 * cos(faceRadian), points[30].y - radius * 1.5 * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[30], direction, radius, strength);

    radius = distance(points[34], points[38]);
    direction = vec2(points[34].x + radius * 1.5 * cos(faceRadian), points[34].y + radius * 1.5 * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[34], direction, radius, strength);

    return coordinate;
}

vec2 eyesSize(vec2 coordinate){
    //if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_ENLARGE] == 0.0) {
    //return coordinate;
    // }
    return coordinate;
    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_ENLARGE] * 0.5 * 0.75;
    highp float radius;
    highp vec2 direction;

    radius = distance(points[13], points[21]);
    direction = vec2(points[13].x - radius * 1.5 * cos(faceRadian), points[13].y - radius * 1.5 * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[13], direction, radius, strength);

    radius = distance(points[17], points[21]);
    direction = vec2(points[17].x + radius * 1.5 * cos(faceRadian), points[17].y + radius * 1.5 * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[17], direction, radius, strength);

    radius = distance(points[30], points[38]);
    direction = vec2(points[30].x - radius * 1.5 * cos(faceRadian), points[30].y - radius * 1.5 * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[30], direction, radius, strength);

    radius = distance(points[34], points[38]);
    direction = vec2(points[34].x + radius * 1.5 * cos(faceRadian), points[34].y + radius * 1.5 * sin(faceRadian));
    coordinate = interactiveWarping(coordinate, points[34], direction, radius, strength);

    radius = distance(points[14], points[15]);
    for (int i = 14; i <= 16; i++){
        direction = vec2(points[i].x + radius * 1.5 * sin(faceRadian), points[i].y - radius * 1.5 * cos(faceRadian));
        coordinate = interactiveWarping(coordinate, points[i], direction, radius, strength);
    }

    radius = distance(points[31], points[32]);
    for (int i = 31; i <= 33; i++){
        direction = vec2(points[i].x + radius * 1.5 * sin(faceRadian), points[i].y - radius * 1.5 * cos(faceRadian));
        coordinate = interactiveWarping(coordinate, points[i], direction, radius, strength);
    }

    radius = distance(points[18], points[19]);
    for (int i = 18; i <= 20; i++){
        direction = vec2(points[i].x - radius * 1.5 * sin(faceRadian), points[i].y + radius * 1.5 * cos(faceRadian));
        coordinate = interactiveWarping(coordinate, points[i], direction, radius, strength);
    }

    radius = distance(points[35], points[36]);
    for (int i = 35; i <= 37; i++){
        direction = vec2(points[i].x - radius * 1.5 * sin(faceRadian), points[i].y + radius * 1.5 * cos(faceRadian));
        coordinate = interactiveWarping(coordinate, points[i], direction, radius, strength);
    }

    return coordinate;
}

vec2 lipsWidth(vec2 coordinate){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_LIPS_WIDTH] == 0.0) {
        return coordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_LIPS_WIDTH];
    highp float radius;
    highp vec2 direction;

    radius = distance(points[58], points[62]) / 2.5;
    direction = vec2(points[58].x - radius * cos(faceRadian) * 1.5, points[58].y - radius * sin(faceRadian) * 1.5);
    coordinate = interactiveWarping(coordinate, points[58], direction, radius, strength);

    direction = vec2(points[62].x + radius * cos(faceRadian) * 1.5, points[62].y + radius * sin(faceRadian) * 1.5);
    coordinate = interactiveWarping(coordinate, points[62], direction, radius, strength);

    return coordinate;
}

vec2 eyesPupil(vec2 currentCoordinate){
    float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_PUPIL];
    if (strength == 0.5) {
        return currentCoordinate;
    }
    float degree = 0.4;
    float radius = (distance(points[32], points[36]) * 0.15 + distance(points[30], points[34]) * 0.18);
    radius = pow(radius, 1.7);
    if (strength < 0.5) {
        radius = radius * 0.75;
        strength = strength * 0.7;
    }
    vec2 circleCenter = points[38];
    strength = strength * 0.8;
    vec2 textureCoordinateToUse = vec2(currentCoordinate.x, ((currentCoordinate.y - circleCenter.y) * aspectRatio) + circleCenter.y);
    float currentDistance = distance(textureCoordinateToUse, circleCenter);

    float dist = distanceFrom(currentCoordinate, circleCenter);
    float ratio = (degree / (radius * radius)) * dist * dist - 2.0 * degree / radius * dist + degree;
    float toX;
    float toY;
    if (dist < radius) {
        toX = currentCoordinate.x + (circleCenter.x - currentCoordinate.x) * strength * ratio;
        toY = currentCoordinate.y + (circleCenter.y - currentCoordinate.y) * strength * ratio;
        currentCoordinate =  vec2(toX, toY);
    }

    strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_PUPIL];
    radius = (distance(points[15], points[19]) * 0.15 + distance(points[13], points[17]) * 0.18);
    radius = pow(radius, 1.7);
    if (strength < 0.5) {
        radius = radius * 0.75;
        strength = strength * 0.7;
    }
    circleCenter = points[21];
    strength = strength * 0.8;
    textureCoordinateToUse = vec2(currentCoordinate.x, ((currentCoordinate.y - circleCenter.y) * aspectRatio) + circleCenter.y);
    currentDistance = distance(textureCoordinateToUse, circleCenter);

    dist = distanceFrom(currentCoordinate, circleCenter);
    ratio = (degree / (radius * radius)) * dist * dist - 2.0 * degree / radius * dist + degree;
    if (dist < radius) {
        toX = currentCoordinate.x + (circleCenter.x - currentCoordinate.x) * strength * ratio;
        toY = currentCoordinate.y + (circleCenter.y - currentCoordinate.y) * strength * ratio;
        currentCoordinate = vec2(toX, toY);
    }
    return currentCoordinate;
}

vec2 eyesInner(vec2 currentCoordinate){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_INNER] == 0.5) {
        return currentCoordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_INNER];
    highp float radius;
    highp vec2 direction;

    vec2 pointLeft = points[17];
    vec2 pointRight = points[30];
    if (strength < 0.0) {
        pointLeft = vec2(points[17].x * 0.7 + points[47].x * 0.3, points[17].y * 0.7 + points[47].y * 0.3);
        pointRight = (points[30] + points[56]) / 2.0;
        radius = distance(points[13], points[17]) / 2.5;
        direction = vec2(pointLeft.x - radius * cos(faceRadian) * 1.5, pointLeft.y - radius * sin(faceRadian) * 1.5);
        currentCoordinate = interactiveWarping(currentCoordinate, pointLeft, direction, radius, -strength * 1.1);
        radius = distance(points[30], points[34]) / 2.5;
        direction = vec2(pointRight.x + radius * cos(faceRadian) * 1.5, pointRight.y + radius * sin(faceRadian) * 1.5);
        currentCoordinate = interactiveWarping(currentCoordinate, pointRight, direction, radius, -strength * 1.1);
    } else {
        radius = distance(points[13], points[17]) / 2.7;
        direction = vec2(points[17].x - radius * cos(faceRadian) * 1.5, points[17].y - radius * sin(faceRadian) * 1.5);
        currentCoordinate = interactiveWarping(currentCoordinate, pointLeft, direction, radius, -strength);
        radius = distance(points[30], points[34]) / 2.7;
        direction = vec2(points[30].x + radius * cos(faceRadian) * 1.5, points[30].y + radius * sin(faceRadian) * 1.5);
        currentCoordinate = interactiveWarping(currentCoordinate, pointRight, direction, radius, -strength);
    }


    return currentCoordinate;
}

vec2 eyesOuter(vec2 currentCoordinate){
    if (reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_OUTER] == 0.5) {
        return currentCoordinate;
    }

    highp float strength = reshapeIntensitys[RESHAPE_TYPE_INDEX_EYE_OUTER];
    highp float radius;
    highp vec2 direction;

    radius = distance(points[30], points[34]) / 2.5;
    direction = vec2(points[13].x - radius * cos(faceRadian) * 1.5, points[13].y - radius * sin(faceRadian) * 1.5);
    currentCoordinate = interactiveWarping1(currentCoordinate, points[13], direction, radius, strength, 21, 13, 17);
    vec2 temp = (points[30] - points[34]);
    direction = vec2(points[34].x + radius * cos(faceRadian) * 1.5, points[34].y + radius * sin(faceRadian) * 1.5);
    currentCoordinate = interactiveWarping1(currentCoordinate, points[34] - temp * 0.05, direction, radius, strength, 38, 34, 30);

    return currentCoordinate;
}


void main(){
    highp vec2 textureCoordinateToUse = textureCoordinate;
    if (enable == 0){
        gl_FragColor = texture2D(inputImageTexture, textureCoordinateToUse);
        return;
    } else {
        vec2 coordinate = textureCoordinateToUse;

        // 两个瞳孔的距离
        //             float eyeDistance = distance(points[21], points[38]);
        float eyeDistance = distance(points[21], points[38]);
        if (aspectRatio < 1.0) {
            eyeDistance = eyeDistance / aspectRatio;
        } else {
            //                 eyeDistance = eyeDistance * aspectRatio;
        }
        // 脸部
        coordinate = faceSlim(coordinate, eyeDistance);
        // 瘦颧骨
        coordinate = faceCheekBone(coordinate, eyeDistance);
        // 小脸
        coordinate = faceNarrow(coordinate, eyeDistance);
        // 下巴
        coordinate = faceChin(coordinate, eyeDistance);
        // 下颚骨
        coordinate = faceJawBone(coordinate, eyeDistance);
        // 窄脸
        coordinate = faceShave(coordinate, eyeDistance);


        coordinate = noseNarrow(coordinate, eyeDistance);
        coordinate = noseSize(coordinate);
        coordinate = noseWidth(coordinate);
        coordinate = noseLength(coordinate);
        coordinate = noseTip(coordinate, eyeDistance);

        coordinate = eyesSize(coordinate);
        coordinate = eyesRise(coordinate);
        coordinate = eyesDrop(coordinate);
        coordinate = eyesWidth(coordinate);
        coordinate = eyesInner(coordinate);
        coordinate = eyesOuter(coordinate);
        coordinate = eyesPupil(coordinate);
        coordinate = eyeDistanceAdjust(coordinate, eyeDistance);


        //coordinate = mouthEnlarge(coordinate, eyeDistance);
        coordinate = lipsWidth(coordinate);

        gl_FragColor = texture2D(inputImageTexture, coordinate);
    }
}