precision highp float;
precision highp int;

varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform float uBrightness;

//gimp
float rgb_max(vec3 rgb) {
    return mix(mix(rgb.r, rgb.b, step(rgb.r, rgb.b)), mix(rgb.g, rgb.b, step(rgb.g, rgb.b)), step(rgb.r, rgb.g));
}

//gimp
float rgb_min(vec3 rgb) {
    return mix(mix(rgb.r, rgb.b, step(rgb.b, rgb.r)), mix(rgb.g, rgb.b, step(rgb.b, rgb.g)), step(rgb.g, rgb.r));
}

//gimp
vec3 rgb2hsl(vec3 rgb) {
    float rgbmax = rgb_max(rgb);
    float rgbmin = rgb_min(rgb);
    vec3 hsl = vec3(0.);
    hsl.z = (rgbmax + rgbmin) / 2.;
    if (rgbmax == rgbmin) {
        hsl.y = 0.;
        hsl.x = -1.;
    } else {
        float delta = rgbmax-rgbmin;
        hsl.y = mix(delta/(2.-rgbmax-rgbmin), delta/(rgbmax+rgbmin), step(hsl.z, .5));
        if (delta == 0.) delta = 1.;
        if (rgb.r == rgbmax) hsl.x = (rgb.g-rgb.b)/delta;
        else if (rgb.g == rgbmax) hsl.x = 2. + (rgb.b-rgb.r)/delta;
        else hsl.x = 4.+(rgb.r-rgb.g)/delta;
        hsl.x /= 6.;
        hsl.x = mix(hsl.x+1., hsl.x, step(0., hsl.x));
    }
    return hsl;
}

//gimp
float hsl_value(float n1, float n2, float hue) {
    hue = mix(hue-6., mix(hue+6., hue, step(0., hue)), step(hue, 6.));
    return mix(n1 + (n2 - n1)*hue, mix(n2, mix(n1 + (n2 - n1)*(4.-hue), n1, step(4., hue)), step(3., hue)), step(1., hue));
}

//gimp
vec3 hsl2rgb(vec3 hsl) {
    vec3 rgb = vec3(0.);
    if (hsl.y == 0.) {
        rgb.r = hsl.z;
        rgb.g = hsl.z;
        rgb.b = hsl.z;
    } else {
        float m2 = mix(hsl.z + hsl.y - hsl.z*hsl.y, hsl.z*(1.+hsl.y), step(hsl.z, .5));
        float m1 = 2.*hsl.z-m2;
        rgb.r = hsl_value(m1, m2, hsl.x*6.+2.);
        rgb.g = hsl_value(m1, m2, hsl.x*6.);
        rgb.b = hsl_value(m1, m2, hsl.x*6.-2.);
    }
    return rgb;
}


vec4 adjustHSL(vec4 source, vec3 target_hsl) {
    vec3 source_hsl = rgb2hsl(source.rgb);

    vec3 result_hsl = target_hsl;

    result_hsl.z = clamp(source_hsl.z - (0.5 - target_hsl.z) * 0.5 + uBrightness * 0.3, 0.0, 1.0) * 0.9 + 0.03;

    return vec4(hsl2rgb(result_hsl), source.a);
}

vec4 normalResult(vec2 uv, vec4 source) {
    vec3 target_rgb;

    vec4 target = texture2D(inputImageTexture2, uv);
    target_rgb = target.rgb;

    vec3 target_hsl = rgb2hsl(target_rgb);
    vec4 result = adjustHSL(source, target_hsl);
    return result;
}


void main() {
    vec2 uv = textureCoordinate;
    vec4 source = texture2D(inputImageTexture, uv);

    gl_FragColor = normalResult(uv, source);
}
