precision highp float;

varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform sampler2D inputImageTexture4;
uniform sampler2D inputImageTexture5;
uniform sampler2D inputImageTexture6;

#define texCoord textureCoordinate
#define sucaiTexCoord textureCoordinate

#define sourceTexture inputImageTexture
#define lutHImageTexture inputImageTexture2
#define lutSImageTexture inputImageTexture3
#define maskImageTexture inputImageTexture4
#define maskImageTexture2 inputImageTexture5
#define maskImageTexture3 inputImageTexture6

uniform float intensity;
uniform float intensity2;
uniform float intensity3;

uniform int maskCount;

vec4 addLutColor(vec4 textureColor, sampler2D lutImageTexture) {

    highp float blueColor = textureColor.b * 63.0;

    highp vec2 quad1;
    quad1.y = floor(floor(blueColor) / 8.0);
    quad1.x = floor(blueColor) - (quad1.y * 8.0);

    highp vec2 quad2;
    quad2.y = floor(ceil(blueColor) / 8.0);
    quad2.x = ceil(blueColor) - (quad2.y * 8.0);

    highp vec2 texPos1;
    texPos1.x = (quad1.x * 0.125) + 0.5 / 512.0 + ((0.125 - 1.0 / 512.0) * textureColor.r);
    texPos1.y = (quad1.y * 0.125) + 0.5 / 512.0 + ((0.125 - 1.0 / 512.0) * textureColor.g);

    highp vec2 texPos2;
    texPos2.x = (quad2.x * 0.125) + 0.5 / 512.0 + ((0.125 - 1.0 / 512.0) * textureColor.r);
    texPos2.y = (quad2.y * 0.125) + 0.5 / 512.0 + ((0.125 - 1.0 / 512.0) * textureColor.g);

    lowp vec4 newColor1 = texture2D(lutImageTexture, texPos1);
    lowp vec4 newColor2 = texture2D(lutImageTexture, texPos2);

    lowp vec4 newColor = mix(newColor1, newColor2, fract(blueColor));

    return newColor;
}

vec4 touchup(vec4 src, vec4 mask, float intensity) {
    vec4 dst = vec4(0.0);
    //    vec4 color = src;
    //    vec4 color2 = src;
    if (mask.r > 0.001 || mask.b > 0.001){
        vec4 color1 = addLutColor(src, lutHImageTexture);
        vec4 color2 = addLutColor(src, lutSImageTexture);
        vec4 res1 = mix(src, color1, mask.r);
        vec4 result = mix(src, res1, intensity);

        vec4 res2 = mix(result, color2, mask.b);
        vec4 finalRes = mix(src, res2, intensity);
        //        result = mix(src, result, intensity);

        dst = vec4(finalRes.rgb, 1.0);
        // gl_FragColor = mask;
    } else {
        dst = vec4(src.rgb, 1.0);
    }
    return dst;
}

void main(void)
{
    vec4 src = texture2D(inputImageTexture, texCoord);
    vec4 dst = src;

    if (maskCount >= 1) {
        vec4 mask = texture2D(maskImageTexture, sucaiTexCoord);
        dst = touchup(dst, mask, intensity);
    }
    if (maskCount >= 2) {
        vec4 mask = texture2D(maskImageTexture2, sucaiTexCoord);
        dst = touchup(dst, mask, intensity2);
    }
    if (maskCount >= 3) {
        vec4 mask = texture2D(maskImageTexture3, sucaiTexCoord);
        dst = touchup(dst, mask, intensity3);
    }

    gl_FragColor = dst;
}