precision highp float;

varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;

uniform vec2 iChannelResolution;
uniform vec4 uFace;
uniform vec4 uCrop;
uniform vec4 uBound;
uniform int uHasCrop;
uniform int uCompWay;
uniform float alpha;

const highp mat3 kRGBtoYIQ = mat3(
0.298996569551942, 0.5957169229774190, 0.2114643107967030,
0.586992906987039, -0.274455296657454, -0.522594696929978,
0.114010523461019, -0.321261626319965, 0.3111303861332740);
const highp mat3 kYIQtoRGB = mat3(
1.0000, 1.0000, 1.0000,
0.9563, -0.2721, -1.1070,
0.6210, -0.6474, 1.7046);

vec3 rgb2yiq(vec3 color) {
    return kRGBtoYIQ * color;
}

vec3 yiq2rgb(vec3 color) {
    return kYIQtoRGB * color;
}

float asdg(float x, float a, float b) {
    return mix(smoothstep(0.,1.,x/b)*a, smoothstep(0.,1.,(x-b)/(1.0-b))*(1.0-a)+a, step(b, x));
}

//gimp
float rgb_max(vec3 rgb) {
    return mix(mix(rgb.r, rgb.b, step(rgb.r, rgb.b)), mix(rgb.g, rgb.b, step(rgb.g, rgb.b)), step(rgb.r, rgb.g));
}

//gimp
float rgb_min(vec3 rgb) {
    return mix(mix(rgb.r, rgb.b, step(rgb.b, rgb.r)), mix(rgb.g, rgb.b, step(rgb.b, rgb.g)), step(rgb.g, rgb.r));
}

//gimp
vec3 rgb2hsl(vec3 rgb) {
    float rgbmax = rgb_max(rgb);
    float rgbmin = rgb_min(rgb);
    vec3 hsl = vec3(0.);
    hsl.z = (rgbmax + rgbmin) / 2.;
    if (rgbmax == rgbmin) {
        hsl.y = 0.;
        hsl.x = -1.;
    } else {
        float delta = rgbmax-rgbmin;
        hsl.y = mix(delta/(2.-rgbmax-rgbmin), delta/(rgbmax+rgbmin), step(hsl.z, .5));
        if (delta == 0.) delta = 1.;
        if (rgb.r == rgbmax) hsl.x = (rgb.g-rgb.b)/delta;
        else if (rgb.g == rgbmax) hsl.x = 2. + (rgb.b-rgb.r)/delta;
        else hsl.x = 4.+(rgb.r-rgb.g)/delta;
        hsl.x /= 6.;
        hsl.x = mix(hsl.x+1., hsl.x, step(0., hsl.x));
    }
    return hsl;
}

//gimp
float hsl_value(float n1, float n2, float hue) {
    hue = mix(hue-6., mix(hue+6., hue, step(0., hue)), step(hue, 6.));
    return mix(n1 + (n2 - n1)*hue, mix(n2, mix(n1 + (n2 - n1)*(4.-hue), n1, step(4., hue)), step(3., hue)), step(1., hue));
}

//gimp
vec3 hsl2rgb(vec3 hsl) {
    vec3 rgb = vec3(0.);
    if (hsl.y == 0.) {
        rgb.r = hsl.z;
        rgb.g = hsl.z;
        rgb.b = hsl.z;
    } else {
        float m2 = mix(hsl.z + hsl.y - hsl.z*hsl.y, hsl.z*(1.+hsl.y), step(hsl.z, .5));
        float m1 = 2.*hsl.z-m2;
        rgb.r = hsl_value(m1, m2, hsl.x*6.+2.);
        rgb.g = hsl_value(m1, m2, hsl.x*6.);
        rgb.b = hsl_value(m1, m2, hsl.x*6.-2.);
    }
    return rgb;
}

vec4 adjustHSL(vec4 source, vec3 target_hsl) {
    vec3 source_hsl = rgb2hsl(source.rgb);
    source_hsl.xy = target_hsl.xy;
    source_hsl.z = source_hsl.z*0.6+0.2;
    return vec4(hsl2rgb(source_hsl), source.a);
}


vec4 blendScreen(vec4 base, vec4 overlayer) {
    vec4 whiteColor = vec4(1.0);
    return whiteColor - ((whiteColor - overlayer) * (whiteColor - base)), overlayer * base + overlayer * (1.0 - base.a) + base * (1.0 - overlayer.a);
}

vec4 blendOverlay(vec4 base, vec4 overlayer, float opacity) {
    vec4 overlay = overlayer * opacity;
    mediump float ra;
    if (2.0 * base.r < base.a) {
        ra = 2.0 * overlay.r * base.r + overlay.r * (1.0 - base.a) + base.r * (1.0 - overlay.a);
    } else {
        ra = overlay.a * base.a - 2.0 * (base.a - base.r) * (overlay.a - overlay.r) + overlay.r * (1.0 - base.a) + base.r * (1.0 - overlay.a);
    }

    mediump float ga;
    if (2.0 * base.g < base.a) {
        ga = 2.0 * overlay.g * base.g + overlay.g * (1.0 - base.a) + base.g * (1.0 - overlay.a);
    } else {
        ga = overlay.a * base.a - 2.0 * (base.a - base.g) * (overlay.a - overlay.g) + overlay.g * (1.0 - base.a) + base.g * (1.0 - overlay.a);
    }

    mediump float ba;
    if (2.0 * base.b < base.a) {
        ba = 2.0 * overlay.b * base.b + overlay.b * (1.0 - base.a) + base.b * (1.0 - overlay.a);
    } else {
        ba = overlay.a * base.a - 2.0 * (base.a - base.b) * (overlay.a - overlay.b) + overlay.b * (1.0 - base.a) + base.b * (1.0 - overlay.a);
    }

    float alpha = overlay.a + base.a - overlay.a * base.a;
    return vec4(ra, ga, ba, alpha);
}

vec2 position(vec2 uv, vec4 slot, vec4 parent) {
    return (uv*parent.zw-vec2(slot.x, parent.w-slot.y-slot.w))/slot.zw;
}

bool inside(vec2 pos) {
    return pos.x >= 0.0 && pos.x <= 1.0 && pos.y >= 0.0 && pos.y <= 1.0;
}

float getMaskPct(sampler2D maskSamp, vec2 uv) {
    float pct = 0.;
    if (uHasCrop == 1) {
        vec2 pos = position(uv, vec4(uCrop.xy, uCrop.z-uCrop.x, uCrop.w-uCrop.y), vec4(0., 0, iChannelResolution.xy));
        if (inside(pos)) {
            pct = texture2D(maskSamp, pos).r;
        }
    } else {
        pct = texture2D(maskSamp, uv).r;
    }
    return pct;
}

float gradientMask(float mask) {
    //    return gradientMask(mask, .1);
    //    float res =  1. - mask;
    //    return smoothstep(res-1.0, res+0.8, 0.);
    //    return easeOutSine(mask, 0., 0.4, 1.);//reMap(mask);
    return mask*.4;
}

vec4 softLightBlend(vec4 src, vec4 dst) {

    mediump float rr;
    if (dst.r < 0.5) {
        rr = 2.0 * src.r * dst.r + src.r * src.r - 2.0 * src.r * src.r * dst.r;
    } else {
        rr = 2.0 * sqrt(src.r) * dst.r - sqrt(src.r) + 2.0 * src.r - 2.0 * src.r * dst.r;
    }

    mediump float rg;
    if (dst.g < 0.5) {
        rg = 2.0 * src.g * dst.g + src.g * src.g - 2.0 * src.g * src.g * dst.g;
    } else {
        rg = 2.0 * sqrt(src.g) * dst.g - sqrt(src.g) + 2.0 * src.g - 2.0 * src.g * dst.g;
    }

    mediump float rb;
    if (dst.b < 0.5) {
        rb = 2.0 * src.b * dst.b + src.b * src.b - 2.0 * src.b * src.b * dst.b;
    } else {
        rb = 2.0 * sqrt(src.b) * dst.b - sqrt(src.b) + 2.0 * src.b - 2.0 * src.b * dst.b;
    }

    mediump float ra;
    if (dst.r < 0.5) {
        ra = 2.0 * src.a * dst.a + src.a * src.a - 2.0 * src.a * src.a * dst.a;
    } else {
        ra = 2.0 * sqrt(src.a) * dst.a - sqrt(src.a) + 2.0 * src.a - 2.0 * src.a * dst.a;
    }

    return vec4(rr, rg, rb, ra);
}

void main() {
    vec2 uv = textureCoordinate;

    //vec2 mapUV = position(uv, vec4(uBound.xy, uBound.z-uBound.x, uBound.w-uBound.y), vec4(0., 0, iChannelResolution.xy));
    vec2 mapUV = uv;
    vec3 target = texture2D(inputImageTexture3, mapUV).rgb;
    vec4 maskColor = texture2D(inputImageTexture3, mapUV) ;

    vec4 source = texture2D(inputImageTexture, uv);
    vec4 result = vec4(0.);
    if (uCompWay == 0) {
        vec3 target_hsl = rgb2hsl(target.rgb);
        result = adjustHSL(source, target_hsl);
    } else if (uCompWay == 1) {
        float mask = getMaskPct(inputImageTexture2, uv);
        mask = gradientMask(mask);
        result = blendScreen(source, vec4(target, 1.));
        result = mix(source, result, mask);
    } else if (uCompWay == 2) {
        //float mask = getMaskPct(inputImageTexture2, uv);
        //mask = gradientMask(mask);
        //result = blendOverlay(source, vec4(target, 1.), maskColor.a * maskColor.a * maskColor.a * maskColor.a * maskColor.a);
        //result = mix(source,result,alpha);
        /*if(maskColor.a > 0.0) {

        } else {
            result = source;
        }*/

        //result = mix(source, result, mask);

        /*result = softLightBlend(source, maskColor);
        result = mix(source, result, maskColor.a);
        result = mix(source,result,alpha);*/

        // yiq
        float mask = maskColor.a * maskColor.a * maskColor.a * maskColor.a * maskColor.a;

        vec3 actualMaskColor = maskColor.rgb / (maskColor.a + step(maskColor.a, 0.0));

        vec3 sourceyiq = rgb2yiq(source.rgb);
        vec3 targetyiq = rgb2yiq(actualMaskColor);

        sourceyiq.r = clamp(pow(asdg(pow(sourceyiq.r, 2.2), pow(targetyiq.r, 2.2), 0.5), 1.0/2.2), 0.0, 1.0);
        sourceyiq.gb = targetyiq.gb;

        result = vec4(mix(source.rgb, yiq2rgb(sourceyiq), alpha * mask), source.a);
    }

    gl_FragColor = result;
}