#define V_FLIP

precision highp float;

varying vec2 srcTexCoord;
varying vec2 maskTexCoord;

uniform sampler2D srcTexture;
uniform sampler2D maskTexture;

uniform float intensity;
uniform vec2 scaleFactor;
uniform float radian;
uniform vec2 size;

//uniform float debug_params[5];
//float param(int i) {
//    return debug_params[i - 1];
//}

vec2 rotate(vec2 src, vec2 anchor, float r) {
    src -= vec2(anchor.x, anchor.y);
    return (src*mat2(cos(r), -sin(r), sin(r), cos(r)))+vec2(anchor.x, anchor.y);
}

#ifdef ENABLE_FLOAT_TEXTURE
vec2 data2offset(vec4 data) {
    return data.xy;
}
vec4 offset2data(vec2 uv) {
    return vec4(uv, 0.0, 0.0);
}
#else
vec2 data2uv(vec4 data) {
    vec2 xy = floor(data.xy * 255. + 0.5);
    vec2 fxy = floor(data.zw * 255. + 0.5);
    vec2 uv = (xy + fxy / 255.) / 255.;
    return uv;
}
vec4 uv2data(vec2 uv) {
    vec2 xy = uv * 255.;
    vec2 fxy = fract(xy) * 255.;
    xy = floor(xy) / 255.;
    fxy = floor(fxy + 0.5) / 255.;
    return vec4(xy, fxy);
}
vec2 data2offset(vec4 data) {
    return (data2uv(data) - 0.5) * 2.0;
}
vec4 offset2data(vec2 offset) {
    return uv2data(offset / 2.0 + vec2(0.5));
}
#endif

void main() {
    vec4 iMask = texture2D(maskTexture, maskTexCoord);

    vec2 offset = (iMask.rg - 0.498039) * size;
    offset = rotate(offset, vec2(0.0), radian);
    offset *= scaleFactor;
    offset /= size;

#ifdef V_FLIP
    offset.y = -offset.y;
#endif

    //gl_FragColor = texture2D(srcTexture, srcTexCoord + offset * intensity);

    offset *= intensity;

    vec2 inOffset = data2offset(texture2D(srcTexture, srcTexCoord));
    gl_FragColor = offset2data(inOffset + offset);
}





